#!/bin/bash

. ../cfg/current_cfg.sh

#echo "ROUTER_CLIENT_PORT: $ROUTER_CLIENT_PORT"

REQUESTS=1
LOG_LEVEL=3
SMALL_TIMEOUT=150000
TTL=5000
STEP_FILE=""
REPEAT_LIMIT=1

TASK_ID="12345"
TEST_NUMBER="00"

DRCE_TEST_JSON_FILE_SET="$DATA_DIR$DRCE_DATA_DIR_EXCEEDING_TTL_SYNC/c112_localhost_drce_json_set$TEST_NUMBER.txt"

MANAGE_DIR=../manage

CHANGE_CONFIG=./config.sh
START_CLUSTER=./start_m1.sh
STOP_CLUSTER=./stop_m1.sh
STOP_REPLICAS=./stop_replicas_m1.sh


RED='\e[0;31m'
GREEN='\e[0;32m'
BLUE='\e[0;34m'
NC='\e[0m' # No Color

LENGTH=20 # default length for format output

printSpace() 
## $1 - print space length
{ 
	for i in $(seq $1)
		do 
			echo -n ' '
	done 
}

printSuccess()
{
	local length=$(($LENGTH-${#1}))	
	echo -n "$1:"
	printSpace "$length"
	echo -e "[ ${GREEN}SUCCESS${NC} ]"
}

printFail()
{
	local length=$(($LENGTH-${#1}))
	echo -n "$1:"
	printSpace "$length"
	echo -e "[  ${RED}FAIL${NC}  ]"
	exit 1
}

checkExecution()
# $1 - exit code
# $2 - message string
{
  if [ "$1" -eq "0" ]
  then
    printSuccess "$2"
  else
    printFail "$2"
  fi
}

makeStepName()
{
	STEP_FILE=$LOG_DIR$TASK_ID.$NODE_APP_LOG_PREFIX"_test_route"$TEST_NUMBER"_step"$1".json"
}

executeManageScript()
# $1 - script in manage folder
# $2 - message string
{
  curDir=`pwd`
  cd $MANAGE_DIR
  $1
  checkExecution $? "$2"
  cd $curDir
}


setConfiguration()
# $1 - configuraion name
{
  executeManageScript "$CHANGE_CONFIG $1 s" "Set configuration $1"
}

startCluster()
{
  executeManageScript "$START_CLUSTER" "Start cluster"
}

stopCluster()
{
  executeManageScript "$STOP_CLUSTER" "Stop cluster"
}

stopReplicas()
{
  executeManageScript "$STOP_REPLICAS" "Stop replicas"
}


#######################
## Main process block
#######################
# preinitialization
setConfiguration "m1"

# initialization actual port and host from config
. ../cfg/current_cfg.sh

startCluster

#echo "ROUTER_CLIENT_PORT: $ROUTER_CLIENT_PORT"
echo "Start: `date`"

a=0
while [ $a -lt $REPEAT_LIMIT ]
  do
    a=$(($a+1))

    makeStepName $a

    $BIN_DIR$DRCE_COMMAND --request=SET --host=$ROUTER --port=$ROUTER_CLIENT_PORT --n=$REQUESTS --ttl=$TTL --t=$SMALL_TIMEOUT --l=$LOG_LEVEL --json=$DRCE_TEST_JSON_FILE_SET --id=$TASK_ID --cover=0 > $STEP_FILE

    checkExecution $? "New sync task ($a)" 
    #stopReplicas
  done
echo "Started: `date`"
#echo "sleep 10"
#sleep 10
#stopCluster
#setConfiguration "n"

