#!/usr/bin/python
# coding: utf-8

"""
HCE project, Python bindings, Distributed Tasks Manager application.
date_time_type.py - cli tool for convert date from string to datetime format  

@package: dc
@file error-mask-info.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

"""
Examples of usage:
cd /home/hce/hce-node-bundle/api/python/bin

echo "2018 年 5 月 5 日 02:28 JST"|./date_time_type.py
or
./date_time_type.py --input="2018 年 5 月 5 日 02:28 JST"
or
./date_time_type.py --input="2018 年 5 月 5 日 02:28 JST" --output=./outputFile.txt
or
./date_time_type.py --input=./inputFile.txt 
or
./date_time_type.py --input=./inputFile.txt --output=./outputFile.txt
"""

import os
import sys
import logging

import ppath
from ppath import sys

from cement.core import foundation
from datetime import datetime
import datetime
import time

from app.DateTimeType import DateTimeType


class DateTimeTypeApp(object):


  # Internal class
  class AppParams():


    def __init__(self):
      self.inputFileName = None
      self.outputFileName = None


  # # Initialization
  def __init__(self):
    self.appParams = self.getAppParams()


   # # main execution
  #
  # @param - None
  # @return - None
  def run(self):
    # get input data
    inputData = self.getInputData(self.appParams.inputFileName)

    # parse
    # outputData = DateTimeType.parse(inputData, True, self.getLogger(), True)
    outputData = DateTimeType.parse(inputData, True)

    # output data
    self.setOutputData(self.appParams.outputFileName, outputData)


  # # Get app params
  #
  # @param - None
  # @return AppParams instance
  def getAppParams(self):

    app = foundation.CementApp(self.__class__.__name__)
    app.setup()
    app.add_arg('-i', '--input', action='store', metavar='input_file', help='input file with date string')
    app.add_arg('-o', '--output', action='store', metavar='output_file', help='output file with date string')
    app.run()

    appParams = self.AppParams()
    appParams.inputFileName = app.pargs.input
    appParams.outputFileName = app.pargs.output
    app.close()

    return appParams


  def getLogger(self):
    # create logger
    logger = logging.getLogger('test')
    logger.setLevel(logging.DEBUG)
  
    # create console handler and set level to debug
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
  
    # create formatter
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
  
    # add formatter to ch
    ch.setFormatter(formatter)
  
    # add ch to logger
    logger.addHandler(ch)
  
    return logger

  # # Get input data
  #
  # @param fileName - file name
  # @return input data
  def getInputData(self, fileName):
    # variable for result
    ret = None
    if isinstance(fileName, basestring):
      if os.path.isfile(fileName):
        with open(fileName) as f:
          ret = f.read()
      else:
        ret = fileName
    else:
      ret = sys.stdin.read()

    return ret


  # # Set input data
  #
  # @param fileName - file name
  # @param outputData - output data
  # @return - None
  def setOutputData(self, fileName, outputData):
    if isinstance(fileName, basestring) and fileName != "":
      f = open(fileName, 'w')
      f.write(str(outputData))
      f.close()
    else:
      sys.stdout.write(str(outputData) + '\n')
      sys.stdout.flush()


if __name__ == '__main__':
  try:
    app = DateTimeTypeApp()
    app.run()
  except Exception, err:
    sys.stderr.write(str(err) + '\n')

