#!/usr/bin/python


'''
HCE project, Python bindings, Logs Analyser tool.
Tool to get some data from stat logs.

@package: hce-bundle-tools
@author bgv bgv.hce@gmail.com
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''


from optparse import OptionParser
import datetime
import json
import ppath
from ppath import sys


parser = OptionParser()
parser.add_option("--tf", "--time_format", type="string", help="A time format, by default is `{0:08d}`", dest="time_format")
parser.add_option("-u", "--time_units", type="string", help="A time unit: ms - millisecond, s - secs, m - min, h - hour, d - day, by default is `s`",
                  dest="time_units")
parser.add_option("--fd", "--fields_delimiter", type="string", help="A fields delimiter, by default is ` - `", dest="fields_delimiter")
parser.add_option("--tsf", "--timestamp_format", type="string", help="A log timestamp format, by default is `%Y-%m-%d %H:%M:%S,%f` for a Python logs",
                  dest="timestamp_format")
parser.add_option("--tsi", "--timestamp_index", type="string", help="A log timestamp field index in the fields list, by default is `0`",
                  dest="timestamp_index")
parser.add_option("-d", "--debug", type="string", help="A debug mode, by default: 0 - Off; 1 - info, 2 - debug, 3 - error, ", dest="debug")
parser.add_option("-s", "--skip", type="string", help="Skip lines without a timestamp, by default: 1 - skip, 0 - include", dest="skip")


def getTimeStampDelta(timeStampStr1, timeStampStr2, timeStampFormat, units, debug=0):
  try:
    v = 0

    d1 = datetime.datetime.strptime(timeStampStr1, timeStampFormat)
    d2 = datetime.datetime.strptime(timeStampStr2, timeStampFormat)
    d = d2 - d1

    if (debugMode & 2) > 0:
      logDebugInfo(u'Input: d1:`' + timeStampStr1 + u'`, d2:`' + timeStampStr2 + u'`, format:' + timeStampFormat + \
                   u', units:' + units + u', d:' + str(d) + u', seconds:' + str(d.seconds))

    if units == u'ms':
      v = d.microseconds / 1000
    elif units == u'm':
      v = d.seconds / 60
    elif units == u'h':
      v = d.seconds / 60 / 60
    elif units == u'd':
      v = d.days
    else:
      v = d.seconds
  except Exception, e:
    if (debugMode & 4) > 0:
      logDebugInfo(u'Error time dif computation: ' + str(e) + u', d1:`' + timeStampStr1 + u'`, d2:`' + timeStampStr2 + u'`, format:' + timeStampFormat + \
                   u', units:' + units)
    v = None

  return v


def logDebugInfo(msg, device=0, endL=u'\n\n'):
  if device == 0:
    sys.stdout.write(msg + endL)
    sys.stdout.flush()
  else:
    sys.stderr.write(msg + endL)
    sys.stderr.flush()


if __name__ == "__main__":
  options, arguments = parser.parse_args()
  if options.__dict__["time_format"]:
    timeFormat = options.__dict__["time_format"]
  else:
    timeFormat = u'{0:08d}'
  if options.__dict__["time_units"]:
    timeUnits = options.__dict__["time_units"]
  else:
    timeUnits = u's'
  if options.__dict__["fields_delimiter"]:
    fieldsFelimiter = options.__dict__["fields_delimiter"]
  else:
    fieldsFelimiter = u' - '
  if options.__dict__["timestamp_format"]:
    timestampFormat = options.__dict__["timestamp_format"]
  else:
    timestampFormat = u'%Y-%m-%d %H:%M:%S,%f'
  if options.__dict__["timestamp_index"]:
    timestampIndex = options.__dict__["timestamp_index"]
  else:
    timestampIndex = 0
  if options.__dict__["debug"]:
    debugMode = int(options.__dict__["debug"])
  else:
    debugMode = 0
  if options.__dict__["skip"]:
    skipWithoutTimeStamp = int(options.__dict__["skip"])
  else:
    skipWithoutTimeStamp = 1


  try:
    sourceStr = sys.stdin.read()
  except:
    sys.exit()

  if sourceStr is not None and sourceStr != u'':
    try:
      lines = sourceStr.strip().split(u'\n')
      if (debugMode & 1) > 0:
        logDebugInfo(u'Lines: ' + str(len(lines)))

      times = [0]
      fields = []
      r = u''

      if len(lines) == 1:
        f = lines[0].split(fieldsFelimiter)
        lines.insert(0, f[timestampIndex])

      for i in xrange(len(lines)):
        f = lines[i].split(fieldsFelimiter)
        fields.append(f)
        if i > 0:
          et = getTimeStampDelta(fields[i - 1][timestampIndex], fields[i][timestampIndex], timestampFormat, timeUnits, debugMode)
          times.append(et)
        if times[i] is not None:
          v = times[i]
        else:
          if skipWithoutTimeStamp:
            v = None
          else:
            v = 0
        if v is not None:
          r += timeFormat.format(v) + fieldsFelimiter + lines[i] + u'\n'

      if (debugMode & 1) > 0:
        times = sorted(times)
        minTime = times[1]
        maxTime = times[-1:][0]
        avgTime = sum(times) / len(times)
        d = maxTime - minTime
        di = d / 10
        cl = []

        for i in xrange (0, 10):
          cl.append({u'from':0, u'to':0, u'f':0})
          for t in xrange(1, len(times)):
            minL = minTime + i * di
            maxL = minTime + (i + 1) * di
            cl[i][u'from'] = minL
            cl[i][u'to'] = maxL
            if times[t] >= minL and times[t] < maxL:
              cl[i][u'f'] += 1
        logDebugInfo(u'Min time: ' + str(minTime) + ', max time:' + str(maxTime) + u', avg time: ' + str(avgTime) + u'\ndistribution:' + json.dumps(cl))

      sys.stdout.write(r)
    except Exception, e:
      if (debugMode & 4) > 0:
        sys.stderr.write("Error log processing: " + str(e) + "\n")
        sys.stderr.flush()
  else:
    if (debugMode & 4) > 0:
      sys.stdout.write("Input log is empty.\n")

  sys.stdout.flush()
