#!/usr/bin/python
# coding: utf-8

import re
import os
import sys
import json
import copy
import base64
import datetime
import urllib2
import urlparse
import hashlib
import tempfile
import collections
from subprocess import Popen
from subprocess import PIPE
import logging
from cement.core import foundation


class FieldsSum(object):
  # # Constants used in class
  

  # # Internal class for application parameters
  class AppParams(object):
    # #Initialization
    def __init__(self):
      self.inputFile = None
      self.outputFile = None
      self.logfile = None
      
            
  def __init__(self):
    self.appParams = self.getAppParams()
    self.logger = self.getLogger(self.appParams.logfile)


  def getAppParams(self):
    app = foundation.CementApp(self.__class__.__name__)
    app.setup()
    app.add_arg('-i', '--inputFile', action='store', metavar='input_file_name', help='input file name. ', required=True)
    app.add_arg('-o', '--outputFile', action='store', metavar='output_file_name', help='output file name. ', required=False)   
    app.add_arg('-l', '--logfile', action='store', metavar='log_file_name', help='log file name', required=True)
    app.run()

    # variable for result
    appParams = self.AppParams()
    appParams.inputFile = app.pargs.inputFile
    appParams.outputFile = app.pargs.outputFile
    appParams.logfile = app.pargs.logfile
    app.close()
    
    return appParams



  # # get console logger
  #
  # @param logfile - log file name
  # @return logger instance
  @staticmethod
  def getLogger(logfile):
    # create logger
    logger = logging.getLogger('T')
    logger.setLevel(logging.DEBUG)

    # create console handler and set level to debug
    ch = logging.FileHandler(logfile)
    ch.setLevel(logging.DEBUG)

    # create formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # add formatter to ch
    ch.setFormatter(formatter)

    # add ch to logger
    logger.addHandler(ch)

    return logger
  
  
  @staticmethod
  def jsonLoadsFile(fileName, log=None):
    #variable for result
    ret = None
    try:
      if log is not None:
        log.debug("Try load json from file '%s", str(fileName))
      
      with open(fileName, 'r') as f:
        ret = json.load(f, encoding='utf-8')
    except Exception, err:
      if log is not None:
        log.error("Load json from file '%s' failed. Error: ", str(fileName), str(err))
        
    return ret
  
  
  @staticmethod
  def outputValue(fileName, value, log=None):
    try:
      if fileName is None:
        sys.stdout.write(str(value))
      else:
        f=open(fileName, 'w')
        f.write(str(value))
        f.close()
        
        if log is not None:
          log.debug("File '%s' was saved.", str(fileName))
    except Exception, err:
      if log is not None:
        log.error("Save file '%s' failed. Error: %s", str(fileName), str(err))
        

  @staticmethod
  def getItemDataList(jsonDict, log=None):
    # variable for result
    itemObjectList = []
 
    if jsonDict is not None and 'itemsList' in jsonDict and isinstance(jsonDict['itemsList'], list):
      for itemsListElem in jsonDict['itemsList']:
        if itemsListElem is not None and 'itemObject' in itemsListElem and isinstance(itemsListElem['itemObject'], list):
          if log is not None:
            log.debug("itemsListElem['itemObject'] len = %s", str(len(itemsListElem['itemObject'])))
          
          itemObjectList.extend(itemsListElem['itemObject'])
        else:
          if log is not None:
            log.info("itemsListElem: %s", str(itemsListElem))
    else:
      if log is not None:
        log.info("jsonDict: %s", str(jsonDict))
 
    return itemObjectList


  @staticmethod
  def getTotalCount(itemObjectList):
    ret = 0
    for itemObject in itemObjectList:
      if isinstance(itemObject, dict) and 'contents' in itemObject:
        ret += int(itemObject['contents'])
    
    return ret    
  
  
  def run(self):
    self.logger.info("Started...")

    jsonDict = self.jsonLoadsFile(self.appParams.inputFile, self.logger)    
    self.logger.debug("jsonDict len= %s", str(len(jsonDict)))   
    
    itemObjectList = self.getItemDataList(jsonDict, self.logger)    
    self.logger.debug("itemObjectList len = %s", str(len(itemObjectList)))  
    
    count = self.getTotalCount(itemObjectList)    
    
    self.outputValue(self.appParams.outputFile, count, self.logger)    
    self.logger.info("Finished... count = %s", str(count))


if __name__ == "__main__":
  app = FieldsSum()
  app.run()

