#!/bin/bash

STOP="no"

if [ "$1" != "" ]
then
	STOP="yes"
fi

SLEEP=2
EXIT_CODE=1
WAIT_RAW_CONTENT=120
WAIT_PROCESSED_CONTENT=200

BIN=~/hce-node-bundle/api/python/bin
MANAGE=~/hce-node-bundle/api/python/manage
TEST_DIR=~/hce-node-bundle/api/python/data/ftests/test_logger

HEADERS_FILE1=~/hce-node-bundle/api/python/data/dc_rdata/951a36c2a958644b86268e6b8c1b7e76/45/3b59b7c9b1b1f7526b79e2a1eba67d/453b59b7c9b1b1f7526b79e2a1eba67d.headers.txt
HEADERS_FILE2=~/hce-node-bundle/api/python/data/dc_rdata/242d4d9744b40b17a5e78b20b2871917/7f/8b31a96a6a93a9f2571e01abe5210a/7f8b31a96a6a93a9f2571e01abe5210a.headers.txt
HEADERS_FILE3=~/hce-node-bundle/api/python/data/dc_rdata/5b136696c99b463acbd306c95640f1ca/af/27c639e8fee4a54959f93686aad60b/af27c639e8fee4a54959f93686aad60b.headers.txt

REQUESTS_FILE1=~/hce-node-bundle/api/python/data/dc_rdata/951a36c2a958644b86268e6b8c1b7e76/45/3b59b7c9b1b1f7526b79e2a1eba67d/453b59b7c9b1b1f7526b79e2a1eba67d.requests.txt
REQUESTS_FILE2=~/hce-node-bundle/api/python/data/dc_rdata/242d4d9744b40b17a5e78b20b2871917/7f/8b31a96a6a93a9f2571e01abe5210a/7f8b31a96a6a93a9f2571e01abe5210a.requests.txt
REQUESTS_FILE3=~/hce-node-bundle/api/python/data/dc_rdata/5b136696c99b463acbd306c95640f1ca/af/27c639e8fee4a54959f93686aad60b/af27c639e8fee4a54959f93686aad60b.requests.txt



DAEMON_PROCESSES=("dc-daemon.py" "dtm-daemon.py")

copyFile()
# $1 - src
# $2 - dst
{
  if [ -f $1 ]
  then
    cmd="cp $1 $2"
    echo "$cmd"  
    $cmd
  fi
}

copyHeadersFiles()
{
  dst=../log/
  copyFile $HEADERS_FILE1 $dst
  copyFile $HEADERS_FILE2 $dst
  copyFile $HEADERS_FILE3 $dst
  copyFile $REQUESTS_FILE1 $dst
  copyFile $REQUESTS_FILE2 $dst
  copyFile $REQUESTS_FILE3 $dst  
}

cleanup()
{
	cd $BIN
	echo "Cleanup log files"
	rm -f ../log/*
}

isExistProcess()
# $1 - process name
# return 1 if exist or 0 othewise
{
	PID=$(pgrep $1)

	if [ $PID ]
	then
		return 1
	else
		return 0
	fi
}

processes=0
for p in ${DAEMON_PROCESSES[@]}
	do
		isExistProcess "$p"
		isExist=$?

		if [ $isExist -eq 1 ]
		then
			processes=$(($processes+1))
		fi
	done

if [ $processes -eq 0 ]
then
	cleanup
else
  # start cluster
  cd $MANAGE
  ./dtm-daemon_start.sh 
  ./dc-daemon_start.sh

  sleep $SLEEP
fi



cd $BIN

NEW_SITE_FILES=$TEST_DIR/site_new*.json
for f in $NEW_SITE_FILES
do
  echo "Create new site"
  echo $f > ../log/$0.log 2>&1
  ./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=$f >> ../log/$0.log 2>&1
done

sleep $SLEEP

NEW_URL_FILES=$TEST_DIR/url_new*.json
for f in $NEW_URL_FILES
do
  echo "Create new url"
  echo $f >> ../log/$0.log 2>&1
  ./dc-client.py --config=../ini/dc-client.ini --command=URL_NEW --file=$f >> ../log/$0.log 2>&1
done

sleep $SLEEP

echo "Wait for getting raw content:"
a=0
while [ $a -lt $WAIT_RAW_CONTENT ]
	do
		sleep 10
		a=$(($a+10))
		echo "time: $a sec."
	done 

TMP_DIR="/tmp"
TMP_CONTENT_FILE="$TMP_DIR/content.json.tmp"
RAW_CONTENTS_FILE="$TMP_DIR/rawContents.json"

CONTENT_URL_FILES=$TEST_DIR/url_content*.json
for f in $CONTENT_URL_FILES
do
  echo $f >> ../log/$0.log 2>&1
	./dc-client.py --config=../ini/dc-client.ini --command=URL_CONTENT --file=$f > "$TMP_CONTENT_FILE" 2>&1
	cat "$TMP_CONTENT_FILE" >> ../log/$0.log 2>&1


	$BIN/json-field.py -f "itemsList:0:itemObject:0:rawContents:0:buffer" -b 1 < "$TMP_CONTENT_FILE" > "$RAW_CONTENTS_FILE"
	RAW_CONTENTS=`cat "$RAW_CONTENTS_FILE"`

	if [ -n "$RAW_CONTENTS" ]
	then
		echo "Raw content was got successfully"
		EXIT_CODE=0
	fi
done

copyHeadersFiles

rm -f $RAW_CONTENTS_FILE
rm -f $TMP_CONTENT_FILE

sleep $SLEEP

DELETE_URL_FILES=$TEST_DIR/url_delete*.json
for f in $DELETE_URL_FILES
do
  echo "Delete url"
  echo $f >> ../log/$0.log 2>&1
  ./dc-client.py --config=../ini/dc-client.ini --command=URL_DELETE --file=$f >> ../log/$0.log 2>&1
done

sleep $SLEEP

DELETE_SITE_FILES=$TEST_DIR/site_delete*.json
for f in $DELETE_SITE_FILES
do
	echo "Delete site"
  echo $f >> ../log/$0.log 2>&1
  ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=$f >> ../log/$0.log 2>&1
done

# stop cluster if necessary
if [ "$STOP" == "yes" ]
then
	cd $MANAGE
	./dc-daemon_stop.sh
	./dtm-daemon_stop.sh 
fi

exit $EXIT_CODE

