#!/bin/bash

if [[ -z "$@" ]]
then
    echo "Usage: $0 id=\"site_id\""
    exit 1
fi

JSONPARSER='../bin/json-field.py'

function parser(){
  # $1 - json file
  HOST='itemsList:0:host'
  CHECK='itemsList:0:errorCode'
  ERROR='itemsList:0:errorMessage'
  CHECK_ERROR=$($JSONPARSER -f "$CHECK"< $1)
  HOSTS=$($JSONPARSER -f "$HOST"< $1)
  hosts=(${HOSTS//;/ })
  
  set -f                      # avoid globbing (expansion of *).
  array=(${CHECK_ERROR//;/ })
  for i in "${!array[@]}"
  do
    #echo "$i = ${array[$i]}"
    if [[ ${array[$i]} -eq 0 ]]
    then  
      echo "${hosts[$i]} - Success"
    else
      echo "Error:"
      $JSONPARSER -f "$ERROR"
      break
    fi
        
  done
}

# CHECK ARGUMENTS #
for ARG in "$@"
do
    case "$ARG" in
        id=*|-id=*|--id=*)
            ID=$(echo $ARG | awk -F= '{print $2}');;
        force|-force|--force)
            FORCE="TRUE";;
    esac
done

if [[ -z $ID ]]
then
    echo "Usage: $0 id=\"site_id\""
    exit 1
fi

# FORCE? #
if [[ -z $FORCE ]]
then
    echo "Reprocess site? If yes, print YES (in uppercase)"
    read READ
    if [[ $READ == "YES" ]] || [[ $READ == "yes" ]]
    then
        :
    else
        echo "Canceled, exit"
        exit 1
    fi
fi

cd ../manage/
# CHECK STATUS OF DC #
echo "Check status of DC daemon. Please, wait..."
CHECK=$(./dc-daemon_status.sh | grep stopped)
if [[ $CHECK ]]
then
    echo "DC service not started. Canseled"
    exit 1
fi

# CHECK STATUS OF DTMD #
echo "Check status of DTM daemon. Please, wait..."
CHECK=$(./dtm-daemon_status.sh | grep stopped)
if [[ $CHECK ]]
then
    echo "DTM service not started. Canseled"
    exit 1
fi

TEMPFILE=`mktemp /tmp/$(basename $0).XXXXXX`
trap "rm -f $TEMPFILE" 0 1 2 5 15

FIELDS='{"id": "'$ID'"}'
#FIELDS='{"id": "'$ID'", "status":4, "oldStatuses":[5,6,7]}'
cd ../bin && ./dc-client.py --config=../ini/dc-client.ini --command=SITE_REPROCESS --fields="$FIELDS" > $TEMPFILE
cp $TEMPFILE /tmp/dc_test_sites_reprocess.log

parser $TEMPFILE

echo "Done"

exit 0
