#!/bin/bash

# Executor - Maksym Masalov
# Subject of testing - property DETECT_MODIFIED

set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
RESULT_FILE_CONTENT=$(mktemp /tmp/content_result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)

        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
	    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
	    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	    return 1
                fi
        else
	echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	return 1
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

echo "Test start"
echo ''
echo -e '\e[35m----- Check of algorithm 1: HTTP GET "ETag" + "If-None-Match"; compare: "ETag" old and new -----\e[0m'
echo ''
cd /home/hce/hce-node-bundle/api/python/bin
echo 'Create new site' >> ../log/${base}.log 2>&1

res_upd=0
res_fetch=0
res_alg1=0
res_alg2=0
res_alg3=0
res_alg4=0
res_alg5=0

# Create a new project in the DC service, sets root URLs, initialize templates, patterns and another related structures.
# In case of conditions are succeed new site included in to the crawling right after creation.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_NEW" $TMP_ERROR_MESSAGE_FILE

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'

# Updates fields of existing project in the DC service.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_update_algorithm1.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_UPDATE" ${res_upd}

echo 'Start project processing'
echo 'waiting 20 sec...'
sleep 20
echo 'Processed'
echo ''

# Returns the array of url objects satisfies with criterions and selection conditions.
# Can be used to check state of resources, also, used internally by DC service to make crawling batches. Can also change state and other fields of selected URL objects
./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
ITEM_OBJECT=$(jq '.itemsList[0].itemObject[]' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "200" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
#    echo 'HTTP code of crawled resource is 200'
elif [ "$HTTP_CODE" = "0" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
elif [ "$ITEM_OBJECT" = "[]" ]; then
    echo -e "\e[36mITEM_OBJECT\e[0m - \e[1;31mError\e[0m"
else
    res_alg1=${res_alg1}+1
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
fi

echo ''
echo -e '\e[35m----- Check of algorithm 2: HTTP GET "If-Modified-Since"; compare: CDate and "Last-modified" -----\e[0m'
echo ''
echo 'Start project re-crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'

# Updates fields of existing project in the DC service.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_update_algorithm2.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_UPDATE" ${res_upd}

echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Processed'
echo ''

# Returns the array of url objects satisfies with criterions and selection conditions.
# Can be used to check state of resources, also, used internally by DC service to make crawling batches. Can also change state and other fields of selected URL objects
./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

echo '...'
echo 'waiting 20 sec...'
sleep 20

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
ITEM_OBJECT=$(jq '.itemsList[0].itemObject[]' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "200" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
#    echo 'HTTP code of crawled resource is 200'
elif [ "$HTTP_CODE" = "0" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
elif [ "$ITEM_OBJECT" = "[]" ]; then
    echo -e "\e[36mITEM_OBJECT\e[0m - \e[1;31mError\e[0m"
else
    res_alg1=${res_alg1}+1
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
fi

echo ''
echo -e '\e[35m----- Check of algorithm 3: HTTP HEAD compare: CDate and "Last-modified". If modified to send a second request GET -----\e[0m'
echo ''
echo 'Start project re-crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'

# Updates fields of existing project in the DC service.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_update_algorithm3.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_UPDATE" ${res_upd}

echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Processed'
echo ''

# Returns the array of url objects satisfies with criterions and selection conditions.
# Can be used to check state of resources, also, used internally by DC service to make crawling batches. Can also change state and other fields of selected URL objects
./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

echo '...'
echo 'waiting 20 sec...'
sleep 20

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
ITEM_OBJECT=$(jq '.itemsList[0].itemObject[]' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "200" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
#    echo 'HTTP code of crawled resource is 200'
elif [ "$HTTP_CODE" = "0" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
elif [ "$ITEM_OBJECT" = "[]" ]; then
    echo -e "\e[36mITEM_OBJECT\e[0m - \e[1;31mError\e[0m"
else
    res_alg1=${res_alg1}+1
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
fi

echo ''
echo -e '\e[35m----- Check of algorithm 4: HTTP GET compare: raw content crc32 old and new -----\e[0m'
echo ''
echo 'Start project re-crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'

# Updates fields of existing project in the DC service.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_update_algorithm4.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_UPDATE" ${res_upd}

echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Processed'
echo ''

# Returns the array of url objects satisfies with criterions and selection conditions.
# Can be used to check state of resources, also, used internally by DC service to make crawling batches. Can also change state and other fields of selected URL objects
./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

echo '...'
echo 'waiting 20 sec...'
sleep 20

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
ITEM_OBJECT=$(jq '.itemsList[0].itemObject[]' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "200" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
#    echo 'HTTP code of crawled resource is 200'
elif [ "$HTTP_CODE" = "0" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
elif [ "$ITEM_OBJECT" = "[]" ]; then
    echo -e "\e[36mITEM_OBJECT\e[0m - \e[1;31mError\e[0m"
else
    res_alg1=${res_alg1}+1
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
fi

echo ''
echo -e '\e[35m----- Check of algorithm 5: HTTP GET compare: raw content size old and new -----\e[0m'
echo ''
echo 'Start project re-crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'

# Updates fields of existing project in the DC service.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_update_algorithm5.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_UPDATE" ${res_upd}

echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Processed'
echo ''

# Returns the array of url objects satisfies with criterions and selection conditions.
# Can be used to check state of resources, also, used internally by DC service to make crawling batches. Can also change state and other fields of selected URL objects
./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

echo '...'
echo 'waiting 20 sec...'
sleep 20

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
ITEM_OBJECT=$(jq '.itemsList[0].itemObject[]' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "200" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
#    echo 'HTTP code of crawled resource is 200'
elif [ "$HTTP_CODE" = "0" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
elif [ "$ITEM_OBJECT" = "[]" ]; then
    echo -e "\e[36mITEM_OBJECT\e[0m - \e[1;31mError\e[0m"
else
    res_alg1=${res_alg1}+1
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
fi

if [ "$res_upd" = "0" ] && [ "$res_fetch" = "0" ] && [ "$res_alg1" = "0" ] && [ "$res_alg2" = "0" ] && [ "$res_alg3" = "0" ] && [ "$res_alg4" = "0" ] && [ "$res_alg5" = "0" ] ; then
  echo 'Everything is ok, the project is deleting'
#  Delete project and all related data and structures from DC service.
#  ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_delete.json > $TMP_RESULT_FILE
#  checkErrors
#  breakOnError $? "SITE_DELETE"
  echo ''
  echo 'waiting 10 sec...'
  sleep 10
  echo ''
  echo 'Project was deleted'
  exit 0
else
  echo ''
  echo -e '\e[1;31mSomething was wrong\e[0m'
  exit 1
fi