#!/bin/bash

# Executor - Maksym Masalov
# Subject of testing - property DETECT_MODIFIED (ETag)

set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
RESULT_FILE_CONTENT=$(mktemp /tmp/content_result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)

        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
	    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
	    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	    return 1
                fi
        else
	echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	return 1
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

echo "Test start"

#Check if project exists
cd /home/hce/hce-node-bundle/api/python/bin
./dc-client.py --config=../ini/dc-client.ini --command=SITE_STATUS --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_status.json > $TMP_RESULT_FILE

exists=$(jq -r '.itemsList[0].itemObject' $TMP_RESULT_FILE)
echo ''
if [[ "${exists}" != "null"  ]]; then
    #Delete project
    echo 'Project already exists. Deleting project'
    cd /home/hce/hce-node-bundle/api/python/bin
    ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_delete_etag.json > $TMP_RESULT_FILE
    checkErrors
    breakOnError $? "SITE_DELETE" $TMP_ERROR_MESSAGE_FILE
    sleep 15
fi

echo ''
echo -e '\e[35m----- Check: HTTP GET "ETag" + "If-None-Match"; compare: "ETag" old and new -----\e[0m'
cd /home/hce/hce-node-bundle/api/python/bin
echo 'Create new site' >> ../log/${base}.log 2>&1

res_upd=0
res_fetch=0
res_1=0
res_2=0
res_3=0
res_4=0
res_5=0
res_6=0

# Create a new project in the DC service, sets root URLs, initialize templates, patterns and another related structures.
# In case of conditions are succeed new site included in to the crawling right after creation.
#./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=/home/hce/test/DC_DETECT_MODIFIED/test_DETECT_MODIFIED.site.etag.json > $TMP_RESULT_FILE
./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site.etag.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_NEW" $TMP_ERROR_MESSAGE_FILE

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 360 sec...'
sleep 360
echo 'Crawled'

# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = hash-123-noquotes -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag1.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "308" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_1=${res_1}+1
fi


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = "hash-123" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag2.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "308" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_2=${res_2}+1
fi


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = "hash-123-gzip" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag3.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "308" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_3=${res_3}+1
fi

# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = W/hash-123-noquotes -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag4.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "308" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_4=${res_4}+1
fi


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = W/"hash-123 algo" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag5.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "308" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_5=${res_5}+1
fi


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = W/"hash-123 algo-gzip" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag6.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "308" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_6=${res_6}+1
fi

# ----------------------------------------------------------------------------------------------------------------------

# Updates fields of existing project in the DC service.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_update_etag.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_UPDATE" ${res_upd}


echo 'Start project re-crawling'
echo '...'
echo 'waiting 400 sec...'
sleep 400
echo 'Re-crawled'


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = hash-123-noquotes -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag1.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "304" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_1=${res_1}+1
fi

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = "hash-123" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag2.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "304" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_2=${res_2}+1
fi

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = "hash-123-gzip" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag3.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "304" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_3=${res_3}+1
fi

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = W/hash-123-noquotes -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag4.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "304" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_4=${res_4}+1
fi

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = W/"hash-123 algo" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag5.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "304" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_5=${res_5}+1
fi

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag


# ----------------------------------------------------------------------------------------------------------------------
echo ''
echo -e '\e[35m----- "ETag" = W/"hash-123 algo-gzip" -----\e[0m'
echo ''

./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.url_fetch_etag6.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/test_DETECT_MODIFIED_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res_fetch}

HTTP_CODE=$(jq '.itemsList[0].itemObject[].httpCode' $TMP_RESULT_FILE)
echo $HTTP_CODE
# Check the HTTP code of the crawled resource
if [ "$HTTP_CODE" = "304" ]; then
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[36mHTTP_CODE\e[0m - \e[1;31mError\e[0m"
    res_6=${res_6}+1
fi

eTag=$(jq '.itemsList[0].itemObject[].eTag' $TMP_RESULT_FILE)
echo $eTag


if [ "$res_upd" = "0" ] && [ "$res_fetch" = "0" ] && [ "$res_1" = "0" ] && [ "$res_2" = "0" ] && [ "$res_3" = "0" ] && [ "$res_4" = "0" ] && [ "$res_5" = "0" ] && [ "$res_6" = "0" ] ; then
  echo -e '\e[1;32mEverything is ok, the project is deleting\e[0m'
  echo ''
  # Delete project and all related data and structures from DC service.
  ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/DETECT_MODIFIED/test_DETECT_MODIFIED.site_delete_etag.json > $TMP_RESULT_FILE
  checkErrors
  breakOnError $? "SITE_DELETE"
  echo 'waiting 10 sec...'
  sleep 10
  echo 'Project was deleted'
  exit 0
else
  echo ''
  echo -e '\e[1;31mSomething was wrong\e[0m'
  exit 1
fi
