#!/bin/bash

# Executor - Maksym Masalov
# Subject of testing - property HTTP_FREQ_LIMITS

set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.${base}.XXXXX)
RESULT_FILE_CONTENT=$(mktemp /tmp/content_result.${base}.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.${base}.XXXXX)
echo 'RESULT FILE: '${TMP_RESULT_FILE}
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' ${TMP_RESULT_FILE})
        res=0
        ERROR_MESSAGE=$(jq -r '.errorMessage' ${TMP_RESULT_FILE})
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' ${TMP_RESULT_FILE})

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' ${TMP_RESULT_FILE})

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
	    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
	    return 0
                else
                        echo ${ERROR_MESSAGE} > ${TMP_ERROR_MESSAGE_FILE}
                        res=${res}+1
	    return 1
                fi
        else
	 echo ${ERROR_MESSAGE} > ${TMP_ERROR_MESSAGE_FILE}
	 return 0
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

echo "Test start"

#Check if project exists
cd /home/hce/hce-node-bundle/api/python/bin
./dc-client.py --config=../ini/dc-client.ini --command=SITE_STATUS --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.site_status.json > $TMP_RESULT_FILE

exists=$(jq -r '.itemsList[0].itemObject' $TMP_RESULT_FILE)
echo ''
if [[ "${exists}" != "null"  ]]; then
    #Delete project
    echo 'Project already exists. Deleting project'
    cd /home/hce/hce-node-bundle/api/python/bin
    ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.site_delete.json > $TMP_RESULT_FILE
    checkErrors
    breakOnError $? "SITE_DELETE" $TMP_ERROR_MESSAGE_FILE
    sleep 15
fi

echo ''
echo -e '\e[35m----- Check HTTP_FREQ_LIMITS property without random -----'
echo ''
cd /home/hce/hce-node-bundle/api/python/bin
echo 'Create new site' >> ../log/${base}.log 2>&1

res=0
res_without=0
res_with=0

./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.site.json > ${TMP_RESULT_FILE}
checkErrors
breakOnError $? "SITE_NEW" ${TMP_ERROR_MESSAGE_FILE}

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 420 sec...'
sleep 420
echo 'Crawled'

result_without=$(jq '.res' /home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/http_limits_time.json)
if [[ "${result_without}" -ge "99" ]]; then
    echo -e "\e[36mTIME\e[0m - \e[1;32mSuccess\e[0m"
    echo ${result_without}
else
    echo -e "\e[36mTIME\e[0m - \e[1;31mFail\e[0m"
    echo ${result_without}
    res_without=${res_without}+1
fi

echo ''
echo -e '\e[35m----- Check HTTP_FREQ_LIMITS property with random ----- \e[0m'

./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.site_update.json > ${TMP_RESULT_FILE}
cp ${TMP_RESULT_FILE} /tmp/HTTP_FREQ_LIMITS_site_update.json
checkErrors
breakOnError $? "SITE_UPDATE" ${res}

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 500 sec...'
sleep 500
echo 'Crawled'

result_with=$(jq '.res' /home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/http_limits_time.json)
if [[ "${result_with}" -ge "199" && "${result_with}" -le "300" ]]; then
    echo -e "\e[36mTIME\e[0m - \e[1;32mSuccess\e[0m"
    echo ${result_with}
else
    echo -e "\e[36mTIME\e[0m - \e[1;31mFail\e[0m"
    echo ${result_with}
    res_with=${res_with}+1
fi

echo ''
echo -e '\e[35m----- Check HTTP_FREQ_LIMITS property without key "storage" ----- \e[0m'

./dc-client.py --config=../ini/dc-client.ini --command=SITE_UPDATE --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.site_update_for_storage.json > ${TMP_RESULT_FILE}
cp ${TMP_RESULT_FILE} /tmp/HTTP_FREQ_LIMITS_site_update_for_storage.json
checkErrors
breakOnError $? "SITE_UPDATE" ${res}

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 420 sec...'
sleep 420
echo 'Crawled'

if [[ -f /tmp/127.0.0.1/192.168.253.240/http_limits_699fcf4591fc23e79b839d8819904298.json ]]; then

    echo -e "\e[36mRESULT_FILE\e[0m - \e[1;32mEXIST into /tmp/ directory\e[0m"

    ./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.first_request.url_fetch.json > ${TMP_RESULT_FILE}
    cp ${TMP_RESULT_FILE} /tmp/HTTP_FREQ_LIMITS_first_url_fetch.json
    checkErrors
    breakOnError $? "URL_FETCH_FIRST" ${res}

    ./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.second_request.url_fetch.json > ${TMP_RESULT_FILE}
    cp ${TMP_RESULT_FILE} /tmp/HTTP_FREQ_LIMITS_second_url_fetch.json
    checkErrors
    breakOnError $? "URL_FETCH_SECOND" ${res}

    first=$(jq -r '.itemsList[0].itemObject[].UDate' /tmp/HTTP_FREQ_LIMITS_first_url_fetch.json)
    second=$(jq -r '.itemsList[0].itemObject[].UDate' /tmp/HTTP_FREQ_LIMITS_second_url_fetch.json)
    echo "First - ${first}"
    echo "Second - ${second}"

    res_first=$(date --date="${first}" +"%s")
    res_second=$(date --date="${second}" +"%s")
    echo "First res - ${res_first}"
    echo "Second res - ${res_second}"

    result=$((${res_second} - ${res_first}))
    echo "Result - ${result}"

    if [[ "${result}" -ge "99" ]]; then
        echo -e "\e[36mTIME\e[0m - \e[1;32mSuccess\e[0m"
        echo ${result}
    else
        echo -e "\e[36mTIME\e[0m - \e[1;31mFail\e[0m"
        echo ${result}
        res_without=${res_without}+1
    fi
else
    echo -e "\e[36mRESULT_FILE\e[0m - \e[1;31mDOESN'T_CREATED\e[0m"
fi

rm -R -f /tmp/127.0.0.1
rm -f /home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/http_limits_time.json
rm -R -f /home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/127.0.0.1

if [ "${res}" = "0" ] && [ "${res_without}" = "0" ] && [ "${res_with}" = "0" ]; then
  echo -e '\e[1;32mEverything is ok, the project is deleting\e[0m'
  # Delete project and all related data and structures from DC service.
  ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/HTTP_FREQ_LIMITS/test_HTTP_FREQ_LIMITS.site_delete.json > $TMP_RESULT_FILE
  checkErrors
  breakOnError $? "SITE_DELETE"
  echo ''
  echo 'waiting 10 sec...'
  sleep 10
  echo ''
  echo -e '\e[1;32mProject was deleted\e[0m'
  exit 0
else
  echo ''
  echo -e '\e[1;31mSomething was wrong\e[0m'
  exit 1
fi


