#!/bin/bash

# Executor - Maksym Masalov
# Subject of testing - property SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_240

set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
RESULT_FILE_CONTENT=$(mktemp /tmp/content_result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)
        res=0
        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
	    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
	    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
                        res=$res+1
	    return 1
                fi
        else
	 echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	 return 0
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

echo "Test start"
echo ''
echo -e '\e[35m----- Check SQL EXPRESSION FIELDS UPDATE PROCESSOR and CRAWLER property -----'
echo ''
cd /home/hce/hce-node-bundle/api/python/bin
echo 'Create new site' >> ../log/${base}.log 2>&1
res=0
# Create a new project in the DC service, sets root URLs, initialize templates, patterns and another related structures.
# In case of conditions are succeed new site included in to the crawling right after creation.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=/home/hce/trunk/api/python/data/ftests/properties_json/SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER/test_SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER.site.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_NEW" $TMP_ERROR_MESSAGE_FILE

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'
echo ''
echo '...'
echo 'waiting 10 sec...'
sleep 10
echo ''

# Returns the array of url objects satisfies with criterions and selection conditions.
# Can be used to check state of resources, also, used internally by DC service to make crawling batches. Can also change state and other fields of selected URL objects
./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER/test_SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER.url_fetch.json > $TMP_RESULT_FILE
cp $TMP_RESULT_FILE /tmp/SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER_url_fetch.json
checkErrors
breakOnError $? "URL_FETCH" ${res}

priority=$(jq '.itemsList[0].itemObject[].priority' $TMP_RESULT_FILE)
if [ "$priority" = "111" ]; then
  echo -e "\e[36mPriority after crawling\e[0m - \e[1;32mSuccess\e[0m"
  echo ''
  echo 'Start project processing'
  echo '...'
  echo 'waiting 180 sec...'
  sleep 180
  echo 'Processed'
  echo ''
  ./dc-client.py --config=../ini/dc-client.ini --command=URL_FETCH --file=/home/hce/trunk/api/python/data/ftests/properties_json/SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER/test_SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER.url_fetch.json > $TMP_RESULT_FILE
  cp $TMP_RESULT_FILE /tmp/SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER_url_fetch.json
  checkErrors
  breakOnError $? "URL_FETCH" ${res}
elif [ "$priority" = "112" ]; then
  echo '\e[36mPriority after processing\e[0m - \e[1;31mSuccess\e[0m'
  echo $priority
else
  echo -e '\e[36mPriority after crawling\e[0m - \e[1;31mError\e[0m'
  res_crawl=${res}+1
fi

priority=$(jq '.itemsList[0].itemObject[].priority' $TMP_RESULT_FILE)
if [ "$priority" = "112" ]; then
  echo -e '\e[36mPriority after processing\e[0m - \e[1;31mSuccess\e[0m'
  echo $priority
else
  echo -e '\e[36mPriority after processing\e[0m - \e[1;31mError\e[0m'
  res_process=${res}+1
fi

if [ "$res" = "0" ] && [ "$res_crawl" = "0" ] && [ "$res_process" = "0" ]; then
  echo 'Everything is ok, the project is deleting'
#  Delete project and all related data and structures from DC service.
#  ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER/test_SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR_AND_CRAWLER.site_delete.json > $TMP_RESULT_FILE
#  checkErrors
#  breakOnError $? "SITE_DELETE"
  echo ''
  echo 'waiting 10 sec...'
  sleep 10
  echo ''
  echo 'Project was deleted'
  exit 0
else
  echo ''
  echo -e '\e[1;31mSomething was wrong\e[0m'
  exit 1
fi