#!/bin/bash

# Executor - Anton Dmytrenko
# Subject of testing - property USER_PROXY
# PROXY SERVER:  /home/hce/trunk/api/python/data/ftests/third_party_software/server.py
# cgi script : /var/www/html/python_cgi/cgi_check_proxy.py
#
#
#

set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
RESULT_FILE_CONTENT=$(mktemp /tmp/content_result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
errors=0
proxy=0
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)

        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [[ "${proxy}" != "0"  ]]; then

                    decoded3=$(jq -r '.itemsList[0].itemObject[0].headers[0].buffer' $TMP_RESULT_FILE | base64 -di)

                    echo "$decoded3"
                    if [[ "${decoded3}" == *"proxy-detected2: TRUE"* ]]; then
                        echo ''
                        echo -e "\e[1;31mERROR-----Proxy response IS PRESENT in a request to a url which was not included in --domains-- array in USER_PROXY property\e[0m"
                        ((errors+=1))
                        return 1
                    elif [[ "${decoded3}" != *"proxy-detected2: TRUE"* ]]; then
                        echo ''
                        echo -e "\e[1;32mOK-----Proxy response IS NOT PRESENT in a request to a url which was not included in --domains-- array in USER_PROXY property\e[0m"
                        return 0
                    fi
                fi
                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
	    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
	    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	    return 1
                fi
        else
	echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
	return 1
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
fi
}
function ctrl_c()
{
echo "Killing proxy server"
kill -9 $server_process_id
exit
}
echo "Starting test of USER_PROXY "
echo ''
echo ''
echo -e '\e[35mStarting proxy server. Proxy: "user_proxy" (127.0.0.2) at port 2222'

cd /home/hce/trunk/api/python/data/ftests/third_party_software
python3 user_proxy_server.py & #&>/dev/null &
server_process_id=$!
echo "Python server process id: ${server_process_id}"
trap ctrl_c INT

#Check if project exists
cd /home/hce/hce-node-bundle/api/python/bin
./dc-client.py --config=../ini/dc-client.ini --command=SITE_STATUS --file=/home/hce/trunk/api/python/data/ftests/properties_json/USER_PROXY/test_USER_PROXY_present_checkproxy.site_status.json > $TMP_RESULT_FILE

exists=$(jq -r '.itemsList[0].itemObject' $TMP_RESULT_FILE)
echo ''
if [[ "${exists}" != "null"  ]]; then
    #Delete project
    echo 'Project already exists. Deleting project'
    cd /home/hce/hce-node-bundle/api/python/bin
    ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/USER_PROXY/test_USER_PROXY_present_checkproxy.site_delete.json > $TMP_RESULT_FILE
    checkErrors
    breakOnError $? "SITE_DELETE - USER_PROXY" $TMP_ERROR_MESSAGE_FILE
    sleep 15
fi

echo ''
echo ''
echo "Triple redirect where second page shouldn't go through proxy"
cd /home/hce/hce-node-bundle/api/python/bin
echo 'Create new site WITH CUSTOM PROXY PRESENT' >> ../log/${base}.log 2>&1
# Create a new project in the DC service, sets root URLs, initialize templates, patterns and another related structures.
# In case of conditions are succeed new site included in to the crawling right after creation.
./dc-client.py --config=../ini/dc-client.ini --command=SITE_NEW --file=/home/hce/trunk/api/python/data/ftests/properties_json/USER_PROXY/test_USER_PROXY_present_checkproxy.site_new.json > $TMP_RESULT_FILE
checkErrors
breakOnError $? "SITE_NEW - USER_PROXY" $TMP_ERROR_MESSAGE_FILE

echo ''
echo 'Start project crawling'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Crawled'
echo 'Start project processing'
echo '...'
echo 'waiting 180 sec...'
sleep 180
echo 'Processed'
echo ''

#Execute URL_CONTENT to validate whether the custom proxy was engaged
echo ''
echo ''
cd /home/hce/hce-node-bundle/api/python/bin
echo 'Send URL_CONTENT command for USER_PROXY_present_checkproxy DC test' >> ../log/${base}.log 2>&1
# Run URL_CONTENT command to check http headers
./dc-client.py --config=../ini/dc-client.ini --command=URL_CONTENT --file=/home/hce/trunk/api/python/data/ftests/properties_json/USER_PROXY/test_USER_PROXY_present_checkproxy.url_content.json > $TMP_RESULT_FILE
((proxy+=1))
checkErrors
breakOnError $? "URL_CONTENT - USER_PROXY"

echo ''
if [[ $errors != 0 ]]; then
    echo -e "\e[1;31mTest did not complete successfully. Errors: ${errors}. Returning Test Failed\e[0m"
    echo ''
    echo ''
    echo ''
    echo -e '\e[1;36m--- Examples of proxy usage ---\e[0m'
    echo ''
    echo ''
    echo -e '\e[1;32m--- CORRECT --- Curl with proxy for 1st page (Proxy-detected1: TRUE in response headers):\e[0m'
    curl -vv --proxy user_proxy:2222 "http://127.0.0.1/python_cgi/cgi_check_proxy.py?1"

    echo ''
    echo -e '\e[1;31m--- WRONG --- Curl without proxy for 1st page (Proxy-detected1: FALSE and HTTP 404 in response headers):\e[0m'
    curl -vv "http://127.0.0.1/python_cgi/cgi_check_proxy.py?1"

    echo ''
    echo -e '\e[1;31m--- WRONG --- Curl with proxy for 2nd page (Proxy-detected2: TRUE and HTTP 404 in response headers):\e[0m'
    curl -vv --proxy user_proxy:2222 "http://127.0.0.1/python_cgi/cgi_check_proxy.py?2"

    echo ''
    echo -e '\e[1;32m--- CORRECT --- Curl without proxy for 2nd page (Proxy-detected1: FALSE and in response headers):\e[0m'
    curl -vv "http://127.0.0.1/python_cgi/cgi_check_proxy.py?1"

    echo ''
    echo -e '\e[1;32m--- CORRECT --- Curl with proxy for 3rd page (Proxy-detected1: TRUE in response headers):\e[0m'
    curl -vv --proxy user_proxy:2222 "http://127.0.0.1/python.cgi/cgi_check_proxy.py?3"

    echo ''
    echo -e '\e[1;31m--- WRONG --- Curl without proxy for 3rd page (Proxy-detected1: FALSE and HTTP 404 in response headers):\e[0m'
    curl -vv "http://127.0.0.1/python_cgi/cgi_check_proxy.py?3"

    kill -9 $server_process_id
    exit 1
elif [[ $errors == 0 ]]; then
    echo -e '\e[1;32mTest completed successfully with no errors. Deleting project\e[0m'
    #Delete project
    cd /home/hce/hce-node-bundle/api/python/bin
    ./dc-client.py --config=../ini/dc-client.ini --command=SITE_DELETE --file=/home/hce/trunk/api/python/data/ftests/properties_json/USER_PROXY/test_USER_PROXY_present_checkproxy.site_delete.json > $TMP_RESULT_FILE
    kill -9 $server_process_id
    exit 0
fi
#
#
#
#
#
#
#
#
