#!/bin/bash
# Executor - Anton Dmytrenko
# Subject of testing - character encoding of the processed content in the demo-form for task DC-3089


set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)

        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
                    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
                    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
                        return 1
                fi
        else
                echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
                return 1
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

# /////////// WRONG CHARSET META TAG IN HTML WITH DOUBLE == FOR EUC-JP  ////////////////

cd /home/hce/hce-node-bundle/api/python/bin
echo "Checking ==euc_jp encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/test/test_nonUTF8_tony/test_==euc-jp.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="鐃出わ申鐃俊￥申哲鐃竣「まわ申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃遵げ鐃緒申鐃獣て」￥申鐃楯人わ申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃緒申鐃瞬わ申鐃獣わ申鐃緒申鐃緒申鐃緒申鐃緒申"
    EXPECTED_CONTENT_ENCODED="<p>鐃緒申鐃緒申発表鐃緒申鐃曙た鐃瞬経営ワ申鐃宿バワ申鐃緒申鐃所ー鐃術逸申鐃緒申鐃竣につわ申鐃緒申鐃緒申鐃緒申鐃緒申鐃緒申鐃叔誌申鐃緒申鐃緒申鐃述べ￥申鐃楯人わ申鐃緒申鐃瞬まわ申鐃緒申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃緒申鐃遵げ鐃緒申鐃獣て」と居申鐃旬社逸申鐃緒申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃淑わ申鐃緒申鐃緒申</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36m==euc-jp title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36m==euc-jp title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36m==euc-jp content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36m==euc-jp content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED

    fi
else
  echo "BATCH euc-jp wrong answer, check $TMP_RESULT_FILE"
fi

cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/test/test_nonUTF8_tony/test_==euc-jp.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>鐃緒申鐃緒申発表鐃緒申鐃曙た鐃瞬経営ワ申鐃宿バワ申鐃緒申鐃所ー鐃術逸申鐃緒申鐃竣につわ申鐃緒申鐃緒申鐃緒申鐃緒申鐃緒申鐃叔誌申鐃緒申鐃緒申鐃述べ￥申鐃楯人わ申鐃緒申鐃瞬まわ申鐃緒申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃緒申鐃遵げ鐃緒申鐃獣て」と居申鐃旬社逸申鐃緒申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃淑わ申鐃緒申鐃緒申</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE="鐃出わ申鐃俊￥申哲鐃竣「まわ申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃遵げ鐃緒申鐃獣て」￥申鐃楯人わ申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃緒申鐃瞬わ申鐃獣わ申鐃緒申鐃緒申鐃緒申鐃緒申"

# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: ==EUC-JP'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: ==EUC-JP'
    echo ''
fi

