#!/bin/bash
# Executor - Anton Dmytrenko

set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)

        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
		    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
		    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
			return 1
                fi
        else
		echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
		return 1
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

echo "Test start"
cd /home/hce/hce-node-bundle/api/python/bin

echo "Checking uft8 (jp) encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/test/test_nonUTF8_tony/test_UTF8jp.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="「ボクは･･･」で論点逸らす闇営業会見の悍ましさ"
    EXPECTED_CONTENT_ENCODED="<p>　片や参院選、片や京都アニメーション放火殺人事件と、シリアスな問題が報じられる中で、異例の長時間記者会見その他でマスコミ的には露出の多い「お笑い芸人」の「闇営業」問題。</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo "uft8 (jp) title - OK";
    else
        echo "uft8 (jp) title - WRONG";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo "uft8 (jp) content_encoded - OK";
    else
        echo "uft8 (jp) content_encoded - WRONG";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED

    fi
else
  echo "BATCH uft8 (jp) wrong answer, check $TMP_RESULT_FILE"
fi

