#!/bin/bash
# Executor - Anton Dmytrenko
# Subject of testing - character encoding of the processed content in the demo-form for task DC-3089


set +e
# The 'basename $0' returns the filename.
base=$(basename $0)
TMP_RESULT_FILE=$(mktemp /tmp/result.$base.XXXXX)
TMP_ERROR_MESSAGE_FILE=$(mktemp /tmp/message.$base.XXXXX)
echo 'RESULT FILE: '$TMP_RESULT_FILE
function checkErrors()
## Return 1 - if check is fail, or 0 - otherwise
{
        ERROR_CODE=$(jq -r '.errorCode' $TMP_RESULT_FILE)

        ERROR_MESSAGE=$(jq -r '.errorMessage' $TMP_RESULT_FILE)
#echo "--${ERROR_MESSAGE}-- **${ERROR_CODE}**"

        if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
        then
                ERROR_CODE=$(jq -r '.itemsList[0].errorCode' $TMP_RESULT_FILE)

                ERROR_MESSAGE=$(jq -r '.itemsList[0].errorMessage' $TMP_RESULT_FILE)

                if [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "" ]
                then
                    return 0
                elif [ "$ERROR_CODE" = "0" ] && [ "$ERROR_MESSAGE" = "OK" ]
                then
                    return 0
                else
                        echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
                        return 1
                fi
        else
                echo $ERROR_MESSAGE > $TMP_ERROR_MESSAGE_FILE
                return 1
        fi
}

function breakOnError()
# exit for error
{
if [ $1 -eq 0 ]; then
    echo -e "\e[36m$2\e[0m - \e[1;32mSuccess\e[0m"
else
    echo -e "\e[1;36m$2\e[0m - \e[1;31mFail\e[0m"
    if [ "$3" != "" ]; then
    cat $3
    fi
    exit 1
fi
}

cd /home/hce/hce-node-bundle/api/python/bin
echo "Checking shift_jis encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_SHIFT_JIS.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="目指すぞ全国優勝　２チームが掛川市役所で抱負"
    EXPECTED_CONTENT_ENCODED="<p>　女子中学生のクラブチーム「５ＳＴＡＲＴ（ファイブスタート）」が二十二日、掛川市役所に松井三郎市長を訪ね、大阪市で八月十一日に開幕する全日本中学生男女ソフトボール大会への出場を報告するとともに、「優勝旗を持ち帰る」「全国制覇」と健闘を誓った。</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36mshift_jis title\e[0m - \e[1;32mOK\e[0m";
	echo ''
    else
        echo -e "\e[36mshift_jis title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36mshift_jis content encoded\e[0m - \e[1;32mOK\e[0m";
	echo ''
    else
        echo -e "\e[36mshift_jis content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED

    fi
else
  echo "BATCH shift_jis wrong answer, check $TMP_RESULT_FILE"
fi

cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_SHIFT_JIS.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>　女子中学生のクラブチーム「５ＳＴＡＲＴ（ファイブスタート）」が二十二日、掛川市役所に松井三郎市長を訪ね、大阪市で八月十一日に開幕する全日本中学生男女ソフトボール大会への出場を報告するとともに、「優勝旗を持ち帰る」「全国制覇」と健闘を誓った。</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE="目指すぞ全国優勝　２チームが掛川市役所で抱負"

# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: SHIFT_JIS'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: SHIFT_JIS'
    echo ''
fi


echo "Checking euc_jp encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_EUC_JP.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="笑い飯・哲夫「まず社員さんの給料上げたって」　芸人の待遇改善より「そっちが大事」"
    EXPECTED_CONTENT_ENCODED="<p>から発表された「経営アドバイザリー委員会」について大阪市内で持論を述べ、芸人より「まず、社員さんの給料を上げたって」と吉本社員の待遇改善を訴えた。</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
       	echo -e "\e[36meuc-jp title\e[0m - \e[1;32mOK\e[0m";
       	echo ''
    else
       	echo -e "\e[36meuc-jp title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36meuc-jp content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36meuc-jp content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED

    fi
else
  echo "BATCH euc-jp wrong answer, check $TMP_RESULT_FILE"
fi

cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_EUC_JP.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>から発表された「経営アドバイザリー委員会」について大阪市内で持論を述べ、芸人より「まず、社員さんの給料を上げたって」と吉本社員の待遇改善を訴えた。</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE="笑い飯・哲夫「まず社員さんの給料上げたって」　芸人の待遇改善より「そっちが大事」"

# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: EUC-JP'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: EUC-JP'
    echo ''
fi

echo "Checking uft8 (jp) encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_UTF8jp.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="「ボクは･･･」で論点逸らす闇営業会見の悍ましさ"
    EXPECTED_CONTENT_ENCODED="<p>　片や参院選、片や京都アニメーション放火殺人事件と、シリアスな問題が報じられる中で、異例の長時間記者会見その他でマスコミ的には露出の多い「お笑い芸人」の「闇営業」問題。</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36muft8 (jp) title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36muft8 (jp) title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36muft8 (jp) content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36muft8 (jp) content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED

    fi
else
  echo "BATCH uft8 (jp) wrong answer, check $TMP_RESULT_FILE"
fi
cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_UTF8jp.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>　片や参院選、片や京都アニメーション放火殺人事件と、シリアスな問題が報じられる中で、異例の長時間記者会見その他でマスコミ的には露出の多い「お笑い芸人」の「闇営業」問題。</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE="「ボクは･･･」で論点逸らす闇営業会見の悍ましさ"

# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: UTF8 (jp)'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: UTF8 (jp)'
    echo ''
fi
echo "Checking windows-1256 (arab) encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_WIN-1256_arab.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="تحديد موعد قرعة التصفيات الإفريقية لمونديال"
    EXPECTED_CONTENT_ENCODED="<p>وأعلن الاتحاد في بيان إعلامي امس أنه سيتم إجراء القرعة يوم يوليو الجاري بالعاصمة المصرية القاهرة، بحضور مندوبي المنتخبات التي ستشارك في التصفيات.</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36mwindows-1256 (arab) title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36mwindows-1256 (arab) title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36mwindows-1256 (arab) content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36mwindows-1256 (arab) content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED
    fi
else
  echo "BATCH windows-1256 (arab) wrong answer, check $TMP_RESULT_FILE"
fi
cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_WIN-1256_arab.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>وأعلن الاتحاد في بيان إعلامي امس أنه سيتم إجراء القرعة يوم يوليو الجاري بالعاصمة المصرية القاهرة، بحضور مندوبي المنتخبات التي ستشارك في التصفيات.</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE="تحديد موعد قرعة التصفيات الإفريقية لمونديال"


# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: WINDOWS-1256'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: WINDOWS-1256'
    echo ''
fi

echo "Checking windows-1251 (rus) encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_WIN-1251_RUS.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE='Россияне смогут оформлять небольшие ДТП с помощью мобильного приложения "Европротокол"'
    EXPECTED_CONTENT_ENCODED="<p>Пока что приложением смогут воспользоваться автомобилисты двух столицы, Подмосковья и Ленинградской области.</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36mwindows-1251 (rus) title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36mwindows-1251 (rus) title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36mwindows-1251 (rus) content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36mwindows-1251 (rus) content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED
    fi
else
  echo "BATCH windows-1251 (rus) wrong answer, check $TMP_RESULT_FILE"
fi
cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_WIN-1251_RUS.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>Пока что приложением смогут воспользоваться автомобилисты двух столицы, Подмосковья и Ленинградской области.</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE='Россияне смогут оформлять небольшие ДТП с помощью мобильного приложения "Европротокол"'


# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: WINDOWS-1251'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: WINDOWS-1251'
    echo ''
fi

echo "Checking utf-8 (rus) encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_UTF-8_RUS.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE='Врачи вернут полпреда свердловского губернатора, госпитализированного с приступом, в СИЗО'
    EXPECTED_CONTENT_ENCODED="<p>Амбулаторное обследование и лечение главы Южного управленческого округа Свердловской области Михаила Астахова, осужденного на восемь лет колонии строгого режима, будет завершено в течение недели, сообщил «URA.RU» его адвокат.</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36mutf-8 (rus) title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36mutf-8 (rus) title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36mutf-8 (rus) content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36mutf-8 (rus) content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED
    fi
else
  echo "BATCH uft-8 (rus) wrong answer, check $TMP_RESULT_FILE"
fi
cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_UTF-8_RUS.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>Амбулаторное обследование и лечение главы Южного управленческого округа Свердловской области Михаила Астахова, осужденного на восемь лет колонии строгого режима, будет завершено в течение недели, сообщил «URA.RU» его адвокат.</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE='Врачи вернут полпреда свердловского губернатора, госпитализированного с приступом, в СИЗО'


# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: utf-8'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: utf-8'
    echo ''
fi

# /////////// WRONG CHARSET META TAG IN HTML WITH DOUBLE ==  FOR UTF-8 (RUS) ////////////////

echo "Checking ==utf-8 (rus) encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_==.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE='Врачи вернут полпреда свердловского губернатора, госпитализированного с приступом, в СИЗО'
    EXPECTED_CONTENT_ENCODED='<p>Амбулаторное обследование и лечение главы Южного управленческого округа Свердловской области Михаила Астахова, осужденного на восемь лет колонии строгого режима, будет завершено в течение недели, сообщил "URA.RU" его адвокат.</p>'
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36m==utf-8 (rus) title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36m==utf-8 (rus) title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36m==utf-8 (rus) content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36m==utf-8 (rus) content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED
    fi

else
  echo "BATCH ==utf-8 (rus) wrong answer, check $TMP_RESULT_FILE"
fi

cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_==.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT='<p>Амбулаторное обследование и лечение главы Южного управленческого округа Свердловской области Михаила Астахова, осужденного на восемь лет колонии строгого режима, будет завершено в течение недели, сообщил "URA.RU" его адвокат.</p>'
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE='Врачи вернут полпреда свердловского губернатора, госпитализированного с приступом, в СИЗО'


# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: ==utf-8'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: ==utf-8'
    echo ''
fi

# /////////// WRONG CHARSET META TAG IN HTML WITH DOUBLE == FOR EUC-JP  ////////////////

echo "Checking ==euc_jp encoding"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_==euc-jp.batch.json > $TMP_RESULT_FILE
checkErrors
if [[ $? -eq 0 ]]; then
    PROCESSED_CONTENT=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
    CONTENT_ENCODED=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].content_encoded')
    TITLE=$(echo ${PROCESSED_CONTENT} | jq --raw-output '.[].title')
    EXPECTED_TITLE="鐃出わ申鐃俊￥申哲鐃竣「まわ申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃遵げ鐃緒申鐃獣て」￥申鐃楯人わ申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃緒申鐃瞬わ申鐃獣わ申鐃緒申鐃緒申鐃緒申鐃緒申"
    EXPECTED_CONTENT_ENCODED="<p>鐃緒申鐃緒申発表鐃緒申鐃曙た鐃瞬経営ワ申鐃宿バワ申鐃緒申鐃所ー鐃術逸申鐃緒申鐃竣につわ申鐃緒申鐃緒申鐃緒申鐃緒申鐃緒申鐃叔誌申鐃緒申鐃緒申鐃述べ￥申鐃楯人わ申鐃緒申鐃瞬まわ申鐃緒申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃緒申鐃遵げ鐃緒申鐃獣て」と居申鐃旬社逸申鐃緒申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃淑わ申鐃緒申鐃緒申</p>"
    if [[ "$TITLE" == $EXPECTED_TITLE ]]; then
        echo -e "\e[36m==euc-jp title\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36m==euc-jp title\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$TITLE
        echo -e "Expected:\n"$EXPECTED_TITLE
    fi
    if [[ "$CONTENT_ENCODED" == $EXPECTED_CONTENT_ENCODED ]]; then
        echo -e "\e[36m==euc-jp content encoded\e[0m - \e[1;32mOK\e[0m";
        echo ''
    else
        echo -e "\e[36m==euc-jp content encoded\e[0m - \e[1;31mWRONG\e[0m";
        echo -e "Actual:\n"$CONTENT_ENCODED
        echo -e "Expected:\n"$EXPECTED_CONTENT_ENCODED

    fi
else
  echo "BATCH euc-jp wrong answer, check $TMP_RESULT_FILE"
fi

cd /home/hce/hce-node-bundle/api/python/bin
echo "Launching of demo-form (2nd pass)"
# Execute the command BATCH - JSON request sent by demo-form through dc-client
./dc-client.py --config=../ini/dc-client.ini --command=BATCH --file=/home/hce/trunk/api/python/data/ftests/charsets/test_==euc-jp.batch.json > $TMP_RESULT_FILE
checkErrors

echo ''
CONTENT_GET=$(jq -r '.itemsList[0].itemObject[0].processedContents[0].buffer' $TMP_RESULT_FILE | base64 -di)
#echo $TMP_RESULT_FILE

#echo $CONTENT_GET

CONTENT_ENCODED=$(echo ${CONTENT_GET} | jq --raw-output '.[].content_encoded')
RIGHT_CONTENT="<p>鐃緒申鐃緒申発表鐃緒申鐃曙た鐃瞬経営ワ申鐃宿バワ申鐃緒申鐃所ー鐃術逸申鐃緒申鐃竣につわ申鐃緒申鐃緒申鐃緒申鐃緒申鐃緒申鐃叔誌申鐃緒申鐃緒申鐃述べ￥申鐃楯人わ申鐃緒申鐃瞬まわ申鐃緒申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃緒申鐃遵げ鐃緒申鐃獣て」と居申鐃旬社逸申鐃緒申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃淑わ申鐃緒申鐃緒申</p>"
TITLE=$(echo ${CONTENT_GET} | jq --raw-output '.[].title')
RIGHT_TITLE="鐃出わ申鐃俊￥申哲鐃竣「まわ申鐃述逸申鐃緒申鐃緒申鐃塾居申鐃緒申鐃遵げ鐃緒申鐃獣て」￥申鐃楯人わ申鐃峻駈申鐃緒申鐃緒申鐃緒申鐃緒申鐃瞬わ申鐃獣わ申鐃緒申鐃緒申鐃緒申鐃緒申"

# Check the presence of URL’s content
if [ -n "$CONTENT_GET" ]; then
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;32mSuccess\e[0m"
    echo 'The content of URL processed successfully and is not empty'
    echo ''
else
    echo -e "\e[36mCONTENT_GET\e[0m - \e[1;31mError\e[0m"
    echo ''

fi

# Check the character encoding of the processed content (particularly, content_encoded)
if [ "$CONTENT_ENCODED" = "$RIGHT_CONTENT" ]; then
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (content_encoded) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mСONTENT_ENCODED_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (content_encoded) in the demo-form: ==EUC-JP'
    echo ''
fi

# Check the character encoding of the processed content (particularly, title)
if [ "$TITLE" = "$RIGHT_TITLE" ]; then
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;32mSuccess\e[0m"
    echo 'Correct character encoding of the processed content (title) in the demo-form: UTF-8'
    echo ''
else
    echo -e "\e[36mTITLE_CHECK\e[0m - \e[1;31mError\e[0m"
    echo 'Wrong character encoding of the processed content (title) in the demo-form: ==EUC-JP'
    echo ''
fi
exit 0
