# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
DateTimeType Class content main functional extract of datetime.

@package: dc_processor
@file DateTimeType.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re
# import time
import copy
from datetime import tzinfo
import datetime
import unicodedata

# from dateutil.parser import parserinfo
# from dateutil.parser import parse
from dateutil.relativedelta import relativedelta
from dateutil.tz import gettz
from dateutil import parser

from app.Utils import getTracebackInfo

try:
  from app.Utils import ExceptionLog  # pylint: disable=E0401
except:  # pylint: disable=W0702
  ExceptionLog = None

import DateTimeTimezones  # pylint: disable=W0403



# # Class OffsetTzInfo for calculate offset
class OffsetTzInfo(tzinfo):
  # #Constructor
  def __init__(self, isNegative=False, hours=0, minutes=0):
    self.isNegative = isNegative
    self.hours = hours
    self.minutes = minutes
    super(OffsetTzInfo, self).__init__()

  # #Method utcoffset inheritor from interface of base class
  def utcoffset(self, dt):  # pylint: disable=W0613
    ret = datetime.timedelta(hours=self.hours, minutes=self.minutes)
    if self.isNegative:
      ret = (-1) * ret
    return ret

  # #Method dst inheritor from interface of base class
  def dst(self, dt):  # pylint: disable=W0613
    return datetime.timedelta(0)

  # #Method tzname inheritor from interface of base class
  def tzname(self, dt):  # pylint: disable=W0613
    ret = '{:%H:%M}'.format(datetime.time(hour=self.hours, minute=self.minutes))
    if self.isNegative:
      ret = '−' + ret
    else:
      ret = '+' + ret
    return "UTC" + ret


# # Class DateTimeType for extract data
#
class DateTimeType(object):
  # #Constans used in class
  MIN_ALLOWED_YEAR = 2000
  MIN_ALLOWED_LEN_FOR_DATEUTILS = 20
  ISO_SEP = ' '
  BAD_SIMBOLS = '=(),|@`'
  TAG_NAMES = ['pubdate', 'dc_date']
  MAX_SIZE_TIMEZONES_OFFSET = 6

  DEFAULT_VALUE_TIMEZONE_STRING = None

  LANG_ENG = "ENG"
  LANG_RUS = "RUS"
  LANG_UKR = "UKR"
  LANG_GER = "GERMAN"
  LANG_JAP = "JAPAN"
  LANG_RUS_BAD = "RUS"

  LANG_ARAB = "ARAB"
  LANG_ARAB_IRAQ = "ARAB"
  LANG_ARAB_EGYPT = "ARAB"
  LANG_ARAB_LIBIA = "ARAB"
  LANG_ARAB_LIBIA_SECOND = "ARAB"
  LANG_ARAB_ALGERIA = "ARAB"
  LANG_ARAB_MOROCCO = "ARAB"

  monthListEng = [u'Jan', u'Feb', u'Mar', u'Apr', u'May', u'Jun', u'Jul', u'Aug', u'Sep', u'Oct', u'Nov', u'Dec']
  monthListRus = [u'Янв', u'Февр', u'Март', u'Апр', u'Май', u'Июнь', u'Июль', u'Авг', u'Сент', u'Окт', u'Нояб', u'Дек']
  monthListUkr = [u'Сiч', u'Лют', u'Бер', u'Квiт', u'Трав', u'Черв', u'Лип', u'Серп', u'Вер', u'Жовт', u'Лист', u'Груд']
  monthListGer = [u'Jan', u'Feb', u'März', u'Apr', u'Mai', u'Juni', u'Juli', u'Aug', u'Sept', u'Okt', u'Nov', u'Dez']
  monthListJap = [u'一月', u'二月', u'三月', u'四月', u'五月', u'六月', u'七月', u'八月', u'九月', u'十月', u'十一月', u'十二月']
  monthListRusBad = [u'Янв', u'Февр', u'Март', u'Апр', u'Май', u'Июня', u'Июля', u'Авг', u'Сент', u'Окт', u'Нояб', u'Дек']

  # Iraq, Syria, Lebanon, Palestine and Jordan.
  monthListArabIraq = [u'كَانُون الثَّانِي', u'شُبَاط', u'آذَار', u'نَيْسَان', u'أَيَّار', u'حَزِيرَان', u'تَمُّوز', u'آب', \
                       u'أَيْلُول', u'تِشْرِين الْأَوَّل', u'تِشْرِين الثَّانِي', u'كَانُون الْأَوَّل']

  # Egypt, Sudan and the Persian Gulf states
  monthListArabEgypt = [u'يناير', u'فبراير', u'مارس', u'أبريل / إبريل', u'مايو', u'يونيو / يونية', \
                        u'يوليو / يولية', u'أغسطس', u'سبتمبر', u'أكتوبر', u'نوفمبر', u'ديسمبر']

  # Libyan Arab Jamahiriya
  monthListArabLibya = [u'أي النار', u'النوار', u'الربيع', u'الطير', u'الماء', u'الصيف', u'ناصر', u'هانيبال', u'الفاتح', u'الثمور', u'الحرث', u'الكانون']
  monthListArabLibyaSecond = [u'أي النار', u'النوار', u'الربيع', u'الطير', u'الماء', u'الصيف', u'ناصر', u'هانيبال', u'الفاتح', u'التمور', u'الحرث', u'الكانون']

  # Algeria and Tunisia
  monthListArabAlgeria = [u'جانفي', u'فيفري', u'مارس', u'أفريل', u'ماي', u'جوان', u'جويلية', u'أوت', u'سبتمبر', u'أكتوبر', u'نوفمبر', u'ديسمبر']

  # Morocco
  monthListArabMorocco = [u'يناير', u'فبراير', u'مارس', u'أبريل', u'ماي', u'يونيو', u'يوليوز', u'غشت', u'شتنبر', u'أكتوبر', u'نونبر', u'دجنبر']

  dayStateEng = [u'Today', u'Yesterday', u'Day before yesterday']
  dayStateRus = [u'Сегодня', u'Вчера', u'Поза вчера']
  dayStateUkr = [u'Сьогодні', u'Вчора', u'Позавчора']
  dayStateGer = [u'Heute', u'Gestern', u'Vorgestern']
  dayStateJap = [u'今日', u'イエスタデイ', u'おととい']
  dayStateRusStr = ['Сегодня', 'Вчера', 'Поза вчера']
  dayStateUkrStr = ['Сьогодні', 'Вчора', 'Позавчора']
  dayStateArab = []

  wordsListEng = [u'Jan', u'Feb', u'Mar', u'Apr', u'May', u'Jun', u'Jul', u'Aug', u'Sep', u'Oct', u'Nov', u'Dec', \
                  u'Year', u'Today', u'Yesterday', u'Day before yesterday', u'year', u'month', u'day', u'hour', \
                  u'minute']
  wordsListRus = [u'Янв', u'Февр', u'Мар', u'Апр', u'Май', u'Июнь', u'Июль', u'Авг', u'Сент', u'Окт', u'Нояб', \
                  u'Дек', u'Сегодня', u'Вчера', u'Поза вчера', u'Июня', u'Июля']
  wordsListUkr = [u'Сiч', u'Лют', u'Бер', u'Квiт', u'Трав', u'Черв', u'Лип', u'Серп', u'Вер', u'Жовт', u'Лист', \
                  u'Груд', u'Рік', u'Сьогодні', u'Вчора', u'Позавчора']
  wordsListGer = [u'März', u'Mai', u'Juni', u'Juli', u'Sept', u'Okt', u'Dez', u'Jahr', u'Heute', u'Gestern', \
                  u'Vorgestern', u'Uhr']
  wordsListJap = [u'一月', u'二月', u'三月', u'四月', u'五月', u'六月', u'七月', u'八月', u'九月', u'十月', u'十一月', \
                  u'十二月', u'年', u'今日', u'イエスタデイ', u'おととい', u'月', u'日', u'時', u'分', u'付', u'更新', \
                  u'春', u'夏', u'秋', u'冬', u'火', u'水', u'木', u'金', u'土']

  wordsListArab = monthListArabIraq + monthListArabEgypt + monthListArabLibya + monthListArabLibyaSecond + monthListArabAlgeria + monthListArabMorocco + \
                 [u'يَوم الأحَد', u'يَوم الإثنين', u'يَوم الثلاثاء', u'يَوم الأربعاء', u'يَوم الخميس', u'يَوم الجمعة', u'يَوم السبت', \
                  u'يوم', u'أيام', u'أسبوع', u'أسابيع', u'م']

  LANG_DICT = {LANG_ENG: wordsListEng, LANG_RUS: wordsListRus, LANG_UKR: wordsListUkr, LANG_GER: wordsListGer, \
               LANG_JAP: wordsListJap, LANG_ARAB: wordsListArab}

  MONTH_DICT = {LANG_ENG: monthListEng, LANG_RUS: monthListRus, LANG_UKR: monthListUkr, LANG_GER: monthListGer, \
               LANG_JAP: monthListJap, LANG_RUS_BAD: monthListRusBad, LANG_ARAB_IRAQ:monthListArabIraq , \
               LANG_ARAB_EGYPT:monthListArabEgypt, LANG_ARAB_LIBIA:monthListArabLibya, LANG_ARAB_LIBIA_SECOND:monthListArabLibyaSecond, \
               LANG_ARAB_ALGERIA:monthListArabAlgeria, LANG_ARAB_MOROCCO:monthListArabMorocco}

  DAY_STATE_DICT = {LANG_ENG: dayStateEng, LANG_RUS: dayStateRus, LANG_UKR: dayStateUkr, LANG_GER: dayStateGer, \
               LANG_JAP: dayStateJap, LANG_RUS: dayStateRusStr, LANG_UKR: dayStateUkrStr, LANG_ARAB: dayStateArab}

  patternListDate = [r'\W+ (?P<day>\d{1,2}) (?P<mon>\d{1,2}) (?P<year>\d{4})',
                     r'(?P<day>[0123][0-9])-(?P<mon>[0-1][0-9])-(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>[ 1][0-9]) (?P<day>[ 0123][0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\d{1,2})/(?P<day>[ 0123][0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\d{1,2})/(?P<day>[0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\d{1,2})-(?P<day>[0-9][0-9])-(?P<year>[0-9][0-9][0-9][0-9])',
                     r' (?P<mon>\d{1,2})/(?P<day>[0123][0-9])/(?P<short_year>[0-9][0-9])',
                     r' (?P<mon>\d{1,2})/(?P<day>[0-9])/(?P<short_year>[0-9][0-9])',
                     r'(?P<mon>[A-Z][a-z][a-z]) (?P<day>[ 0123][0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>[A-Z][a-z][a-z]) (?P<day>[0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[ 0123][0-9]) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[0-9])(\w{2}) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>[ 0123][0-9])(\w{2}) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\W+) (?P<day>\d{2})(\w{2}) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>\d{1,2})(\W+\d{1,2}) (?P<year>\d{4})',
                     r'(?P<mon>\w+). (?P<day>\d{1,2}) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\W+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9]) (?P<mon>[0-1][0-9]) (?P<day>[0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])-(?P<mon>[0-1][0-9])-(?P<day>[0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])/(?P<mon>[0-1][0-9])/(?P<day>[0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])/(?P<mon>[1-9])/(?P<day>[0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])/(?P<mon>[1-9])/(?P<day>[1-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])\.(?P<mon>[0-1][0-9])\.(?P<day>[0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])\.(?P<mon>[1-9])\.(?P<day>[0123][0-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])\.(?P<mon>[0-1][0-9])\.(?P<day>[1-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])\.(?P<mon>[1-9])\.(?P<day>[1-9])',
                     r'(?P<year>[0-9][0-9][0-9][0-9])(?P<mon>[0-1][0-9])(?P<day>[0123][0-9])',
                     r'(?P<day>[0-9][0-9])(\w{2}) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9])(\w{2}) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9]) (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9]) (?P<mon>\W+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9])/(?P<mon>[0-9][0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9]).(?P<mon>[0-9][0-9]).(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{2})/(?P<mon>[0-9][0-9])/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{2})/(?P<mon>\d+})/(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{1,2})\.(?P<mon>\d{1,2})\.(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\d{1,2})\.(?P<day>\d{1,2})\.(?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{1,2})\. (?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>\d{1,2})\. (?P<mon>\W+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\w+)',
                     r'(?P<day>[0-9][0-9]) (?P<mon>\W+)',
                     r'(?P<day>[0-9]) (?P<mon>\w+)',
                     r'(?P<day>[0-9]) (?P<mon>\W+)',
                     r'(?P<mon>\d{1,2})\.(?P<day>[0123][0-9])\.(?P<short_year>[0-9][0-9])',
                     r'(?P<mon>\d{1,2})\.(?P<day>[0-9])\.(?P<short_year>[0-9][0-9])',
                     r'(?P<day>[0123][0-9])\.(?P<mon>[01][0-9])\.(?P<short_year>[0-9][0-9])',
                     r'(?P<mon>\w+) (?P<year>[0-9][0-9][0-9][0-9])',
                     r'(?P<mon>\w+) (?P<day>\d{1,2})',
                     r'(?P<mon>\W+) (?P<day>\d{1,2})']

  patternListTime = [r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2}) (?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2}) (?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2})(?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>\d{2})(?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}) (?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}) (?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2})(?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2})(?P<tf>[PpAaMm]{2})',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}):(?P<sec>[0-9][0-9])',
                     r'(?P<hour>[ 0-9][0-9])(?P<min>\d{2}) GMT',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{2}) GMT',
                     r'(?P<hour>[ 0-9][0-9]):(?P<min>\d{1,2})',
                     r'(?P<hour>[ 0-9][0-9]) (?P<tf>[PpAaMm]{2})']

  patternListTimezoneOffset = [r'(?P<offset>[+-][0-9][0-9]:[0-9][0-9])',
                               r'(?P<offset>[+-][0-9][0-9][0-9][0-9])',
                               r'(?P<offset>[+-]\d{1,2})',
                               r'(?P<offset>[−][0-9][0-9])']

  patternListUtcTimezones = [r'(?P<hours>[0-9][0-9]):(?P<minutes>[0-9][0-9])',
                             r'(?P<hours>[0-9][0-9])(?P<minutes>[0-9][0-9])',
                             r'(?P<hours>[0-9][0-9])']

  japaneseEras = {r'平成': 1989, r'令和': 2019}

  patternListJapanese = [
    # 6/17(日) 12:24配信
    r'(?P<mon>\d{1,2})/(?P<day>\d{1,2}).*(?P<hour>\d{2}):(?P<min>\d{2})',
    # 8月20日 20:01
    r'(?P<mon>\d{1,2})月(?P<day>\d{1,2})日.*(?P<hour>\d{2}):(?P<min>\d{2})',
    # 2019年04月20日 01:45
    r'(?P<year>\d{1,4})年(?P<mon>\d{1,2})月(?P<day>\d{1,2})日.*(?P<hour>\d{2})時(?P<min>\d{2})分',
    # 9月3日 11時41分
    r'(?P<mon>\d{1,2})月(?P<day>\d{1,2})日.*(?P<hour>\d{2})時(?P<min>\d{2})分',
    # (2014/6/3 20:13更新)
    r'(?P<year>\d{1,4})/(?P<mon>\d{1,2})/(?P<day>\d{1,2}).*(?P<hour>\d{2}):(?P<min>\d{2})',
    # 2019.4.15（月）
    r'(?P<year>\d{1,4}).(?P<mon>\d{1,2}).(?P<day>\d{1,2}).*（月）',
    # 2014/8/14付
    r'(?P<year>\d{1,4})/(?P<mon>\d{1,2})/(?P<day>\d{1,2})',
    # 2014年8月7日
    r'(?P<year>\d{1,4})年(?P<mon>\d{1,2})月(?P<day>\d{1,2})日',
    # 8月20日
    r'(?P<mon>\d{1,2})月(?P<day>\d{1,2})日']

  # Exclude list of langs for parse by dateutil
  excludeDateutilList = [LANG_ARAB]

  # #Constant of error messages
  ERROR_INPUT_PARAMS = 'Error initialization by input parameters. %s '
  ERROR_FORMAT_TYPE = 'Format has wrong type: %s'
  ERROR_DATA_STRING_TYPE = 'Data string is not string.'

  # #Constructor
  #
  # @param dataString - data string for extract datatime, can be as timestamp or string with data for extract
  # @param formatObj - format for formatting data
  # @param continueIfNotDetected - continue if not detected by format
  # @param log - logger instance
  def __init__(self, dataString=None, formatObj=None, continueIfNotDetected=True, log=None):
    self.datetime = None
    self.isError = False
    self.errorMsg = ''

    try:
      # validate of input type of data string
      if dataString is not None and not isinstance(dataString, basestring):
        raise Exception(self.ERROR_DATA_STRING_TYPE)

      # validate of input type of format
      if formatObj is not None and not isinstance(formatObj, basestring) and not isinstance(formatObj, list):
        raise Exception(self.ERROR_FORMAT_TYPE % str(type(formatObj)))

      self.datetime = self.__initDataTime(dataString, formatObj, continueIfNotDetected, log)

    except Exception, err:
      self.errorMsg = self.ERROR_INPUT_PARAMS % str(err)
      self.isError = True


  # # initialization of datatime
  #
  # @param dataString - data string for extract datatime, can be as timestamp or string with data for extract
  # @param formatObj - format for formatting data
  # @param continueIfNotDetected - continue if not detected by format
  # @param log - logger instance
  # @return datatime - extracted datatime or None
  def __initDataTime(self, dataString=None, formatObj=None, continueIfNotDetected=True, log=None):
    # variable for result
    ret = None

    if log is not None and dataString is not None:
      log.debug("dataString: '%s'", str(dataString))

    if log is not None and formatObj is not None:
      log.debug("formatObj: '%s'", str(formatObj))

    if dataString is not None and isinstance(dataString, int):
      ret = datetime.datetime.fromtimestamp(dataString)
    elif dataString is not None and isinstance(dataString, basestring) and dataString.isdigit():
      ret = datetime.datetime.fromtimestamp(int(dataString))
    elif dataString is not None and formatObj is not None:

      if isinstance(formatObj, basestring):
        ret = self.extractByFormatSafe(dataString, formatObj, log)
      elif isinstance(formatObj, list):
        for formatValue in formatObj:
          ret = self.extractByFormatSafe(dataString, formatValue, log)
          if isinstance(ret, datetime.datetime):
            break

      if log is not None and format is not None:
        log.debug("continueIfNotDetected: '%s'", str(continueIfNotDetected))
        log.debug("ret: '%s'", str(ret))

      if ret is None and continueIfNotDetected:
        ret = self.parse(dataString=dataString, useCurrentYear=True, logger=log, isExtendLog=True)

    return ret


  ## extract by format safe
  #
  # @param dataString - data string for extract datatime
  # @param formatObj - format for formatting data
  # @param log - logger instance
  # @return extracted datetime instance or otherwise None
  @staticmethod
  def extractByFormatSafe(dataString, formatObj, log=None):
    #variable for result
    ret = None
    try:
      if not isinstance(dataString, basestring):
        raise Exception("Wrong type of input data variable: %s", str(type(dataString)))

      if not isinstance(formatObj, basestring):
        raise Exception("Wrong type of input format variable: %s", str(type(formatObj)))

      if dataString == "":
        raise Exception("Empty input data variable.")

      if formatObj == "":
        raise Exception("Empty input format variable.")

      if formatObj[0] == '/':
        # regular expression
        rawDataStringsList = re.findall(pattern=formatObj[1:], string=dataString, flags=re.U + re.I + re.M)
        for rawDataString in rawDataStringsList:
          ret = DateTimeType.parse(rawDataString, True, log, True)
          if ret is not None:
            break

      else:
        ret = datetime.datetime.strptime(unicodedata.normalize('NFKD', dataString.decode('utf-8', 'ignore')), formatObj)

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # Return datatime as timestamp
  #
  # @param - None
  # @return datatime as timestamp
  def getInt(self):
    # variable for result
    ret = None
    try:
      ret = int((self.datetime - datetime.datetime.fromtimestamp(0)).total_seconds())
    except Exception, err:
      self.isError = True
      self.errorMsg = str(err)
      ret = None

    return ret


  # # Return datatime as string
  #
  # @param - formatString - format string for formatting data
  # @return datatime as timestamp
  def getString(self, formatString=None):
    # variable for result
    ret = None

    try:
      if formatString is None:
        ret = self.datetime.isoformat(self.ISO_SEP)
      else:
        if not isinstance(formatString, basestring):
          raise Exception(self.ERROR_FORMAT_TYPE % str(type(formatString)))
        else:
          ret = self.datetime.strftime(formatString)
    except Exception, err:
      self.isError = True
      self.errorMsg = str(err)
      ret = None

    return ret


  # # Intendification of lang
  #
  # @param inputStr - string for detection of lang
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return constant of lang name
  @staticmethod
  def getLang(inputStr, logger=None, isExtendLog=False):
    # variable for result
    ret = None
    found = False
    langDict = DateTimeType.LANG_DICT
    try:
      dataString = inputStr.decode('utf-8', 'ignore')

      for key in langDict.keys():
        for word in langDict[key]:
          if dataString.lower().find(word) > -1 or dataString.lower().find(word.lower()) > -1:
            ret = key
            found = True
            break
        if found:
          break

    except Exception, err:
      ret = DateTimeType.LANG_ENG
      if logger and isExtendLog:
        logger.debug("getLang: '" + str(err) + "'")
        logger.info(getTracebackInfo())

    return ret


  # # Intendification of lang
  #
  # @param inputStr - string for detection of lang
  # @param logger - logger instance
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return constant of month number if found
  @staticmethod
  def getMonthNumber(inputStr, logger, isExtendLog=False):
    if logger and isExtendLog:
      logger.debug("getMonthNumber inputStr: '" + inputStr + "' type: " + str(type(inputStr)))

    ret = None
    inputStr = inputStr.strip()

    if len(inputStr) < 3 and inputStr.isdigit():
      ret = int(inputStr)
    else:
      found = False
      monthDict = DateTimeType.MONTH_DICT

      for key in monthDict.keys():
        # if logger and isExtendLog:
        #  logger.debug("key: '" + str(key) + "'")

        monthNumber = 0
        for months in monthDict[key]:
          monthNumber = monthNumber + 1
          month = months.lower()
          inputMonth = inputStr
          try:
            inputMonth = inputStr.decode('utf-8')
          except UnicodeError, err:
            if logger is not None and isExtendLog:
              logger.debug("Operation decode'utf-8' has error: " + str(err))

          # if logger and isExtendLog:
          #  logger.debug(inputMonth.lower() + ' <=> ' + month)

          if inputMonth.lower().find(month) > -1 or month.lower().find(inputMonth.lower()) > -1:
            ret = monthNumber
            found = True
            break
        if found:
          break

    return ret


  # # Extract english date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateEng(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, isExtendLog, DateTimeType.LANG_ENG)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger, isExtendLog)

    return pubdate


  # # Extract russian date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateRus(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, isExtendLog, DateTimeType.LANG_RUS)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger, isExtendLog)

    return pubdate


  # # Extract ukrainian date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateUkr(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, isExtendLog, DateTimeType.LANG_UKR)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger, isExtendLog)

    return pubdate


  # # Extract german date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateGerman(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, isExtendLog, DateTimeType.LANG_GER)
    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger, isExtendLog)

    return pubdate


  # # Extract japanese date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateJapan(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    # replace japanise simbols
    inputStr = DateTimeType.replaceJapanSimbols(inputStr, logger, isExtendLog)

    # extract data
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, isExtendLog, DateTimeType.LANG_JAP)

    if pubdate is None:
      pubdate = DateTimeType.adjustJapaneseDate(inputStr, useCurrentYear, logger, isExtendLog)

    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger, isExtendLog)

    return pubdate


  # # Extract arabic date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateArab(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    # extract data
    pubdate = DateTimeType.intelligentExtractor(inputStr, useCurrentYear, logger, isExtendLog, DateTimeType.LANG_ARAB)

    if pubdate is None:
      pubdate = DateTimeType.adjustArabDate(inputStr, useCurrentYear, logger, isExtendLog)

    if pubdate is None:
      pubdate = DateTimeType.extractDateCommon(inputStr, useCurrentYear, logger, isExtendLog)

    return pubdate


  # Adjust japanese date
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datetime or None
  @staticmethod
  def adjustJapaneseDate(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    # variable for result
    ret = None
    try:
      if logger and isExtendLog:
        logger.debug("Input string: '%s'", str(inputStr))

      rawPubdate = DateTimeType.convertJapaneseEras(inputStr, logger, isExtendLog)

      for pattern in DateTimeType.patternListJapanese:
        match = re.search(pattern=pattern, string=rawPubdate, flags=re.U + re.I)
        if match is not None:
          year, month, day = DateTimeType.extractDate(rawPubdate, useCurrentYear, logger, False, DateTimeType.patternListJapanese)
          hour, minute, second, tf = DateTimeType.extractTime(rawPubdate, logger, False, DateTimeType.patternListJapanese)  # pylint: disable=W0612

          ret = datetime.datetime(year, month, day, hour, minute, second)
          break

    except Exception, err:
      if logger is not None and isExtendLog:
        logger.error(str(err))

    return ret


  # Adjust arabic date
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datetime or None
  @staticmethod
  def adjustArabDate(inputStr, useCurrentYear, logger=None, isExtendLog=False):
    rawPubdate = inputStr
    try:
      if logger is not None and isExtendLog:
        logger.info("Before 'replace' rawPubdate: %s", str(rawPubdate))

      if not DateTimeType.isUtf8CodePage(rawPubdate, logger, isExtendLog):
        rawPubdate = DateTimeType.changeCodePageToAscii(rawPubdate, logger, isExtendLog)

# #       rawPubdate = re.sub(pattern=r'م', repl='PM', string=rawPubdate, flags=re.U + re.I + re.M)
      rawPubdate = rawPubdate.replace('م', 'PM')

      if logger is not None and isExtendLog:
        logger.info("After 'replace' rawPubdate: %s", str(rawPubdate))

    except Exception, err:
      if logger is not None and isExtendLog:
        logger.error("Operation 'replace' has error %s", str(err))
        logger.info(getTracebackInfo())

    return DateTimeType.extractDateCommon(rawPubdate, useCurrentYear, logger, False)


  # # Exctract date from string by dateutil parser
  #
  # @param dataString - string for parse
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @param lang - language name
  # @return extracted datetime or None
  @staticmethod
  def extractUseDateutil(dataString, logger=None, isExtendLog=False, lang=None):
    # variable for result
    ret = None
    try:
      if logger is not None and isExtendLog:
        logger.debug("inputStr: %s", str(dataString))

      if lang is None or lang in DateTimeType.excludeDateutilList:
        raise Exception("Usage 'dateutil' skipped by language: '%s'" % str(lang))

      if len(dataString) < int(DateTimeType.MIN_ALLOWED_LEN_FOR_DATEUTILS):
        raise Exception("Usage 'dateutil' skipped by  min allowed len = %s" % str(len(dataString)))

#       if not DateTimeType.isUtf8CodePage(dataString, logger, isExtendLog):
#         dataString = DateTimeType.changeCodePageToAscii(dataString, logger, isExtendLog)

      try:
        if logger is not None and isExtendLog:
          logger.debug("try use 'dateutil'")

        ret = parser.parse(dataString)
        if ret is not None:
          if logger is not None and isExtendLog:
            logger.debug("'dateutil' return: " + str(ret.isoformat(DateTimeType.ISO_SEP)))
          ret = ret.replace(microsecond=0)

      except Exception, err:
        if logger is not None and isExtendLog:
          logger.debug("'dateutil' can not parse: " + str(err))
        try:
          normalizedString = DateTimeType.normalizeTimezone(dataString, logger, isExtendLog)
          if dataString != normalizedString:
            if logger is not None and isExtendLog:
              logger.debug("retry parsing use 'dateutil'")
            ret = parser.parse(normalizedString)
            if ret is not None and logger is not None and isExtendLog:
              logger.debug("'dateutil' return: " + str(ret.isoformat(DateTimeType.ISO_SEP)))
        except Exception, err:
          if logger is not None and isExtendLog:
            logger.debug("'dateutil' can not parse: " + str(err))

    except Exception, err:
      if logger is not None and isExtendLog:
        logger.debug(str(err))

    return ret


  # # static method for parse
  #
  # @param dataString - string for parse
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datetime or None
  @staticmethod
  def parse(dataString, useCurrentYear=True, logger=None, isExtendLog=False):
    # variable for result
    ret = None
    if logger is not None and isExtendLog:
      logger.debug("inputStr: '" + dataString + "'")

    if DateTimeType.isAllowedInputString(dataString, logger, isExtendLog):
#       if len(dataString) >= int(DateTimeType.MIN_ALLOWED_LEN_FOR_DATEUTILS):
#         try:
#           if not DateTimeType.isUtf8CodePage(dataString, logger, isExtendLog):
#             dataString = DateTimeType.changeCodePageToAscii(dataString, logger, isExtendLog)
#
#           if logger is not None and isExtendLog:
#             logger.debug("try use 'dateutil'")
#
#           ret = parser.parse(dataString)
#           if ret is not None:
#             if logger is not None and isExtendLog:
#               logger.debug("'dateutil' return: " + str(ret.isoformat(DateTimeType.ISO_SEP)))
#             ret = ret.replace(microsecond=0)
#         except Exception, err:  # pylint: disable=W0702
#           if logger is not None and isExtendLog:
#             logger.debug("'dateutil' can not parse: " + str(err))
#           try:
#             normalizedString = DateTimeType.normalizeTimezone(dataString, logger, isExtendLog)
#             if dataString != normalizedString:
#               if logger is not None and isExtendLog:
#                 logger.debug("retry parsing use 'dateutil'")
#               ret = parser.parse(normalizedString)
#               if ret is not None and logger is not None and isExtendLog:
#                 logger.debug("'dateutil' return: " + str(ret.isoformat(DateTimeType.ISO_SEP)))
#           except Exception, err:
#             if logger is not None and isExtendLog:
#               logger.debug("'dateutil' can not parse: " + str(err))

      if not DateTimeType.isUtf8CodePage(dataString, logger, isExtendLog):
        dataString = DateTimeType.changeCodePageToAscii(dataString, logger, isExtendLog)

      # Identification of language
      langType = DateTimeType.getLang(dataString, logger, isExtendLog)

      if logger is not None and isExtendLog:
        logger.debug('language type detected as: ' + str(langType))

      ret = DateTimeType.extractUseDateutil(dataString, logger, isExtendLog, lang=langType)

      if ret is None:
        if langType == DateTimeType.LANG_ENG:
          # extract english date
          ret = DateTimeType.extractDateEng(dataString, useCurrentYear, logger, isExtendLog)
        elif langType == DateTimeType.LANG_RUS:
          # extract russian date
          ret = DateTimeType.extractDateRus(dataString, useCurrentYear, logger, isExtendLog)
        elif langType == DateTimeType.LANG_UKR:
          # extract ukrainian date
          ret = DateTimeType.extractDateUkr(dataString, useCurrentYear, logger, isExtendLog)
        elif langType == DateTimeType.LANG_GER:
          # extract germany date
          ret = DateTimeType.extractDateGerman(dataString, useCurrentYear, logger, isExtendLog)
        elif langType == DateTimeType.LANG_JAP:
          # extract japanese date
          ret = DateTimeType.extractDateJapan(dataString, useCurrentYear, logger, isExtendLog)
        elif langType == DateTimeType.LANG_ARAB:
          # extract arabic date
          ret = DateTimeType.extractDateArab(dataString, useCurrentYear, logger, isExtendLog)
        else:
          ret = DateTimeType.extractDateCommon(dataString, useCurrentYear, logger, isExtendLog)

    if ret is not None and ret.tzinfo is None:
      timezoneName = DateTimeType.extractUtcTimezoneName(dataString, logger, isExtendLog)
      utcZone = gettz(timezoneName)
      if logger is not None and isExtendLog:
        logger.debug("utcZone: " + str(utcZone))
        logger.debug("timezoneName: " + str(timezoneName))
      if utcZone is not None:
        ret = ret.replace(tzinfo=utcZone)
      else:
        utcOffset = DateTimeType.extractUtcOffset(dataString, logger, isExtendLog=True)
        if logger is not None and isExtendLog:
          logger.debug("utcOffset = %s", str(utcOffset))

        if utcOffset is not None:
          ret = DateTimeType.applyUtcTimezone(ret, utcOffset, DateTimeTimezones.timezonesDict, logger, isExtendLog=False)

    if logger is not None and isExtendLog:
      if ret is not None:
        logger.debug('result pubdate: ' + str(ret.isoformat(DateTimeType.ISO_SEP)))
      else:
        logger.debug('result pubdate: NONE')

    return ret


  # # Apply UTC timezone use tzInfo to datetime object
  #
  # @param dt - datetime instance
  # @param tzName - name of timezone
  # @param timezonesDict - dictionary with timezones
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return datetime instance, already modified if success
  @staticmethod
  def applyUtcTimezone(dt, tzName, timezonesDict=DateTimeTimezones.timezonesDict, logger=None, isExtendLog=False):  # pylint: disable=W0102
    if logger is not None and isExtendLog:
      logger.debug("applyUtcTimezone enter ...")

    if dt is not None and isinstance(tzName, basestring):
      rawOffset = None
      if tzName in timezonesDict and len(timezonesDict[tzName]) > 1:
        rawOffset = timezonesDict[tzName][1]
      elif isinstance(tzName, basestring):
        rawOffset = tzName

      if isinstance(rawOffset, basestring):
        isNegative = False
        if '−' in rawOffset or '-' in rawOffset:
          isNegative = True
        if logger is not None and isExtendLog:
          logger.debug("isNegative: " + str(isNegative))

        for pattern in DateTimeType.patternListUtcTimezones:
          match = re.search(pattern, rawOffset)
          if match:
            hours = 0
            if 'hours' in match.groupdict():
              hours = int(match.groupdict()['hours'])

            minutes = 0
            if 'minutes' in match.groupdict():
              minutes = int(match.groupdict()['minutes'])

            if logger is not None and isExtendLog:
              logger.debug("hours: " + str(hours) + " minutes: " + str(minutes))

            tzInfo = OffsetTzInfo(isNegative, hours, minutes)
            dt = dt.replace(tzinfo=tzInfo)
            if logger is not None and isExtendLog:
              logger.debug("tzname: " + str(dt.tzname()))
            break

    return dt


  # #Split datetime and timezone string
  #
  # @param dt - datetime instance
  # @return datetime instance without tzInfo and timezone string
  @staticmethod
  def split(dt):
    timezone = None
    if dt is not None:
      timezone = dt.strftime('%z')
      dt = dt.replace(tzinfo=None)
      if isinstance(timezone, basestring) and timezone == "":
        timezone = DateTimeType.DEFAULT_VALUE_TIMEZONE_STRING

    return dt, timezone


  # #Get timezone string
  #
  # @param dt - datetime instance
  # @return timezone string
  @staticmethod
  def getTimezone(dt):
    timezone = ''
    if dt is not None:
      timezone = dt.strftime('%z')

    return timezone


  # # Check is allowed input string for next procedure of parse
  #
  # @param dataString - string for parse
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return True if allowed or False otherwise
  @staticmethod
  def isAllowedInputString(dataString, logger=None, isExtendLog=False):
    # variable for result
    ret = True
    if dataString is None or not isinstance(dataString, basestring):
      ret = False
    elif dataString != "" and dataString.isupper() and dataString.isalnum() and \
      not dataString.isalpha():
      ret = False
      if logger and isExtendLog:
        logger.debug('input string has not allowed format')

    return ret


  # # Prepare input string (remove all bad simbols)
  #
  # @param inputStr - input string for preparation
  # @return already prepared string
  @staticmethod
  def prepareString(inputStr):
    ret = inputStr

    for tagName in DateTimeType.TAG_NAMES:
      if inputStr.lower().find('%' + tagName + '%') > -1:
        ret = ret.replace('%' + tagName + '%', '')
      else:
        pass

    for bad in DateTimeType.BAD_SIMBOLS:
      ret = ret.replace(bad, ' ')

    ret = ret.replace('  ', ' ')

    return ret


  # # Extract year, month and day from string
  #
  # @param inputStr - input string for extract date
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @param patternList - pattern list
  # @return year, month, day - trancated input string
  @staticmethod
  def extractDate(inputStr, useCurrentYear=True, logger=None, isExtendLog=False, patternList=None):
    # variables for results
    resMonth = 0
    resDay = 0
    resYear = 0

    resultDatesList = []
    try:
      for pattern in DateTimeType.patternListDate if patternList is None else patternList:
        month = 0
        day = 0
        year = 0

        match = re.search(pattern, inputStr)
        if logger and isExtendLog:
          logger.debug('match: ' + str(match) + ' pattern: ' + str(pattern))

        if match:
          if logger and isExtendLog:
            logger.debug('match.groupdict(): ' + str(match.groupdict()))

          if 'short_year' in match.groupdict():
            year = int(match.groupdict()['short_year']) + int(datetime.date.today().year // 1000 * 1000)
            if year > int(datetime.date.today().year):
              year = 0

          if 'year' in match.groupdict():
            year = match.groupdict()['year']

          if 'mon' in match.groupdict():
            month = match.groupdict()['mon']
            if logger and isExtendLog:
              logger.debug('month: ' + month)
            if month.isdigit() and int(month) > 12:
              if logger and isExtendLog:
                logger.debug('Bad month (' + str(month) + ') scipped!!!')
              continue

          if 'day' in match.groupdict():
            day = match.groupdict()['day']

          if logger and isExtendLog:
            logger.debug('month = ' + month)

          monthNumber = DateTimeType.getMonthNumber(month, logger, isExtendLog)

          if logger and isExtendLog:
            logger.debug('monthNumber = ' + str(monthNumber))

          if monthNumber is not None:
            month = monthNumber
          else:
            month = day = year = 0

          if logger and isExtendLog:
            logger.debug('year: ' + str(year) + ' month: ' + str(month) + ' day: ' + str(day))
            logger.debug("useCurrentYear = %s", str(useCurrentYear))

          if useCurrentYear:
            d = datetime.date.today()

            if year == 0 and month and day:
              year = d.year

            if year and month and day == 0:
              day = d.day

            if logger and isExtendLog:
              logger.debug('year: ' + str(year) + ' month: ' + str(month) + ' day: ' + str(day))

          if int(year) > 0 and int(month) <= 12 and int(day) <= 31:
            if int(day) <= 12:
              try:
                resultDatesList.append(datetime.date(int(year), int(month), int(day)))
              except Exception, err:
                if logger and isExtendLog:
                  logger.debug("Extracted wrong date. Skipped by reason: %s", str(err))
            else:
              resMonth = month
              resDay = day
              resYear = year
              break

          elif logger and isExtendLog:
            logger.debug("Extracted wrong date range. Skipped...")

      if logger and isExtendLog:
        logger.debug("resultDatesList: %s", str(resultDatesList))
        logger.debug("resYear = %s, resMonth = %s, resDay = %s", str(resYear), str(resMonth), str(resDay))

      if len(resultDatesList) > 0:
#         datesList = sorted(resultDatesList, reverse=True)
        datesList = resultDatesList
        if logger and isExtendLog:
          logger.debug("datesList: %s", str(datesList))

        resMonth = datesList[0].month
        resDay = datesList[0].day
        resYear = datesList[0].year

      if logger and isExtendLog:
        logger.debug("resYear = %s, resMonth = %s, resDay = %s", str(resYear), str(resMonth), str(resDay))

    except Exception, err:
      if logger and isExtendLog:
        logger.debug("inputStr: '" + str(inputStr) + "'")
        if logger and ExceptionLog is not None:
          ExceptionLog.handler(logger, err, 'extractDate:', (), {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        elif logger:
          logger.debug('extractDate:' + str(err))

    return int(resYear), int(resMonth), int(resDay)


  # # Extract hour, minute and second from string
  #
  # @param inputStr - input string for extract time
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @param patternList - pattern list
  # @return hour, minute, second and tf (time format)
  @staticmethod
  def extractTime(inputStr, logger=None, isExtendLog=False, patternList=None):  # pylint: disable=W0613

    hour = 0
    minute = 0
    second = 0
    tf = ''

    if logger and isExtendLog:
      logger.debug("inputStr: '" + str(inputStr) + "'")

    try:
      for pattern in DateTimeType.patternListTime if patternList is None else patternList:
        match = re.search(pattern, inputStr)
        if logger and isExtendLog:
          logger.debug('pattern: ' + str(pattern))
        if match:
          if logger and isExtendLog:
            logger.debug('match.groupdict(): ' + str(match.groupdict()))

          if 'hour' in match.groupdict():
            hour = match.groupdict()['hour']

          if 'min' in match.groupdict():
            minute = match.groupdict()['min']

          if 'sec' in match.groupdict():
            second = match.groupdict()['sec']

          if 'tf' in match.groupdict():
            tf = match.groupdict()['tf']

          break

    except Exception, err:
      if logger and isExtendLog:
        logger.debug("inputStr: '" + str(inputStr) + "'")
        if logger and ExceptionLog is not None:
          ExceptionLog.handler(logger, err, 'extractTime:', (), {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        elif logger:
          logger.debug('extractTime:' + str(err))

    return int(hour), int(minute), int(second), tf


  # # Exctract common date from string
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractDateCommon(inputStr, useCurrentYear=True, logger=None, isExtendLog=False):
    # variable for results
    ret = None
    try:
      localStr = DateTimeType.prepareString(inputStr)

      if localStr:
        match = re.search(r'\d{10}', localStr)
        if match:
          ret = datetime.datetime.fromtimestamp(int(match.group()))

          locCurrentTime = datetime.datetime.now()
          utcCurrentTime = datetime.datetime.utcnow()
          tmDelta = locCurrentTime - utcCurrentTime

          isNegative = bool(tmDelta.total_seconds() < 0)
          hours = abs(locCurrentTime.hour - utcCurrentTime.hour)
          minutes = abs(locCurrentTime.minute - utcCurrentTime.minute)
          if logger is not None and isExtendLog:
            logger.debug("isNegative: " + str(isNegative) + " hours: " + str(hours) + " minutes: " + str(minutes))

          # Correct datetime value to GMT
          if isNegative:
            ret = ret + datetime.timedelta(hours=hours, minutes=minutes)
          else:
            ret = ret - datetime.timedelta(hours=hours, minutes=minutes)

          # Apply tzInfo
          # tzInfo = OffsetTzInfo(isNegative, hours, minutes)
          tzInfo = OffsetTzInfo(False, 0, 0)
          ret = ret.replace(tzinfo=tzInfo)
          if logger is not None and isExtendLog:
            logger.debug("tzname: " + str(ret.tzname()))
        else:
          year, month, day = DateTimeType.extractDate(localStr, useCurrentYear, logger, isExtendLog=False)
          if logger and isExtendLog:
            logger.debug('year: ' + str(year) + '\tmonth: ' + str(month) + '\tday: ' + str(day))

          hour, minute, second, tf = DateTimeType.extractTime(localStr, logger, isExtendLog=False)
          if logger and isExtendLog:
            logger.debug('hour: ' + str(hour) + '\tminute: ' + str(minute) + '\tsecond: ' + str(second) + '\ttf: ' + str(tf))

          hour, minute = DateTimeType.checkTimeFormat(hour, minute, tf, logger, isExtendLog)

          if logger is not None and isExtendLog:
            logger.debug('hour: ' + str(hour) + '\tminute: ' + str(minute))

          if year and month and day:
            if useCurrentYear:
              now = datetime.datetime.utcnow()
              if month == now.month and day == now.day:
                if hour == 0 and minute == 0:
                  hour = int(now.hour)
                  minute = int(now.minute)
                  second = now.second

            ret = datetime.datetime(year, month, day, hour, minute, second, tzinfo=None)
          elif useCurrentYear and (year + month + day) == 0 and hour > 0 and minute > 0:
            d = datetime.datetime.today()
            year = d.year
            month = d.month
            day = d.day

            if int(d.hour) < int(hour):
              day = (d - datetime.timedelta(days=1)).day

            ret = datetime.datetime(year, month, day, hour, minute, second)

    except Exception, err:
      if logger is not None and isExtendLog:
        logger.debug("inputStr: '" + str(inputStr) + "'")
        if logger and ExceptionLog is not None:
          ExceptionLog.handler(logger, err, 'extractDateCommon:', (inputStr), {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        elif logger is not None:
          logger.debug('extractDateCommon:' + str(err))

    return ret


  # # Checking is exist offset from PM time format
  #
  # @param hour - hour value
  # @param minute - minute value
  # @param tf - time format (can be 'AM', 'PM' or other)
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return day, hour - result variables after change if necessary
  @staticmethod
  def checkTimeFormat(hour, minute, tf, logger=None, isExtendLog=False):
    if logger is not None and isExtendLog:
      logger.debug("tf: '%s'", str(tf))
      if tf is not None:
        logger.debug("find = %s", str(tf.lower().find('p')))

    if tf:
      if tf.lower().find('p') > -1:  # found 'PM'
        if int(hour) == 12 and int(minute) == 0:  # # 12:00 PM -> 12:00
          pass
        elif int(hour) >= 0 and int(hour) < 12 and int(minute) >= 0:  # # 00:01 PM -> 12:01
          hour += 12

      else:
        if int(hour) == 12 and int(minute) == 0:  # # 12:00 AM -> 00:00
          hour = 0
        elif int(hour) >= 0 and int(hour) < 12 and int(minute) >= 0:  # # 00:01 AM -> 00:01
          pass

    return hour, minute


  # # Extract date from string used state of day ('Today', 'Yesterday', 'Day before yesterday')
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current datetime if wasn't selected
  # @param langName - const name of used language
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return extracted datatime or None
  @staticmethod
  def extractUseDayState(inputStr, useCurrentYear, langName, logger, isExtendLog):
    # variable for results
    ret = None
    index = 0
    for dayState in DateTimeType.DAY_STATE_DICT[langName]:
      if not inputStr.lower().find(dayState) < 0:
        if logger is not None and isExtendLog:
          logger.debug("!!! dayState: " + str(dayState))

        if useCurrentYear:
          d = datetime.date.today()
          hour, minute, second, tf = DateTimeType.extractTime(inputStr, logger, isExtendLog)  # pylint: disable=W0612
          t = datetime.time(hour, minute, second)
          dt = datetime.datetime.combine(d, t)
          ret = dt - datetime.timedelta(days=index)
          break
      else:
        index = index + 1

    return ret


  # # Extract date from string used time name ('7 hours')
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current datetime if wasn't selected
  # @param langName - const name of used language
  # @return extracted datatime or None
  @staticmethod
  def extractUseTimePeriodName(inputStr, useCurrentYear, langName):
    # variable for results
    ret = None
    match = None
    if useCurrentYear and langName == DateTimeType.LANG_ENG:
      for pattern in [r'(?P<hour>\d{1,2}) hours']:
        match = re.search(pattern, inputStr, re.UNICODE)
        if match:
          d = datetime.datetime.utcnow()
          if 'hour' in match.groupdict():
            hour = match.groupdict()['hour']
            if int(hour) >= 0 and int(hour) <= 24:
              ret = d.replace(hour=int(hour), minute=0, second=0, microsecond=0, tzinfo=None)
              break
    # elif useCurrentYear and langName == DateTimeType.LANG_JAP:
    #  match = re.search(r'(?P<hour>\d{1,2})時間前', inputStr, re.U)
    return ret


  # # Extract date from string used period name left ('4  days left')
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current datetime if wasn't selected
  # @param langName - const name of used language
  # @return extracted datatime or None
  @staticmethod
  def extractUseTimePeriodNameLeft(inputStr, useCurrentYear, langName):
    # variable for results
    ret = None
    if useCurrentYear and langName == DateTimeType.LANG_ENG:
      for period in [u'years', u'months', u'days', u'hours', u'minutes']:
        match = re.search(r'(?P<value>\d{1,2}).? ' + period + '.?left', inputStr)
        if match:
          value = 0
          if 'value' in match.groupdict():
            value = int(match.groupdict()['value'])

          dt = datetime.datetime.utcnow()
          if period == u'years':
            ret = dt + relativedelta(years=+value)
          elif period == u'months':
            ret = dt + relativedelta(months=+value)
          elif period == u'days':
            ret = dt + relativedelta(days=+value)
          elif period == u'hours':
            ret = dt + relativedelta(hours=+value)
          elif period == u'minutes':
            ret = dt + relativedelta(minutes=+value)

          ret = ret.replace(second=0, microsecond=0, tzinfo=None)
          break

    return ret


  # # Extract date from string used period name ago ('1 Hour Ago')
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current datetime if wasn't selected
  # @param langName - const name of used language
  # @param logger - instance of logger for log if necessary
  # @return extracted datatime or None
  @staticmethod
  def extractUseTimePeriodNameAgo(inputStr, useCurrentYear, langName, logger):
    # variable for results
    ret = None
    if useCurrentYear and langName == DateTimeType.LANG_JAP and '月' not in inputStr and '日' not in inputStr:
      for pattern in [r'(?P<value>\d{1,2})日前', r'(?P<value>\d{1,2})時間前']:
        match = re.search(pattern, inputStr, re.UNICODE)
        if match is not None:
          value = 0
          if 'value' in match.groupdict():
            value = int(match.groupdict()['value'])

            if pattern.find('日前') > 0:
              ret = datetime.datetime.utcnow() + relativedelta(days=-value)
            elif pattern.find('時間前') > 0:
              ret = datetime.datetime.utcnow() + relativedelta(hours=-value)

            ret = ret.replace(second=0, microsecond=0, tzinfo=None)

    if useCurrentYear and langName == DateTimeType.LANG_ENG:
      for period in [u'years', u'months', u'days', u'hours', u'minutes', u'Hour']:
        for pattern in [r'(?P<value>\d{1,2}).? ' + period + '.?Ago',
                        r'(?P<value>\d{1,2}).? ' + period + '.?ago']:
          match = re.search(pattern, inputStr, re.UNICODE)
          if match:
            value = 0
            if 'value' in match.groupdict():
              value = int(match.groupdict()['value'])

            dt = datetime.datetime.utcnow()
            if period == u'years':
              ret = dt + relativedelta(years=-value)
            elif period == u'months':
              ret = dt + relativedelta(months=-value)
            elif period == u'days':
              ret = dt + relativedelta(days=-value)
            elif period == u'hours' or period == u'Hour':
              ret = dt + relativedelta(hours=-value)
            elif period == u'minutes':
              ret = dt + relativedelta(minutes=-value)

            ret = ret.replace(second=0, microsecond=0, tzinfo=None)
            if logger is not None:
              logger.debug("ret: %s", str(ret))
            break

    return ret


  # # Check code page of input string
  #
  # @param inputStr - input string for preparation
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return True if input string id UTF-8 code page or False otherwise
  @staticmethod
  def isUtf8CodePage(inputStr, logger, isExtendLog):
    # variable for result
    isUtf8 = False
    try:
      inputStr.decode('utf-8')
      isUtf8 = True
    except Exception, err:
      if logger is not None and isExtendLog:
        logger.debug('inputStr.decode: ' + str(err))

    return isUtf8


  # # Change code page of input string to ascii
  #
  # @param inputStr - input string for preparation
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return string with ascii code page
  @staticmethod
  def changeCodePageToAscii(inputStr, logger=None, isExtendLog=False):
    # variable for result
    ret = inputStr
    try:
      dataString = inputStr.decode('latin-1')
      ret = dataString.encode('ascii', errors='ignore')
    except Exception, err:
      if logger and isExtendLog:
        logger.debug("inputStr.decode('latin-1') : " + str(err))

    return ret


  # # Extract date from string used intelligent algorithms
  #
  # @param inputStr - input string for preparation
  # @param useCurrentYear - flag of default usage current year if wasn't selected
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @param langName - const name of used language
  # @return extracted datatime or None
  @staticmethod
  def intelligentExtractor(inputStr, useCurrentYear=True, logger=None, isExtendLog=False, langName=None):
    # variable for results
    ret = None
    dataString = copy.copy(inputStr)
    dataString = DateTimeType.prepareString(dataString)
    try:
      if langName is None:
        langName = DateTimeType.getLang(dataString)
      else:
        pass

      if langName is not None:
        ret = DateTimeType.extractUseTimePeriodNameAgo(dataString, useCurrentYear, langName, logger)

      if langName is not None and langName != DateTimeType.LANG_JAP and langName != DateTimeType.LANG_ARAB:
        if ret is None:
          ret = DateTimeType.extractUseDayState(dataString, useCurrentYear, langName, logger, isExtendLog)
          if ret is None:
            ret = DateTimeType.extractUseTimePeriodName(dataString, useCurrentYear, langName)
            if ret is None:
              ret = DateTimeType.extractUseTimePeriodNameLeft(dataString, useCurrentYear, langName)

      # #TODO here extended functional in future
    except Exception, err:
      if logger is not None and isExtendLog:
        logger.debug("inputStr: '" + inputStr + "'")
        if logger is not None and ExceptionLog is not None:
          ExceptionLog.handler(logger, err, 'intelligentExtractor:', (inputStr), \
                             {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
        elif logger:
          logger.debug('intelligentExtractor:' + str(err))

    if logger is not None and isExtendLog:
      logger.debug('intelligentExtractor return: ' + str(ret))

    return ret


  # # Convert Japanese Eras to full year
  #
  # @param inputStr - input string for preparation
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return modified pubdate string
  @staticmethod
  def convertJapaneseEras(inputStr, logger, isExtendLog):
    # variable for result
    ret = inputStr
    try:
      for name, startPeriod in DateTimeType.japaneseEras.items():
        if logger is not None and isExtendLog:
          logger.debug("Japanese Era has name '%s', period started from %s year.", str(name), str(startPeriod))

        startPeriodYear = startPeriod - 1
        match = re.search(name + r'(?P<year>\d{1,2})年', inputStr)

        if logger is not None and isExtendLog:
          logger.debug("match = %s", str(match))

        if match:
          if 'year' in match.groupdict():
            year = match.groupdict()['year']
            if logger is not None and isExtendLog:
              logger.debug("year = %s, type = %s", str(year), str(type(year)))

            ret = inputStr.replace("%s%s年" % (name, str(year)), "%s年" % str(int(year) + startPeriodYear))
            if logger is not None and isExtendLog:
              logger.debug("ret = %s", str(ret))

          break

    except Exception, err:
      if logger is not None and isExtendLog:
        logger.debug("Extract Japanese Eras period has error: %s", str(err))

    return ret


  # #Replace japan simbols to unicode
  #
  # @param rawPubdate - input raw string content pubdate
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return rawPubdate - already modified string
  @staticmethod
  def replaceJapanSimbols(rawPubdate, logger=None, isExtendLog=False):
    simbolsDict = {'－':'-', '．':'.', '：':':', '／':'/', '，':',', '･':'.', 'ｰ':'-', \
                   '０':'0', '１':'1', '２':'2', '３':'3', '４':'4', '５':'5', '６':'6', '７':'7', '８':'8', '９':'9'}
    # replace simbols
    for key, value in simbolsDict.items():
      try:
        rawPubdate = rawPubdate.replace(key, value)
      except Exception, err:
        if logger is not None and isExtendLog:
          logger.debug(str(err))

    return rawPubdate


  # #Convert timezone name to utc offset
  #
  # @param tzName - name of timezone
  # @param timezonesDict - dictionary with timezones
  # @return utc offset as string
  @staticmethod
  def utcOffset(tzName, timezonesDict=DateTimeTimezones.timezonesDict):  # pylint: disable=W0102
    # variable for result
    ret = None
    if tzName in timezonesDict:
      elem = timezonesDict[tzName]
      if len(elem) > 1:
        if elem[1].find('UTC') > -1:
          ret = elem[1][len('UTC'):]

    return ret


  # # Extract utc offset from string
  #
  # @param inputStr - input string for extract time
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @param timezonesDict - dictionary with timezones
  # @return utc offset numeric as string
  @staticmethod
  def extractUtcOffset(inputStr, logger=None, isExtendLog=False, timezonesDict=DateTimeTimezones.timezonesDict):  # pylint: disable=W0102
    # variable for result
    ret = None
    if inputStr is not None:
      for key in timezonesDict.keys():
        if (inputStr.find(key)) > -1:
          ret = DateTimeType.utcOffset(key, timezonesDict)
          if logger is not None and isExtendLog:
            logger.debug('Timezone: ' + str(key) + ' offset: ' + str(ret))
          break

      if ret is None:
        try:
          # try exctract timezone
          tzParts = inputStr.split()
          if logger is not None and isExtendLog:
            logger.debug("tzParts: %s", str(tzParts))

          for elem in tzParts:
            if logger is not None and isExtendLog:
              logger.debug("elem: '%s', length = %s, is allowed by max size (%s) = %s", str(elem), str(len(elem)), str(DateTimeType.MAX_SIZE_TIMEZONES_OFFSET),
                           str(len(elem) <= DateTimeType.MAX_SIZE_TIMEZONES_OFFSET))

            if len(elem) <= DateTimeType.MAX_SIZE_TIMEZONES_OFFSET:
              for pattern in DateTimeType.patternListTimezoneOffset:
                match = re.search(pattern, elem, re.I + re.U)
                if logger is not None and isExtendLog:
                  logger.debug("elem: %s, pattern: %s, match = %s", str(elem), str(pattern), str(match))
                if match:
                  if 'offset' in match.groupdict():
                    ret = match.groupdict()['offset']
                    break

            elif len(elem) >= DateTimeType.MIN_ALLOWED_LEN_FOR_DATEUTILS:
              dt = DateTimeType.extractUseDateutil(elem, logger, isExtendLog, DateTimeType.LANG_ENG)
              if dt is not None:
                timezone = DateTimeType.getTimezone(dt)
                if logger is not None and isExtendLog:
                  logger.debug("timezone = %s", str(timezone))

                if timezone != "":
                  ret = timezone

            else:
              if logger is not None and isExtendLog:
                logger.debug("Alternative case for '%s'", str(elem))

            if ret is not None:
              break

        except Exception, err:
          if logger is not None and isExtendLog:
            logger.debug('extractUtcOffset error: ' + str(err))

    if logger is not None and isExtendLog:
      logger.debug("!!! ret: %s", str(ret))
    return ret


  # # Extract timezone name from string
  # @param inputStr - input string for extract time
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @param timezonesDict - dictionary with timezones
  # @return utc timezone name as string, in case of fail extraction return empty string
  @staticmethod
  def extractUtcTimezoneName(inputStr, logger=None, isExtendLog=False, timezonesDict=DateTimeTimezones.timezonesDict):  # pylint: disable=W0102
    # variable for result
    ret = ''
    if logger is not None and isExtendLog:
      logger.debug("inputStr: %s", str(inputStr))
    if inputStr is not None:
      for key in timezonesDict.keys():
        pos = inputStr.find(key)
        if (pos) > -1:
          ret = key
          break

    if logger is not None and isExtendLog:
      logger.debug("return: %s", str(ret))

    return ret


  # # Extract timezone name from string
  # @param inputStr - input string for extract time
  # @param logger - instance of logger for log if necessary
  # @param isExtendLog - boolean flag for allowed extend logging if True or only error message otherwise
  # @return normalized string
  @staticmethod
  def normalizeTimezone(inputStr, logger=None, isExtendLog=False):
    # variable for result
    ret = inputStr
    if inputStr is not None:
      pos = inputStr.rfind('+')
      length = len('+')
      if pos == -1:
        pos = inputStr.rfind('-')
        length = len('-')

      if pos > -1:
        oldValue = inputStr[pos + length:].strip()
        newValue = ''
        if logger is not None and isExtendLog:
          logger.debug('oldValue: ' + str(oldValue))
        if oldValue.isdigit() and len(oldValue) > 1:
          newValue = '0' + oldValue[:1] + ':00'
        else:
          match = re.search(r'(?P<tzone>\d{1,4})Z', oldValue)
          if match:
            if 'tzone' in match.groupdict():
              newValue = match.groupdict()['tzone']

        if logger is not None and isExtendLog:
          logger.debug('newValue: ' + str(newValue))
        ret = inputStr.replace(oldValue, newValue)

    return ret


  # # Convert to UTC offset form
  #
  # @param dt - datetime instance
  # @return datetime instance without tzInfo and timezone string
  @staticmethod
  def toUTC(dt):
    # variable for result
    ret = dt
    if dt.utcoffset() is not None:
      ret = dt - dt.utcoffset()
    ret = ret.replace(tzinfo=None)

    return ret
