# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
FiltersLogging class content main functional of support 'FILTERS_LOG' property.

@package: app
@file FiltersLogging.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import os
import re
import datetime
import app.Utils as Utils
import app.Consts as APP_CONSTS

# # The FiltersLogging class for support of a URLs lists logging
class FiltersLogging(object):
  # # Constants used in class
  OPTION_STAGES = 'stages'
  OPTION_FILE = 'file'

  OPTION_STAGES_STAGE = 'stage'
  OPTION_STAGES_ACTION = 'action'
  OPTION_STAGES_URL_MASK = 'urlMask'

  # # Constants of macro used in class
  MACRO_PROJECT_ID = "%PROJECT_ID%"
  MACRO_SITE_MD5 = "%SITE_MD5%"
  MACRO_STAGE = "%STAGE%"

  # # Constants of error messages
  ERROR_MSG_TYPE_INPUT_VALUE = "Wrong type (%s) of the input value for '%s'"
  ERROR_MSG_PROPERTY_TYPE = "Wrong type (%s) of the property: %s"
  ERROR_MSG_UNSUPPORTED_VALUE = "Unsupported value '%s' for '%s'"
  ERROR_MSG_MISSED_OPTION = "Missed mandatory option '%s'"

  DELIMITER = ', '

  # # Initialization
  def __init__(self):
    pass

  # # Check is allowed logging
  #
  # @param propertyDict - property dictionary
  # @param url - URL string
  # @param siteId - site ID (project ID)
  # @param stage - stage for apply
  # @param action - filters action value (>0 - include, <=0 - exclude)
  # @param log - logger instance
  @staticmethod
  def isAllowedLogging(propertyDict, url, siteId, stage, action, log=None):
    #variable for result
    ret = False
    try:
      if not isinstance(propertyDict, dict):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(propertyDict)), 'propertyDict'))

      if not isinstance(url, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(url)), 'url'))

      if not isinstance(siteId, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(siteId)), 'siteId'))

      if not isinstance(stage, int):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(stage)), 'stage'))

      if not isinstance(action, int):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(action)), 'action'))

      if FiltersLogging.OPTION_STAGES in propertyDict:
        if not isinstance(propertyDict[FiltersLogging.OPTION_STAGES], list):
          raise Exception(FiltersLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(propertyDict[FiltersLogging.OPTION_STAGES]), FiltersLogging.OPTION_STAGES))
      else:
        raise Exception(FiltersLogging.ERROR_MSG_MISSED_OPTION % (FiltersLogging.OPTION_STAGES))

      for stageElem in propertyDict[FiltersLogging.OPTION_STAGES]:
        if log is not None:
          log.info("stageElem: %s", str(stageElem))

        if isinstance(stageElem, dict):
          if FiltersLogging.OPTION_STAGES_STAGE in stageElem and int(stageElem[FiltersLogging.OPTION_STAGES_STAGE]) == stage and\
            FiltersLogging.OPTION_STAGES_ACTION in stageElem and int(stageElem[FiltersLogging.OPTION_STAGES_ACTION]) == action and\
            FiltersLogging.OPTION_STAGES_URL_MASK in stageElem and isinstance(stageElem[FiltersLogging.OPTION_STAGES_URL_MASK], basestring) and\
            re.search(stageElem[FiltersLogging.OPTION_STAGES_URL_MASK], url, flags=re.U + re.I) is not None:
            ret = True
            break
        elif int(stageElem) == stage:
          ret = True
          break

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  ## Write data
  #
  # @param propertyDict - property dictionary
  # @param url - URL string
  # @param siteId - site ID (project ID)
  # @param stage - stage for apply
  # @param action - filters action value (>0 - include, <=0 - exclude)
  # @param pattern - patter string
  # @param log - logger instance
  # @return - None
  @staticmethod
  def writeData(propertyDict, url, siteId, stage, action, pattern, log=None):
    try:
      if not isinstance(propertyDict, dict):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(propertyDict)), 'propertyDict'))

      if not isinstance(url, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(url)), 'url'))

      if not isinstance(siteId, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(siteId)), 'siteId'))

      if not isinstance(stage, int):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(stage)), 'stage'))

      if not isinstance(action, int):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(action)), 'action'))

      if not isinstance(pattern, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(pattern)), 'pattern'))

      if FiltersLogging.OPTION_FILE in propertyDict:
        if not isinstance(propertyDict[FiltersLogging.OPTION_FILE], basestring) or propertyDict[FiltersLogging.OPTION_FILE] =="":
          raise Exception(FiltersLogging.ERROR_MSG_UNSUPPORTED_VALUE % (str(propertyDict[FiltersLogging.OPTION_FILE]), FiltersLogging.OPTION_FILE))
      else:
        raise Exception(FiltersLogging.ERROR_MSG_MISSED_OPTION % (FiltersLogging.OPTION_FILE))

      fileName = propertyDict[FiltersLogging.OPTION_FILE]
      fileName = re.sub(pattern=FiltersLogging.MACRO_PROJECT_ID, repl=siteId, string=fileName, flags=re.U+re.I)
      fileName = re.sub(pattern=FiltersLogging.MACRO_SITE_MD5, repl=siteId, string=fileName, flags=re.U+re.I)
      fileName = re.sub(pattern=FiltersLogging.MACRO_STAGE, repl=str(stage), string=fileName, flags=re.U+re.I)
      if log is not None:
        log.debug("fileName: %s", str(fileName))

      dataString = datetime.datetime.now().strftime("%Y-%m-%d %H:%M") + FiltersLogging.DELIMITER
      dataString += 'stage='+ str(stage)+ FiltersLogging.DELIMITER
      dataString += 'action='+ str(action)+ FiltersLogging.DELIMITER
      dataString += "pattern='"+ str(pattern)+"'"+ FiltersLogging.DELIMITER
      dataString += "url=" + str(url) + '\r\n'

      mode = 'a' if os.path.isfile(fileName) else 'w'
      f = open(fileName, mode)
      f.write(dataString)
      f.close()

    except Exception, err:
      if log is not None:
        log.error(str(err))


  # # Add data to log
  #
  # @param siteProperties - site properties value
  # @param url - URL string
  # @param siteId - site ID (project ID)
  # @param stage - stage for apply
  # @param action - filters action value (>0 - include, <=0 - exclude)
  # @param pattern - patter string
  # @param log - logger instance
  # @return - None
  @staticmethod
  def addLog(siteProperties, url, siteId, stage, action, pattern, log=None):
    try:
      if not isinstance(siteProperties, dict):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(siteProperties)), 'siteProperties'))

      if not isinstance(url, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(url)), 'url'))

      if not isinstance(siteId, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(siteId)), 'siteId'))

      if not isinstance(stage, int):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(stage)), 'stage'))

      if not isinstance(action, int):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(action)), 'action'))

      if not isinstance(pattern, basestring):
        raise Exception(FiltersLogging.ERROR_MSG_TYPE_INPUT_VALUE % (str(type(pattern)), 'pattern'))

      if APP_CONSTS.FILTERS_LOG in  siteProperties:
        propertyDict = Utils.jsonLoadsSafe(jsonString=siteProperties[APP_CONSTS.FILTERS_LOG], default=siteProperties[APP_CONSTS.FILTERS_LOG])
        if not isinstance(propertyDict, dict):
          raise Exception(FiltersLogging.ERROR_MSG_PROPERTY_TYPE % (str(type(propertyDict)), str(propertyDict)))

        if FiltersLogging.isAllowedLogging(propertyDict, url, siteId, stage, action, log):
          FiltersLogging.writeData(propertyDict, url, siteId, stage, action, pattern, log)

    except Exception, err:
      if log is not None:
        log.error(str(err))
