# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
PDateExtractor class content main functional of pubdate extraction from raw string.

@package: app
@file PDateExtractor.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import app.Consts as APP_CONSTS
from app.DateTimeType import DateTimeType
from app.PDateFormat import PDateFormat


class PDateExtractor(object):

  # # Initialization
  def __init__(self):
    pass


  # # extract date time
  #
  # @param dataString - data string for extract datatime
  # @param siteProperties - site properties instance
  # @param url - url string
  # @param stage - stage numeric value or None
  # @param log - logger instance
  # @param stagesFormatList - stages format list ([None, None, {...}])
  # @return datetime instance or None
  @staticmethod
  def extract(dataString, siteProperties, url=None, stage=None, log=None, stagesFormatList=None):
    # variable for result
    ret = None
    try:
      if isinstance(stagesFormatList, list):
        for i in range(0, len(stagesFormatList)):
          stageNumber = 1 << i
          if log is not None:
            log.debug("stageNumber: %s", str(stageNumber))

          if stagesFormatList[i] is not None and isinstance(stage, int) and bool(stage & stageNumber):
            if log is not None:
              log.debug("Stage (%s) is allowed to apply format for pubdate extraction.", str(stage))

            formatList, ifNotDetected, allowedStage = PDateFormat.loadProperty(stagesFormatList[i], url, log)
            ret = DateTimeType(dataString=dataString, formatObj=formatList, continueIfNotDetected=bool(ifNotDetected), log=log).datetime
            if log is not None:
              log.debug("Extracted from input format list: %s", str(ret))

      if ret is None:
        if APP_CONSTS.PDATE_FORMAT_PROP_NAME in siteProperties and siteProperties[APP_CONSTS.PDATE_FORMAT_PROP_NAME] is not None:
          if log is not None:
            log.debug("Property '%s' has value: %s", str(APP_CONSTS.PDATE_FORMAT_PROP_NAME), str(siteProperties[APP_CONSTS.PDATE_FORMAT_PROP_NAME]))

          # extract format list
          formatList, ifNotDetected, allowedStage = PDateFormat.loadProperty(siteProperties[APP_CONSTS.PDATE_FORMAT_PROP_NAME], url, log)
          if isinstance(stage, int) and isinstance(allowedStage, int) and bool(stage & allowedStage):
            if log is not None:
              log.debug("Stage (%s) is allowed to apply format for pubdate extraction.", str(stage))

            ret = DateTimeType(dataString=dataString, formatObj=formatList, continueIfNotDetected=bool(ifNotDetected), log=log).datetime
          else:
            ret = DateTimeType.parse(dataString=dataString, useCurrentYear=True, logger=log, isExtendLog=False)
        else:
          ret = DateTimeType.parse(dataString=dataString, useCurrentYear=True, logger=log, isExtendLog=True)

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret
