# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
PDateSourceExpression class content main functional of support 'PDATE_SOURCES_EXPRESSION' property.

@package: app
@file PDateSourceExpression.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re
import datetime

import app.Utils as Utils
from app.DateTimeType import DateTimeType
from app.Exceptions import DatabaseException
from app.Utils import ExceptionLog


class PDateSourceExpression(object):
  # # Constants used in class
  PROPERTY_PATTERN_NAME = 'pattern'
  PROPERTY_SQL_NAME = 'sql'

  MACRO_NAME_PROJECT_ID = '%PROJECT_ID%'
  MACRO_NAME_PROJECT_MD5 = '%PROJECT_MD5%'
  MACRO_NAME_CURRENT_URLMD5 = '%CURRENT_URLMD5%'
  MACRO_NAME_URLMD5 = '%URLMD5%'
  MACRO_NAME_URL = '%URL%'

  KWARG_NAME_PROJECT_ID = 'siteId'
  KWARG_NAME_URLMD5 = 'urlMd5'
  KWARG_NAME_URL = 'url'
  KWARG_NAME_SQL = 'sql'
  KWARG_NAME_LOG = 'log'

  MANDATORY_PART_QUERY_SELECT = 'SELECT'
  MANDATORY_PART_QUERY_FROM = 'FROM'
  TEMPLATE__PART_QUERY_FROM = "FROM urls_%PROJECT_MD5% WHERE `URLMd5` = '%CURRENT_URLMD5%'"

  DATETIME_NOW_VALUE = 'NOW()'

  DB_NAME = 'dc_urls'

  
  # Initialization class
  #
  # @param dbWrapper - instance of DBTasksWrapper for work with DB
  # @param property - property string
  # @param url - url string
  # @param urlMd5 - url md5 string
  # @param siteId - site ID
  # @param log - logger instance
  def __init__(self, dbWrapper, property, url, urlMd5, siteId, log=None):
    self.pDate = self.execute(dbWrapper, property, url, urlMd5, siteId, log)


  # # load property
  #
  # @param property - property string
  # @param url - url string
  # @param log - logger instance
  # @return sql expression string if can extracted or None value
  @staticmethod
  def loadProperty(property, url, log=None):
    # variables for result
    sqlExpression = None
    try:
      if isinstance(property, basestring) and property != "" and \
        property[0] != '{' and property[-1] != '}' and property[0] != '[' and property[-1] != ']':
        sqlExpression = property
      else:
        propertyObj = Utils.jsonLoadsSafe(jsonString=property, default=property, log=log)

        if isinstance(propertyObj, dict):
          sqlExpression = PDateSourceExpression.extractSqlExpressionValue(propertyDict=propertyObj, url=url, log=log)
        elif isinstance(propertyObj, list):
          for propertyDict in propertyObj:
            sqlExpression = PDateSourceExpression.extractSqlExpressionValue(propertyDict=propertyDict, url=url, log=log)
            if isinstance(sqlExpression, basestring) and sqlExpression != "":
              break

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return sqlExpression
  
  
  # # extract sql expression value
  #
  # @param propertyDict - property dictionary
  # @param url - url string
  # @param log - logger instance
  # @return sql expression string if can extracted or None value
  @staticmethod
  def extractSqlExpressionValue(propertyDict, url, log=None):
    # variables for result
    ret = None
    try:
      if isinstance(propertyDict, dict) and isinstance(url, basestring):
        pattern = propertyDict[PDateSourceExpression.PROPERTY_PATTERN_NAME] if PDateSourceExpression.PROPERTY_PATTERN_NAME in propertyDict else None
        sql = propertyDict[PDateSourceExpression.PROPERTY_SQL_NAME] if PDateSourceExpression.PROPERTY_SQL_NAME in propertyDict else None

        if isinstance(pattern, basestring) and re.search(pattern=pattern, string=url, flags=re.U + re.I) is not None and isinstance(sql, basestring):
          ret = sql

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # make sql query
  #
  # @param kwarg - dictionary of arguments (sql, log, url, urlMd5, siteId)
  # @return sql query string ready for execution
  @staticmethod
  def makeSqlQuery(**kwarg):
    # variables for result
    ret = None
    log = None
    try:
      if PDateSourceExpression.KWARG_NAME_LOG in kwarg and kwarg[PDateSourceExpression.KWARG_NAME_LOG] is not None:
        log = kwarg[PDateSourceExpression.KWARG_NAME_LOG]

      if PDateSourceExpression.KWARG_NAME_SQL in kwarg and kwarg[PDateSourceExpression.KWARG_NAME_SQL] is not None:
        rawSqlString = str(kwarg[PDateSourceExpression.KWARG_NAME_SQL])

        rawSqlString = "%s %s" % (rawSqlString, PDateSourceExpression.TEMPLATE__PART_QUERY_FROM) \
          if rawSqlString != PDateSourceExpression.DATETIME_NOW_VALUE and \
          re.search(pattern=PDateSourceExpression.MANDATORY_PART_QUERY_FROM, string=rawSqlString, flags=re.U + re.I) is None else rawSqlString

        if PDateSourceExpression.KWARG_NAME_PROJECT_ID in kwarg and kwarg[PDateSourceExpression.KWARG_NAME_PROJECT_ID] is not None:
          siteId = str(kwarg[PDateSourceExpression.KWARG_NAME_PROJECT_ID])
          rawSqlString = re.sub(pattern=PDateSourceExpression.MACRO_NAME_PROJECT_ID, repl=siteId, string=rawSqlString, flags=re.U+re.I)
          rawSqlString = re.sub(pattern=PDateSourceExpression.MACRO_NAME_PROJECT_MD5, repl=siteId, string=rawSqlString, flags=re.U+re.I)
        
        if PDateSourceExpression.KWARG_NAME_URLMD5 in kwarg and kwarg[PDateSourceExpression.KWARG_NAME_URLMD5] is not None:
          urlMd5 = str(kwarg[PDateSourceExpression.KWARG_NAME_URLMD5])
          rawSqlString = re.sub(pattern=PDateSourceExpression.MACRO_NAME_CURRENT_URLMD5, repl=urlMd5, string=rawSqlString, flags=re.U+re.I)
          rawSqlString = re.sub(pattern=PDateSourceExpression.MACRO_NAME_URLMD5, repl=urlMd5, string=rawSqlString, flags=re.U+re.I)

        if PDateSourceExpression.KWARG_NAME_URL in kwarg and kwarg[PDateSourceExpression.KWARG_NAME_URL] is not None:
          rawSqlString = re.sub(pattern=PDateSourceExpression.MACRO_NAME_URL, repl=str(kwarg[PDateSourceExpression.KWARG_NAME_URL]),
                                string=rawSqlString, flags=re.U + re.I)

        ret = "%s %s" % (PDateSourceExpression.MANDATORY_PART_QUERY_SELECT, rawSqlString) \
          if re.search(pattern=PDateSourceExpression.MANDATORY_PART_QUERY_SELECT, string=rawSqlString, flags=re.U + re.I) is None else rawSqlString

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # Execute sql query
  #
  # @param dbWrapper - instance of DBTasksWrapper for work with DB
  # @param sqlQuery - sql query for execution
  # @param log - logger instance
  # @return result of execution of sql query as datetime string or None if execution failed
  @staticmethod
  def executeSqlQuery(dbWrapper, sqlQuery, log=None):
    # variable for result
    ret = None
    try:
      if dbWrapper is not None:
        affectDB = dbWrapper.affect_db
        dbWrapper.affect_db = True
        customResponse = None
        try:
          customResponse = dbWrapper.customRequest(sqlQuery, PDateSourceExpression.DB_NAME)
        except DatabaseException, err:
          if log is not None:
            ExceptionLog.handler(log=log, error=err, message="SQL Query: %s" % str(sqlQuery))

        dbWrapper.affect_db = affectDB
        if log is not None:
          log.debug("customResponse: " + str(customResponse))
        if customResponse is not None and len(customResponse) > 0 and len(customResponse[0]) > 0:
          if isinstance(customResponse[0][0], datetime.datetime):
            ret = customResponse[0][0].strftime("%Y-%m-%d %H:%M:%S")
          else:
            dt = DateTimeType.parse(customResponse[0][0])
            if dt is not None:
              ret = dt.strftime("%Y-%m-%d %H:%M:%S")

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret

  # # Execute expression
  #
  # @param dbWrapper - instance of DBTasksWrapper for work with DB
  # @param property - property string
  # @param url - url string
  # @param urlMd5 - url md5 string
  # @param siteId - site ID
  # @param log - logger instance
  # @return datetime string or None if execution failed
  @staticmethod
  def execute(dbWrapper, property, url, urlMd5, siteId, log=None):
    # variable for result
    ret = None
    try:
      sqlExpression = PDateSourceExpression.loadProperty(property, url, log)
      if log is not None:
        log.debug("sql: %s", str(sqlExpression))

      sqlQuery = PDateSourceExpression.makeSqlQuery(sql=sqlExpression, log=log, url=url, siteId=siteId, urlMd5=urlMd5)
      if log is not None:
        log.debug("query: %s", str(sqlQuery))

      ret = PDateSourceExpression.executeSqlQuery(dbWrapper=dbWrapper, sqlQuery=sqlQuery, log=log)
      if log is not None:
        log.debug("return: %s", str(ret))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret
