# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
HTMLRedirectResolver Class content main functional for 'HTML_REDIRECTS_MAX' property.

@package: dc_crawler
@file HTMLRedirectResolver.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import app.Utils as Utils
import app.Consts as APP_CONSTS
import dc_crawler.Constants as CONSTS


class HTMLRedirectResolver(object):
  # # Constants used in class
  HEADERS_OPTION_CONTENT_TYPE_NAME = 'content-type'
  HEADERS_OPTION_CONTENT_TYPE_VALUE = 'text/html'

  # # Initialization
  def __init__(self):
    pass


  # # get max allowed HTML redirects
  #
  # @param siteProperties - site properties dictionary
  # @param log - logger instance
  # @return max allowed HTML redirects
  @staticmethod
  def getMaxAllowedRedirects(siteProperties, log=None):
    # variable for result
    ret = CONSTS.MAX_HTML_REDIRECTS_LIMIT
    try:
      if isinstance(siteProperties, dict) and APP_CONSTS.HTML_REDIRECTS_MAX in siteProperties:
        ret = int(siteProperties[APP_CONSTS.HTML_REDIRECTS_MAX])

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # Extract URL
  #
  # @param url - URL string
  # @param rawContent - raw content of the HTML page
  # @param log - logger instance
  # @return extracted HTML redirect URL or None
  @staticmethod
  def extractURL(url, rawContent, log=None):
    # variable for result
    ret = None
    try:
      if isinstance(url, basestring) and url != "" and isinstance(rawContent, basestring) and rawContent != "":
        extractedUrl = Utils.getHTMLRedirectUrl(rawContent, False, log)
        if log is not None:
          log.debug("Extracted HTML redirect URL '%s' from '%s'", str(extractedUrl), str(url))

        if extractedUrl is not None:
          normalizedUrl = Utils.urlNormalization(base=url, url=extractedUrl, log=log)
          if log is not None:
            log.info("Normalized HTML redirect URL: %s", str(normalizedUrl))

          if normalizedUrl is not None:
            ret = normalizedUrl

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret


  # # Check is allowed content-type
  #
  # @param headers - headers dictionary
  # @param log - logger instance
  # @return True if allowed or otherwise False
  @staticmethod
  def isAllowedContentType(headers, log=None):
    # variable for result
    ret = False
    try:
      if headers is not None and HTMLRedirectResolver.HEADERS_OPTION_CONTENT_TYPE_NAME in headers and \
        headers[HTMLRedirectResolver.HEADERS_OPTION_CONTENT_TYPE_NAME].find(HTMLRedirectResolver.HEADERS_OPTION_CONTENT_TYPE_VALUE) > -1:
        ret = True

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret
