# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
HTTPCookieExtractor Class content main functional for 'HTTP_COOKIE' property.

@package: dc_crawler
@file HTTPCookieExtractor.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import app.Consts as APP_CONSTS
import app.Utils as Utils  # pylint: disable=F0401
from dc_crawler.HTTPCookieResolver import HTTPCookieResolver

logger = Utils.MPLogger().getLogger()


class HTTPCookieExtractor(object):
  # # Constants used in class
  REQUEST_COOKIE_HEADER_NAME = 'Cookie'


  # # Initialization
  def __init__(self):
    pass


  # # Update headers by cookies
  #
  # @param siteProperties - site properties dictionary
  # @param siteId - Site/Project ID
  # @param url - URL string or the part of the URL
  # @param stage - allowed stage of usage (support of different stages use bitmask)
  # @param headers - headers dictionary
  # @return - already updated headers dictionary
  @staticmethod
  def updateHeadersByCookies(siteProperties, siteId, url, stage, headers):
    try:
      cachedCookies = HTTPCookieExtractor.getCookies(siteProperties=siteProperties, siteId=siteId, url=url, stage=stage)
      if isinstance(cachedCookies, basestring) and cachedCookies != "" and isinstance(headers, dict):
        headers[HTTPCookieExtractor.REQUEST_COOKIE_HEADER_NAME] = cachedCookies

    except Exception, err:
      logger.error(str(err))

    return headers


  ## Get cookies string
  #
  # @param siteProperties - site properties dictionary
  # @param siteId - Site/Project ID
  # @param url - URL string or the part of the URL
  # @param stage - allowed stage of usage (support of different stages use bitmask)
  # @return cookies string for set stage
  @staticmethod
  def getCookies(siteProperties, siteId, url, stage):
    # variable for result
    ret = None
    try:
      if isinstance(siteProperties, dict) and APP_CONSTS.HTTP_COOKIE in siteProperties:
        cookieResolver = HTTPCookieResolver(properties=siteProperties[APP_CONSTS.HTTP_COOKIE], siteId=siteId)
        cookieResolver.loadStorage(siteId, url)

        logger.info("!!! Try get cookies... for %s", str(url))
        ret = cookieResolver.getCookie(siteId=siteId, url=url, stage=stage)
        logger.debug("!!! Extracted cachedCookies: %s", str(ret))

    except Exception, err:
      logger.error(str(err))

    return ret


  # # Accumulate cookies
  #
  # @param siteProperties - site properties dictionary
  # @param siteId - Site/Project ID
  # @param url - URL string or the part of the URL
  # @param responseCookies - the response of the requests cookies object
  # @return - None
  @staticmethod
  def accumulateCookies(siteProperties, siteId, url, responseCookies):
    try:
      if isinstance(siteProperties, dict) and APP_CONSTS.HTTP_COOKIE in siteProperties:
        cookieResolver = HTTPCookieResolver(properties=siteProperties[APP_CONSTS.HTTP_COOKIE], siteId=siteId)
        cookieResolver.loadStorage(siteId, url)
        cookieResolver.appendCookies(siteId=siteId, responseCookies=responseCookies)
        cookieResolver.saveStorage(siteId=siteId, url=url)
    except Exception, err:
      logger.error(str(err))


  # # Add cookie as raw string
  #
  # @param siteProperties - site properties dictionary
  # @param siteId - Site/Project ID
  # @param url - URL string or the part of the URL
  # @param cookie - cookie string value or dict
  # @return - None
  @staticmethod
  def addCookie(siteProperties, siteId, url, cookie):
    try:
      if isinstance(siteProperties, dict) and APP_CONSTS.HTTP_COOKIE in siteProperties:
        cookieResolver = HTTPCookieResolver(properties=siteProperties[APP_CONSTS.HTTP_COOKIE], siteId=siteId)
        cookieResolver.loadStorage(siteId, url)
        cookieResolver.addCookie(siteId=siteId, url=url, cookie=cookie)
        cookieResolver.saveStorage(siteId=siteId, url=url)
    except Exception, err:
      logger.error(str(err))

