# coding: utf-8
'''
Created on Sep 04, 2019

@package: app
@file HTTPHeadersExtractor.py
@author Evgeniy Arefa <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''


import re
from random import randint

import app.Utils as Utils
import app.Consts as APP_CONSTS
from app.LFSDataStorage import LFSDataStorage


class HTTPHeadersExtractor(object):
  # # Constants used in class
  PROPERTY_DOMAIN_NAME = 'domain'
  PROPERTY_HEADERS_NAME = 'headers'
  PROPERTY_STORAGE_NAME = 'storage'

  FILE_SOURCE_PREFIX = 'file://'
  DISABLE_CHARACTER = '#'
  SEPARATOR = ':'

  ALGORITHM_RANDOM = True

  DEFAULT_SITEID_VALUE = '0'

  ARGUMENT_SITEID = 'siteId'
  ARGUMENT_DOMAIN = 'domain'
  ARGUMENT_STORAGE_DIR = 'storageDir'

  MSG_RAISE_ISNT_LIST = " isn't a list"
  MSG_RAISE_ISNT_NAME_VALUE = "There isn't a name of values in 'useFrequencyAlgorithm' function"
  MSG_RAISE_VALUE_MUST_BE_LIST = "Values must be a list in 'useFrequencyAlgorithm' function"
  MSG_RAISE_MUST_TOGETHER = "Properties 'domain' and 'headers' must be together in 'HTTP_HEADERS'"
  MSG_RAISE_MUST_WITH = "Properties 'domain' and 'headers' must be together in 'HTTP_HEADERS'. Property 'storage' must be only with 'domain' and 'headers'"

  MSG_ERROR_ISNT_SYMBOL = "There isn't a symbol '%s' in string '%s' or it's more then one"
  MSG_ERROR_ISNT_STRING = "'%s' isn't a string"

  MSG_DEBUG_DOMAIN_PASSED = "Domain '%s' passed for url: %s"
  MSG_DEBUG_DOMAIN_NOT_PASSED = "Domain '%s' not passed for url: %s"
  MSG_DEBUG_HOST_NAME_IP = "Current host name or IP: %s"
  MSG_DEBUG_INPUT_ELEMENT = "Input element: %s, type = %s"
  MSG_DEBUG_NODE_ELEMENT = "nodeElem: %s, type = %s"
  MSG_DEBUG_SAVE_ELEMENT = "Save element: %s, type = %s"
  MSG_DEBUG_NAME_AND_VALUE = "name: %s, value: %s"
  MSG_DEBUG_FOUND_PROPERTY = "Found property: '%s': %s, type: %s"
  MSG_DEBUG_RESULT = "!!! Result siteHeaders: %s', type = %s"

  # # Initialization
  def __init__(self):
    pass

  # # convert plain text in list to dictionary
  #
  # @param propertiesFile - plain text in list
  # @param log - logger instance
  # @return dictionary of headers from plain text in list
  @staticmethod
  def plainTextToDict(propertiesFile, log=None):
    # variable for result
    ret = {}

    try:
      if not isinstance(propertiesFile, list):
        raise Exception("'" + str(propertiesFile) + "'" + HTTPHeadersExtractor.MSG_RAISE_ISNT_LIST)

      for property in propertiesFile:
        if isinstance(property, basestring):
          valueKey = property.split(HTTPHeadersExtractor.SEPARATOR)
          if len(valueKey) != 2 or property.find(HTTPHeadersExtractor.SEPARATOR) == -1:
            if log is not None:
              log.error(HTTPHeadersExtractor.MSG_ERROR_ISNT_SYMBOL, HTTPHeadersExtractor.SEPARATOR, property)
            continue
          if property.startswith(HTTPHeadersExtractor.DISABLE_CHARACTER) or not valueKey[0]:
            continue
          ret.update({valueKey[0]: valueKey[1]})
        else:
          if log is not None:
            log.error(HTTPHeadersExtractor.MSG_ERROR_ISNT_STRING, property)

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret

  # # check domain in url
  #
  # @param domain - domain string
  # @param url - url string
  # @param log - logger instance
  # @return dictionary of headers if domain passed for url
  @staticmethod
  def checkDomain(domain, value, url, log=None):
    # variable for result
    ret = {}

    try:
      if re.search(pattern=domain, string=url, flags=re.I + re.U):
        ret = value
        if log is not None:
          log.debug(HTTPHeadersExtractor.MSG_DEBUG_DOMAIN_PASSED, domain, url)
      else:
        if log is not None:
          log.debug(HTTPHeadersExtractor.MSG_DEBUG_DOMAIN_NOT_PASSED, domain, url)

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return ret

  # # check host name or host IP
  #
  # @param name - name of property
  # @param values - values of property
  # @param properties - properties from headres
  # @param log - logger instance
  # @return values if current host name or host IP else None
  @staticmethod
  def checkHost(name, values, properties, log=None):
    # variable for result

    try:
      CurrentHostIP = []
      for item in (Utils.getCurrentHostName(), Utils.getIP()):
        if isinstance(item, basestring) and item:
          CurrentHostIP.append(item)
      if Utils.getCurrentIPInterfaces(log=log) and isinstance(Utils.getCurrentIPInterfaces(), dict):
        for key, keyValues in Utils.getCurrentIPInterfaces().items():
          if isinstance(keyValues, basestring) and keyValues:
            CurrentHostIP.append(keyValues)
          elif isinstance(keyValues, list):
            for keyValue in keyValues:
              if isinstance(keyValue, basestring) and keyValue:
                CurrentHostIP.append(keyValue)

      for item in CurrentHostIP:
        if values.get(item):
          if log is not None:
            log.debug(HTTPHeadersExtractor.MSG_DEBUG_HOST_NAME_IP, item)
          properties.update({name: values[item]})
          values = values[item]
          break
      else:
        values = None

    except Exception, err:
      values = None
      if log is not None:
        log.error(str(err))

    return values

  # # load form storage or save to storage
  #
  # @param element - data to save
  # @param log - logger instance
  # @param load - how operation (True - load, False - save)
  # @param kwarg - named arguments
  @staticmethod
  def loadSaveElement(element=None, log=None, load=True, ** kwarg):
    # variable for result
    nodeElem = None

    try:
      if (isinstance(element, dict) and len(element) > 0) or load:
        siteId = HTTPHeadersExtractor.DEFAULT_SITEID_VALUE
        if HTTPHeadersExtractor.ARGUMENT_SITEID in kwarg:
          siteId = kwarg[HTTPHeadersExtractor.ARGUMENT_SITEID]

        domain = None
        if HTTPHeadersExtractor.ARGUMENT_DOMAIN in kwarg:
          domain = kwarg[HTTPHeadersExtractor.ARGUMENT_DOMAIN]

        storageDir = None
        if HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR in kwarg:
          storageDir = kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR]

        if log is not None:
          log.debug(HTTPHeadersExtractor.MSG_DEBUG_INPUT_ELEMENT, str(element), str(type(element)))

        dataStorage = LFSDataStorage()

        if load:
          nodeElem = dataStorage.loadElement(storageDir, domain, siteId)
          if log is not None:
            log.debug(HTTPHeadersExtractor.MSG_DEBUG_NODE_ELEMENT, str(nodeElem), str(type(nodeElem)))
        else:
          dataStorage.saveElement(storageDir, domain, siteId, element)
          if log is not None:
            log.debug(HTTPHeadersExtractor.MSG_DEBUG_SAVE_ELEMENT, str(element), str(type(element)))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return nodeElem

  # # use low frequency algorithm for values of property
  #
  # @param siteHeaders - site headers has already gotten
  # @param nodeElem - stored element from storage
  # @param name - name of property
  # @param values - values of property
  # @param log - logger instance
  # @return siteHeaders and nodeElem with new value
  @staticmethod
  def useFrequencyAlgorithm(siteHeaders, nodeElem, name, values, log=None):
    siteHeaders = siteHeaders if siteHeaders else {}
    nodeElem = nodeElem if nodeElem else {}

    try:
      if not name:
        raise Exception(HTTPHeadersExtractor.MSG_RAISE_ISNT_NAME_VALUE)

      if not isinstance(values, list):
        raise Exception(HTTPHeadersExtractor.MSG_RAISE_VALUE_MUST_BE_LIST)

      index = 0
      frequency = 0
      # set default value
      if name not in nodeElem or len(nodeElem[name]) != len(values) or "" in values:
        nodeElem[name] = {}
        while "" in values:
          values.remove("")

        for i in range(0, len(values)):
          nodeElem[name].update({str(i): frequency})

      if log is not None:
        log.debug("nodeElem1: %s", Utils.varDump(nodeElem, stringifyType=0))

      if isinstance(nodeElem, dict) and name in nodeElem and isinstance(nodeElem[name], dict):
        if log is not None:
          log.debug("nodeElem['%s']: %s, type = %s", str(name), str(nodeElem[name]), str(type(nodeElem)))

        for key, value in sorted(nodeElem[name].iteritems(), key=lambda (k, v): (v, k)):
          if log is not None:
            log.debug("key: %s, value: %s", str(key), str(value))

        for _index, _frequency in sorted(nodeElem[name].iteritems(), key=lambda (k, v): (v, k)):
          index = int(_index)
          frequency = int(_frequency)
          break

      if log is not None:
        log.debug("index = %s, frequency = %s", str(index), str(frequency))

      if index < len(values):
        siteHeaders.update({name: values[index]})
        nodeElem[name].update({str(index): frequency + 1})

      if log is not None:
        log.debug(HTTPHeadersExtractor.MSG_DEBUG_NODE_ELEMENT, str(nodeElem), str(type(nodeElem)))

    except Exception, err:
      if log is not None:
        log.error(str(err))

    return siteHeaders, nodeElem

  # # get site headers
  #
  # @param properties - properties value
  # @param url - url string
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return site headers dictionary
  @staticmethod
  def getSiteHeaders(properties, url=None, log=None, **kwarg):
    # variable for result
    siteHeaders = {}

    if isinstance(properties, basestring):
      properties = properties.strip('"').strip("'")

    try:
      # if '<local_file_path>' is exist, read data from file and substitute it instead '<local_file_path>'
      if isinstance(properties, basestring) and properties.startswith(HTTPHeadersExtractor.FILE_SOURCE_PREFIX):
        propertiesFromFile = Utils.jsonLoadsSafe(Utils.readFile(properties, protocolPrefix=HTTPHeadersExtractor.FILE_SOURCE_PREFIX, loggerObj=log))
        if propertiesFromFile:
          properties = propertiesFromFile
        else:
          propertiesFromFile = Utils.loadFromFileByReference(fileReference=properties, loggerObj=log)
          properties = HTTPHeadersExtractor.plainTextToDict(propertiesFromFile, log=log)

      if isinstance(properties, basestring):
        properties = Utils.jsonLoadsSafe(jsonString=properties, default=properties, log=None)

      # check "HTTP_HEADERS" is list of dictionaries. Remake it from list to dictionary
      if isinstance(properties, list):
        should_restart = True

        while should_restart:
          resDict = {}
          should_restart = False

          for property in properties:
            # check a domain
            if property.get(HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME) and property.get(HTTPHeadersExtractor.PROPERTY_HEADERS_NAME):
              resDict = HTTPHeadersExtractor.checkDomain(
                property[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME], property[HTTPHeadersExtractor.PROPERTY_HEADERS_NAME], url, log=log
              )
              if not resDict:
                continue

              # check a storage
              if property.get(HTTPHeadersExtractor.PROPERTY_STORAGE_NAME):
                kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR] = property[HTTPHeadersExtractor.PROPERTY_STORAGE_NAME]
                HTTPHeadersExtractor.ALGORITHM_RANDOM = False

              if isinstance(resDict, list):
                properties = resDict
                should_restart = True
              break

            elif property.get(HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME) or property.get(HTTPHeadersExtractor.PROPERTY_HEADERS_NAME):
              raise Exception(HTTPHeadersExtractor.MSG_RAISE_MUST_TOGETHER)

            # simple combination of dictionaries into one
            if isinstance(property, dict):
              for name, values in property.items():
                # check a host name or host IP
                if isinstance(values, dict):
                  values = HTTPHeadersExtractor.checkHost(name, values, property, log=log)
                  if not values:
                    property.pop(name)
                    continue
              resDict.update(property)
          properties = resDict if resDict else {}

      # check a domain
      if properties.get(HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME) and properties.get(HTTPHeadersExtractor.PROPERTY_HEADERS_NAME):
        storage = properties.get(HTTPHeadersExtractor.PROPERTY_STORAGE_NAME)
        properties = HTTPHeadersExtractor.checkDomain(
          properties[HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME], properties[HTTPHeadersExtractor.PROPERTY_HEADERS_NAME], url, log=log
        )
        # check a storage
        if properties and storage:
          kwarg[HTTPHeadersExtractor.ARGUMENT_STORAGE_DIR] = storage
          HTTPHeadersExtractor.ALGORITHM_RANDOM = False

      elif properties.get(HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME) or properties.get(HTTPHeadersExtractor.PROPERTY_HEADERS_NAME) \
              or properties.get(HTTPHeadersExtractor.PROPERTY_STORAGE_NAME):
        raise Exception(HTTPHeadersExtractor.MSG_RAISE_MUST_WITH)

      # load element from storage
      nodeElem = {}
      if not HTTPHeadersExtractor.ALGORITHM_RANDOM:
        nodeElem = HTTPHeadersExtractor.loadSaveElement(log=log, **kwarg)
        if not isinstance(nodeElem, dict):
          nodeElem = {}

      for name, values in properties.items():
        if log is not None:
          log.debug(HTTPHeadersExtractor.MSG_DEBUG_NAME_AND_VALUE, str(name), str(values))

        # check a host name or host IP
        if isinstance(values, dict):
          values = HTTPHeadersExtractor.checkHost(name, values, properties, log=log)
          if not values:
            properties.pop(name)
            continue

        # if '<local_file_path>' is exist, read data from file and substitute it instead '<local_file_path>'
        if isinstance(values, basestring) and values.startswith(HTTPHeadersExtractor.FILE_SOURCE_PREFIX):
          propertiesFromFile = Utils.jsonLoadsSafe(Utils.readFile(values, protocolPrefix=HTTPHeadersExtractor.FILE_SOURCE_PREFIX, loggerObj=log))
          if propertiesFromFile:
            values = propertiesFromFile.get(name)
          else:
            propertiesFromFile = Utils.loadFromFileByReference(fileReference=values, loggerObj=log)
            values = propertiesFromFile

        if isinstance(values, basestring):
          siteHeaders.update({name: values})

        elif isinstance(values, list):
          if HTTPHeadersExtractor.ALGORITHM_RANDOM:
            siteHeaders.update({name: values[randint(0, len(values) - 1)]})
          else:
            # use low frequency algorithm
            siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm(siteHeaders, nodeElem, name, values, log=log)

      # save element to storage if necessary
      if not HTTPHeadersExtractor.ALGORITHM_RANDOM:
        HTTPHeadersExtractor.loadSaveElement(element=nodeElem, log=log, load=False, **kwarg)

    except Exception, err:
      if log is not None:
        log.error(str(err))
        log.info(Utils.getTracebackInfo())

    return siteHeaders

  # # extract HTTP headers
  #
  # @param siteProperties - site properties dictionary
  # @param url - url string
  # @param headers - headers dictionary as default value
  # @param log - logger instance
  # @param kwarg - named arguments
  # @return extracted HTTP headers
  @staticmethod
  def extract(siteProperties, url, headers=None, log=None, **kwarg):
    # variable for result
    siteHeaders = headers if isinstance(headers, dict) else {}
    try:
      if APP_CONSTS.HTTP_HEADERS in siteProperties and siteProperties[APP_CONSTS.HTTP_HEADERS] != "":
        if log is not None:
          log.debug(HTTPHeadersExtractor.MSG_DEBUG_FOUND_PROPERTY, str(APP_CONSTS.HTTP_HEADERS), str(siteProperties[APP_CONSTS.HTTP_HEADERS]),
                    str(type(siteProperties[APP_CONSTS.HTTP_HEADERS])))

        propertiesValue = siteProperties[APP_CONSTS.HTTP_HEADERS]
        extractedSiteHeaders = HTTPHeadersExtractor.getSiteHeaders(properties=propertiesValue, url=url, log=log, **kwarg)

        if isinstance(extractedSiteHeaders, dict) and len(extractedSiteHeaders) > 0:
          siteHeaders.update(extractedSiteHeaders)

    except Exception, err:
      if log is not None:
        log.error(str(err))
        log.info(Utils.getTracebackInfo())

    if log is not None:
      log.debug(HTTPHeadersExtractor.MSG_DEBUG_RESULT, str(siteHeaders), str(type(siteHeaders)))

    return siteHeaders
