# coding: utf-8
"""
HCE project,  Python bindings, Distributed Tasks Manager application.
UrlChain Class content main functional of url chain processing.

@package: dc_crawler
@file UrlChain.py
@author Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import re
import copy
# import json
# import requests
# import dc.EventObjects as dc_event
import app.Utils as Utils
# from app.Utils import urlNormalization


class UrlChain(object):
  # # Constants used in class
  OPTION_URL_PATTERN_EXCLUDE = 'url_pattern_exclude'
  OPTION_URL_PATTERN = 'url_pattern'
  OPTION_TAGS_NAME = 'tags_name'
  OPTION_DELIMITER = 'delimiter'
  OPTION_URL_PATTERN_RE_DOMAIN_NAME = 'REDN'
  OPTION_URL_PATTERN_DOMAIN_NAME_MACRO = 'DNM'
  OPTION_URL_PATTERN_PATTERN_RE = 'PRE'

  DEFAULT_CHAIN_DELIMITER = ' '
  DEFAULT_MAX_BATCH_ITERATIONS = 5


  # # Initialization
  #
  # @param properties - properties string
  # @param log - logger instance
  def __init__(self, properties, log):
    self.logger = log
    self.properties = Utils.jsonLoadsSafe(jsonString=properties, default={}, log=self.logger)
    self.maxBatchIterations = self.DEFAULT_MAX_BATCH_ITERATIONS


  # # Get pattern
  #
  # @param url - url string
  # @param urlPatternOptionName - url pattern option name
  # @return regular expression url pattern
  def getPattern(self, url, urlPatternOptionName):
    # variable for result
    urlPattern = None
    try:
      if isinstance(urlPatternOptionName, basestring) and urlPatternOptionName in self.properties and isinstance(url, basestring):
        if isinstance(self.properties[urlPatternOptionName], basestring):
          urlPattern = self.properties[urlPatternOptionName]

        elif isinstance(self.properties[urlPatternOptionName], list):
          urlPatternList = []
          for urlPatternListItem in self.properties[urlPatternOptionName]:
            if isinstance(urlPatternListItem, basestring):
              urlPatternList.append(urlPatternListItem)
          urlPattern = urlPatternList

        elif isinstance(self.properties[urlPatternOptionName], dict):
          self.logger.info("Processing complexity pattern rules...")
          # # For "url_pattern":{"REDN":"[/](.*)\\w+.html", "DNM":"%DM%", "PRE":"%DM%\\w+.html"}
          if self.OPTION_URL_PATTERN_RE_DOMAIN_NAME in self.properties[urlPatternOptionName] and \
            isinstance(self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_RE_DOMAIN_NAME], basestring) and \
            self.OPTION_URL_PATTERN_DOMAIN_NAME_MACRO in self.properties[urlPatternOptionName] and \
            isinstance(self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_DOMAIN_NAME_MACRO], basestring) and \
            self.OPTION_URL_PATTERN_PATTERN_RE in self.properties[urlPatternOptionName] and \
            isinstance(self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_PATTERN_RE], basestring):

            self.logger.debug("!!! urlPattern: %s", str(self.properties[urlPatternOptionName]))
            self.logger.debug("!!! url: %s, type = %s", str(url), str(type(url)))

            reDomainNames = re.findall(pattern=self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_RE_DOMAIN_NAME],
                                       string=url,
                                       flags=re.U + re.I)
            self.logger.debug("reDomainNames: %s", str(reDomainNames))
            if len(reDomainNames) == 0:
              reDomainNames = re.findall(pattern=re.escape(self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_RE_DOMAIN_NAME]),
                                         string=url,
                                         flags=re.U + re.I)
              self.logger.debug("reDomainNames: %s", str(reDomainNames))

            if len(reDomainNames) > 0:
              if not isinstance(reDomainNames[0], basestring):
                raise Exception("Option '%s' used wrong pattern '%s' return value: %s" %
                                (urlPatternOptionName, self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_RE_DOMAIN_NAME], str(reDomainNames)))

              domainName = Utils.parseHost(reDomainNames[0], True)
#               domainName = reDomainNames[0]
              if domainName is None or domainName == "":
                self.logger.debug("Extracted domain name '%s' is wrong.", str(domainName))
              else:
                self.logger.debug("Replace '%s' by domain name: %s", str(self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_DOMAIN_NAME_MACRO]),
                                  str(domainName))

                urlPatternRe = copy.deepcopy(self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_PATTERN_RE])
                urlPattern = re.sub(pattern=self.properties[urlPatternOptionName][self.OPTION_URL_PATTERN_DOMAIN_NAME_MACRO],
                                    repl=re.escape(domainName),
                                    string=urlPatternRe,
                                    flags=re.U + re.I)

              self.logger.debug("urlPattern: %s", str(urlPattern))

    except Exception, err:
      self.logger.error(str(err))
#       self.logger.info(Utils.getTracebackInfo())

    return urlPattern


  # # Get url pattern
  #
  # @param url - url string
  # @return regular expression url pattern
  def getUrlPattern(self, url):
    return self.getPattern(url=url, urlPatternOptionName=self.OPTION_URL_PATTERN)


  # # Get url pattern exclude
  #
  # @param url - url string
  # @return regular expression url pattern exclude
  def getUrlPatternExclude(self, url):
    return self.getPattern(url=url, urlPatternOptionName=self.OPTION_URL_PATTERN_EXCLUDE)


  # # url pattern apply
  #
  # @param urlPattern - regular expression url pattern
  # @param url - url string
  # @return True if pattern passed
  def urlPatternApply(self, urlPatternInclude, url):
    # variable for result
    ret = False
    try:
      urlPatternList = []
      if isinstance(urlPatternInclude, basestring):
        urlPatternList.append(urlPatternInclude)
      elif isinstance(urlPatternInclude, list):
        urlPatternList = urlPatternInclude

      for urlPattern in urlPatternList:
        if isinstance(urlPattern, basestring) and \
          (re.search(urlPattern, url, re.I + re.U) is not None or re.search(re.escape(urlPattern), url, re.I + re.U) is not None):
          ret = True
          break

    except Exception, err:
      self.logger.error(str(err))

    return ret


  # # url pattern exclude apply
  #
  # @param urlPatternExclude - regular expression url pattern exclude
  # @param url - url string
  # @return True if pattern passed
  def urlPatternExcludeApply(self, urlPatternExclude, url):
    # variable for result
    ret = False
    try:
      if urlPatternExclude is None:
        ret = True
      else:
        urlPatternExcludeList = []
        if isinstance(urlPatternExclude, basestring):
          urlPatternExcludeList.append(urlPatternExclude)
        elif isinstance(urlPatternExclude, list):
          urlPatternExcludeList = urlPatternExclude

        for urlPattern in urlPatternExcludeList:
          if isinstance(urlPattern, basestring) and \
            (re.search(urlPattern, url, re.I + re.U) is None and re.search(re.escape(urlPattern), url, re.I + re.U) is None):
            ret = True
            break

    except Exception, err:
      self.logger.error(str(err))

    return ret


  # # Get tags name
  # @param - None
  # @return tags name list
  def getTagsName(self):
    # variable for result
    ret = None
    if self.OPTION_TAGS_NAME in self.properties and isinstance(self.properties[self.OPTION_TAGS_NAME], list):
      ret = self.properties[self.OPTION_TAGS_NAME]

    return ret


  # # Get delimiter
  # @param - None
  # @return delimiter
  def getDelimiter(self):
    # variable for result
    ret = self.DEFAULT_CHAIN_DELIMITER
    if self.OPTION_DELIMITER in self.properties and isinstance(self.properties[self.OPTION_DELIMITER], basestring):
      ret = self.properties[self.OPTION_DELIMITER]

    return ret

