'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''

import hashlib
import dc.EventObjects
from dtm.EventObjects import GeneralResponse
import dc_db.Constants as Constants
from dc_db.FieldRecalculator import FieldRecalculator
from dc_db.BaseTask import BaseTask
from dc_db.URLCleanupTask import URLCleanUpTask
from dc_db.StatisticLogManager import StatisticLogManager
from dc_db.AttrDeleteTask import AttrDeleteTask
import app.Utils as Utils  # pylint: disable=F0401
from app.UrlsLogging import UrlsLogging

logger = Utils.MPLogger().getLogger()


# #process urlDelete event
class URLDeleteTask(BaseTask):


  # #constructor
  #
  # @param keyValueStorageDir path to keyValue storage work dir
  # @param rawDataDir path to raw data dir
  def __init__(self, keyValueStorageDir, rawDataDir, dBDataTask):
    super(URLDeleteTask, self).__init__()
    self.uRLCleanUpTask = URLCleanUpTask(keyValueStorageDir, rawDataDir, dBDataTask)
    self.recalculator = FieldRecalculator()
    self.urlMd5 = None
    # self.dBDataTask = dBDataTask


  # #make all necessary actions to delete urls data from db
  #
  # @param urlDelete list of URLDelete objects
  # @param queryCallback function for queries execution
  # @return generalResponse instance of GeneralResponse object
  def process(self, urlDeletes, queryCallback):
    ret = GeneralResponse()
    for urlDelete in urlDeletes:
      if urlDelete.siteId == "":
        urlDelete.siteId = "0"
      if self.isSiteExist(urlDelete.siteId, queryCallback):
        try:
          localUrls = []
          if urlDelete.url is None:
            isUrlExtract = False
            if urlDelete.urlType == dc.EventObjects.URLStatus.URL_TYPE_URL:
              isUrlExtract = True
            localUrls = self.uRLCleanUpTask.extractUrlByCriterions(urlDelete.siteId, isUrlExtract,
                                                                   urlDelete.criterions, queryCallback)
          else:
            localUrls.append(urlDelete.url)
          logger.debug(">>> [URLDelete] localUrls size = " + str(len(localUrls)))
          for localUrl in localUrls:
            try:
              urlDelete.url = localUrl

              # Add URLs logging
              self.processUrlsLogging(urlDelete.siteId, self.calculateMd5FormUrl(urlDelete.url, urlDelete.urlType, True), queryCallback)

              if urlDelete.delayedType == dc.EventObjects.NOT_DELAYED_OPERATION:
                self.uRLCleanUpTask.deleteFromDataStorage(urlDelete, queryCallback)
                self.uRLCleanUpTask.deleteFromRawStorage(urlDelete)
              elif urlDelete.delayedType == dc.EventObjects.DELAYED_OPERATION:
                self.copyUrlToDeleteDB(urlDelete, queryCallback)
              self.deleteFromMysqlDB(urlDelete, queryCallback)
              AttrDeleteTask.deleteUrlsAttributes(urlDelete.siteId, self.urlMd5, queryCallback)
              if self.urlMd5 is not None:
                StatisticLogManager.statisticUpdate(queryCallback, Constants.StatFreqConstants.FREQ_DELETE, 1,
                                                    urlDelete.siteId, self.urlMd5)
                StatisticLogManager.statisticUpdate(queryCallback, Constants.StatFreqConstants.FREQ_DELETED_STATE, 1,
                                                    urlDelete.siteId, self.urlMd5)
                StatisticLogManager.logUpdate(queryCallback, "LOG_DELETE", urlDelete, urlDelete.siteId, self.urlMd5)

            except Exception as ex:
              logger.debug(">>> [URLDelete] Some Type Exception [LOOP] = " + str(type(ex)) + " " + str(ex))
          ret.statuses.append(True)
          self.recalculator.commonRecalc(urlDelete.siteId, queryCallback)
        except Exception as excp:
          logger.debug(">>> [URLDelete] Some Type Exception = " + str(type(excp)) + " " + str(excp))
          ret.statuses.append(False)
      else:
        ret.statuses.append(False)

    return ret


  # # process URLs logging
  #
  # @param siteId - Site/Project ID
  # @param urlMd5 - URL Md5 string
  # @param queryCallback function for queries execution
  # @return - None
  def processUrlsLogging(self, siteId, urlMd5, queryCallback):
    try:
      # get property
      selectPropertyQuery = "SELECT `Value` FROM `sites_properties` WHERE `Site_Id`='%s' AND `Name`='URLS_LOGGING' LIMIT 1" % str(siteId)
      selectPropertyResult = queryCallback(selectPropertyQuery, Constants.PRIMARY_DB_ID)
      if selectPropertyResult is not None:
        for selectPropertyResultItem in selectPropertyResult:
          if selectPropertyResultItem is not None and len(selectPropertyResultItem) > 0:
            # property exists and necessary processing
            logger.debug("!!! Extracted property 'URLS_LOGGING' has value: %s, type = %s",
                         str(selectPropertyResultItem[0]),
                         str(type(selectPropertyResultItem[0])))

            # get URL's fields
            # Make query for URLs select
            query = "SELECT * FROM %s WHERE `URLMd5`='%s'" % (Constants.DC_URLS_TABLE_NAME_TEMPLATE % str(siteId), str(urlMd5))
            logger.debug("query: %s", str(query))
            # Select execution
            urlsRes = queryCallback(query, Constants.SECONDARY_DB_ID, Constants.EXEC_NAME)
            logger.debug("response: %s", str(urlsRes))

            for urlsRes in urlsRes:
              if isinstance(urlsRes, dict):
                fields = {}
                for key, value in urlsRes.items():
                  for urlTableDictKey, urlTableDictValue in Constants.URLTableDict.items():
                    if urlTableDictValue == key:
                      if isinstance(value, long):
                        value = type(int())(value)

                      fields[urlTableDictKey] = value
                      break
                logger.debug("fields: %s", str(fields))

                urlObj = dc.EventObjects.URL(fields['siteId'], fields['url'])
                for key, value in fields.items():
                  if hasattr(urlObj, key):
                    setattr(urlObj, key, fields[key])

                UrlsLogging.addUrl(properties=selectPropertyResultItem[0],
                                   url=urlObj.url,
                                   siteId=urlObj.siteId,
                                   stage=UrlsLogging.STAGE_DELETED,
                                   urlObj=urlObj,
                                   log=logger)
    except Exception, err:
      logger.error(str(err))
      logger.info(Utils.getTracebackInfo())


  # #update data in mysql db
  #
  # @param urlDelete instance of URLDelete object
  # @param queryCallback function for queries execution
  def deleteFromMysqlDB(self, urlDelete, queryCallback):
    self.urlMd5 = self.calculateMd5FormUrl(urlDelete.url, urlDelete.urlType, True)
    SQL_CLAUSE = ("`URLMd5` = '%s'" % self.urlMd5)
    UPDATE_SQL_TEMPLATE = "DELETE FROM `%s` WHERE %s"
    tableName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % urlDelete.siteId
    query = UPDATE_SQL_TEMPLATE % (tableName, SQL_CLAUSE)
    queryCallback(query, Constants.SECONDARY_DB_ID)


  # #update data in mysql db
  #
  # @param urlDelete instance of URLDelete or URLCleanup object
  # @param queryCallback function for queries execution
  def copyUrlToDeleteDB(self, urlDelete, queryCallback):
    if urlDelete.urlType == dc.EventObjects.URLStatus.URL_TYPE_URL:
      localMd5 = hashlib.md5(urlDelete.url).hexdigest()
    else:
      localMd5 = urlDelete.url
    SQL_COPY_QUERY_TEMPLATE = "INSERT INTO %s SELECT * FROM dc_urls.%s WHERE `URLMd5` = '%s'"
    tbName = Constants.DC_URLS_TABLE_NAME_TEMPLATE % urlDelete.siteId
    # TODO: One more query for each delete request is too heavy for DB
    # query = Constants.SQL_CREATE_QUERY_TEMPLATE % (tbName, tbName)
    # queryCallback(query, Constants.FOURTH_DB_ID)
    query = SQL_COPY_QUERY_TEMPLATE % (tbName, tbName, localMd5)
    queryCallback(query, Constants.FOURTH_DB_ID, Constants.EXEC_INDEX, True)
