# coding: utf-8
"""
Created on Sep 11, 2019

@package: app
@file HTTPHeadersExtractor.py
@author Evgeniy Arefa <developers.hce@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2019 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import copy

import app.Utils as Utils


class ScraperTagsValidation(object):
  # # Constants used in class

  SCRAPER_TAGS_VALIDATION = 'SCRAPER_TAGS_VALIDATION'

  SCRAPY = 'ScrapyExtractor'
  GOOSE = 'GooseExtractor'
  NEWSPAPER = 'NewspaperExtractor'

  SCRAPING_MODULE_NAMES = {'SCRAPY': [SCRAPY, ], 'GOOSE': [GOOSE, ], 'NEWSPAPER': [NEWSPAPER, ], '*': [SCRAPY, GOOSE, NEWSPAPER]}
  SCRAPING_MODULE_NAMES.update({'SCRAPYEXTRACTOR': SCRAPING_MODULE_NAMES['SCRAPY']})
  SCRAPING_MODULE_NAMES.update({'GOOSEEXTRACTOR': SCRAPING_MODULE_NAMES['GOOSE']})
  SCRAPING_MODULE_NAMES.update({'NEWSPAPEREXTRACTOR': SCRAPING_MODULE_NAMES['NEWSPAPER']})

  TAG = 'TAG'
  METHOD = 'METHOD'
  OPTIONS = 'OPTIONS'
  ACTION = 'ACTION'
  DEFAULT_VALUE = 'DEFAULT'
  TIMEOUT = 'TIMEOUT'

  METHOD_NAME_HTTP = 'HTTP'

  OPTIONS_REQUEST = 'HTTP_REQUEST'
  REQUEST_NAMES = ['HEAD', 'GET']
  OPTIONS_NAMES = {'CONTENT_TYPES': 'content-type'}

  ACTIONS_TRUNCATE = 0
  ACTIONS_EMPTY = 1
  ACTIONS_DEFAULT = 2
  ACTIONS = {'TRUNCATE': ACTIONS_TRUNCATE, 'EMPTY': ACTIONS_EMPTY, 'DEFAULT': ACTIONS_DEFAULT}

  TITLE = 'title'
  EXTRACTOR = 'extractor'
  DATA = 'data'

  TIMEOUT_DEFAULT = 5

  MSG_RAISE_ISNT_DICT = " isn't a dictionary"

  MSG_ERROR_NOT_DICT = "Value must be a dictionary"
  MSG_ERROR_NOT_LIST = "Value must be a list or string"
  MSG_ERROR_NO_EXTRACTOR = "There isn't an extractor!!!"
  MSG_ERROR_STATUS_CODE = "Status code of response isn't a 200"
  MSG_ERROR_SCRAPING_NAME = "Name of Scraping must be one of: 'Scrapy', 'Goose' or 'Newspaper', took value: "
  MSG_ERROR_METHOD_NAME = "Name of method must be one of "
  MSG_ERROR_REQUEST_NAME = "Name of request must be one of "
  MSG_ERROR_VALUE_IS_INVALID = " value is invalid"

  MSG_DEBUG_PASSED = "There is one of value '%s' in response: '%s' for TAG: '%s'"
  MSG_DEBUG_NO_PASSED = "There isn't one of value '%s' in response: '%s'"
  MSG_DEBUG_ACTIONS_TRUNCATE = "TAG '%s' deleted%s"
  MSG_DEBUG_ACTIONS_EMPTY = "Field 'Data' in tag '%s' changed on empty value %s"
  MSG_DEBUG_ACTIONS_DEFAULT = "Field 'Data' in tag '%s' changed on default value: '%s'"

  # # Initialization
  def __init__(self):
    pass

  # # main process
  #
  # @param result - scraper result instance
  # @param log - logger instance
  # @return - result after scraper tags validation
  @staticmethod
  def process(result, properties, log=None):
    ret = {}

    try:
      ret = copy.deepcopy(result)
      extractor = ''
      properties = Utils.jsonLoadsSafe(jsonString=properties.get(ScraperTagsValidation.SCRAPER_TAGS_VALIDATION),
                                       default=properties.get(ScraperTagsValidation.SCRAPER_TAGS_VALIDATION), log=log)

      if not isinstance(properties, dict):
        raise Exception("'" + str(properties) + "' " + ScraperTagsValidation.MSG_RAISE_ISNT_DICT)
      elif not isinstance(result.tags, dict):
        raise Exception(ScraperTagsValidation.MSG_RAISE_ISNT_DICT)
      elif result.tags.get(ScraperTagsValidation.TITLE):
        extractor = result.tags[ScraperTagsValidation.TITLE].get(ScraperTagsValidation.EXTRACTOR)

      for key, value in properties.items():
        message = ''
        keys = ScraperTagsValidation.SCRAPING_MODULE_NAMES.get(key.upper())

        if not extractor:
          message = ScraperTagsValidation.MSG_ERROR_NO_EXTRACTOR
        if not ScraperTagsValidation.SCRAPING_MODULE_NAMES.get(key.upper()):
          message = ScraperTagsValidation.MSG_ERROR_SCRAPING_NAME + str(key)
        elif not isinstance(value, dict) or not isinstance(result.tags, dict):
          message = ScraperTagsValidation.MSG_ERROR_NOT_DICT
        elif not value.get(ScraperTagsValidation.TAG.upper()) or not isinstance(value.get(ScraperTagsValidation.TAG.upper()), basestring):
          message = ScraperTagsValidation.TAG + ScraperTagsValidation.MSG_ERROR_VALUE_IS_INVALID
        elif not value.get(ScraperTagsValidation.METHOD.upper()) or not isinstance(value.get(ScraperTagsValidation.METHOD.upper()), basestring):
          message = ScraperTagsValidation.METHOD + ScraperTagsValidation.MSG_ERROR_VALUE_IS_INVALID
        elif not value.get(ScraperTagsValidation.OPTIONS.upper()) or not isinstance(value.get(ScraperTagsValidation.OPTIONS.upper()), dict):
          message = ScraperTagsValidation.OPTIONS + ScraperTagsValidation.MSG_ERROR_VALUE_IS_INVALID
        elif not result.tags.get(value.get(ScraperTagsValidation.TAG.upper()).lower()) or extractor not in keys:
          continue

        if message:
          if log is not None:
            log.error(message)
          break

        tag = value.get(ScraperTagsValidation.TAG.upper())
        method = value.get(ScraperTagsValidation.METHOD.upper())
        options = value.get(ScraperTagsValidation.OPTIONS.upper())
        actionName = value.get(ScraperTagsValidation.ACTION.upper()).upper() if value.get(ScraperTagsValidation.ACTION.upper()) else None
        action = ScraperTagsValidation.ACTIONS.get(actionName, ScraperTagsValidation.ACTIONS_TRUNCATE)
        default = value.get(ScraperTagsValidation.DEFAULT_VALUE.upper())
        timeout = value.get(ScraperTagsValidation.TIMEOUT.upper())

        links = result.tags[tag.lower()].get(ScraperTagsValidation.DATA)
        links = [links] if isinstance(links, basestring) else links

        for link in links:
          if not method or method.upper() != ScraperTagsValidation.METHOD_NAME_HTTP:
            message = ScraperTagsValidation.MSG_ERROR_METHOD_NAME + str(ScraperTagsValidation.METHOD_NAME_HTTP)
          elif (not options.get(ScraperTagsValidation.OPTIONS_REQUEST)
                or options.get(ScraperTagsValidation.OPTIONS_REQUEST).upper() not in ScraperTagsValidation.REQUEST_NAMES):
            message = ScraperTagsValidation.MSG_ERROR_REQUEST_NAME + str(ScraperTagsValidation.REQUEST_NAMES)

          if message:
            if log is not None:
              log.error(message)
            break

          timeoutResponse = float(timeout if timeout else ScraperTagsValidation.TIMEOUT_DEFAULT)
          HttpRequest = options.get(ScraperTagsValidation.OPTIONS_REQUEST).upper()
          response = Utils.httpRequestWithTimeout(timeoutResponse, log, HttpRequest, link)

          if not response or response.status_code != 200:
            log.error(ScraperTagsValidation.MSG_ERROR_STATUS_CODE)
            break

          for itemKey, itemValue in options.items():
            itemKey = ScraperTagsValidation.OPTIONS_NAMES.get(itemKey) if ScraperTagsValidation.OPTIONS_NAMES.get(itemKey) else itemKey

            if itemKey == ScraperTagsValidation.OPTIONS_REQUEST:
              continue

            if not isinstance(itemValue, (basestring, list)):
              if log is not None:
                log.error(ScraperTagsValidation.MSG_ERROR_NOT_LIST)
              break

            elif response.headers.get(itemKey.lower().replace('_', '-')) and response.headers.get(itemKey.lower().replace('_', '-')) in itemValue:
              if log is not None:
                log.debug(ScraperTagsValidation.MSG_DEBUG_PASSED, itemValue, response.headers.get(itemKey.lower().replace('_', '-')), tag)

            else:
              defaulyValue = default if default else ''

              if action == ScraperTagsValidation.ACTIONS_TRUNCATE:
                ret.tags.pop(tag.lower())
                message = ScraperTagsValidation.MSG_DEBUG_ACTIONS_TRUNCATE
              elif action == ScraperTagsValidation.ACTIONS_EMPTY:
                ret.tags[tag.lower()][ScraperTagsValidation.DATA] = ''
                message = ScraperTagsValidation.MSG_DEBUG_ACTIONS_EMPTY
              else:
                ret.tags[tag.lower()][ScraperTagsValidation.DATA] = defaulyValue
                message = ScraperTagsValidation.MSG_DEBUG_ACTIONS_DEFAULT

              if log is not None and message:
                log.debug(ScraperTagsValidation.MSG_DEBUG_NO_PASSED, options, response.headers)
                log.debug(message, tag, defaulyValue)
              break

          if result.tags != ret.tags:
            break

    except Exception, err:
      if log is not None:
        log.error(str(err))
        log.info(Utils.getTracebackInfo())

    return ret
