"""@package docstring
 @file goose_extractor.py
 @author Alexey <developers.hce@gmail.com>, alexv
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""

import signal
from goose import Goose
from goose.configuration import Configuration

from dc_processor.base_extractor import BaseExtractor
# from dc_processor.scraper_result import Result
import dc_processor.Constants as CONSTS
from app.Utils import varDump
from app.Utils import ExceptionLog
import app.Utils as Utils  # pylint: disable=F0401
import app.Consts as APP_CONSTS

# Logger initialization
logger = Utils.MPLogger().getLogger()


class GooseExtractor(BaseExtractor):
  goose = None

  def __init__(self, config, templ=None, domain=None, processorProperties=None):
    try:
      BaseExtractor.__init__(self, config, templ, domain, processorProperties)
      logger.debug("Properties: %s", varDump(self.properties))

      self.setName(CONSTS.EXTRACTOR_GOOSE_NAME)
      # set module rank from module's properties
      self.rankReading(self.__class__.__name__)

    except Exception as err:
      ExceptionLog.handler(logger, err, "Goose extractor constructor error: possible /tmp not permitted to write", (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
      raise


  def extractTags(self, resource, result):
    try:
      # set time execution limit
      if isinstance(self.processorProperties, dict) and CONSTS.EXTRACTOR_GOOSE_MAX_EXECUTION_NAME in self.processorProperties:
        self.maxExecutionTimeLimit = int(self.processorProperties[CONSTS.EXTRACTOR_GOOSE_MAX_EXECUTION_NAME])

      logger.debug("Max execution time signal handler set timeout as: %s", str(self.maxExecutionTimeLimit))
      signal.signal(signal.SIGALRM, self.signal_handler)
      signal.alarm(self.maxExecutionTimeLimit)

      userAgent = None
      if isinstance(self.processorProperties, dict) and APP_CONSTS.EXTRACTOR_USER_AGENT in self.processorProperties:
        userAgent = self.processorProperties[APP_CONSTS.EXTRACTOR_USER_AGENT]

      # initialization structure for goose
      configuration = Configuration()
      configuration.http_timeout = self.maxExecutionTimeLimit
      # set user-agent
      if userAgent is not None:
        configuration.browser_user_agent = userAgent
        logger.debug("Set user-agent: %s", str(userAgent))

      # instantiate
      goose = Goose(configuration)
      article = goose.extract(raw_html=str(resource.raw_html), url=resource.url)

      self.addTag(result=result, tag_name=CONSTS.TAG_TITLE, tag_value=article.title)
      self.addTag(result=result, tag_name=CONSTS.TAG_LINK, tag_value=article.canonical_link)
      self.addTag(result=result, tag_name=CONSTS.TAG_PUB_DATE, tag_value=article.publish_date)
      # self.addTag(result=result, tag_name=CONSTS.TAG_GUID, tag_value=article.link_hash)
      self.addTag(result=result, tag_name=CONSTS.TAG_DC_DATE, tag_value=article.additional_data)
      self.addTag(result=result, tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, tag_value=article.cleaned_text)
      self.addTag(result=result, tag_name=CONSTS.TAG_KEYWORDS, tag_value=article.meta_keywords)
      self.addTag(result=result, tag_name=CONSTS.HTML_LANG, tag_value=article.meta_lang)

      if CONSTS.TAG_MEDIA in result.tags.keys() and not self.isTagNotFilled(result, CONSTS.TAG_MEDIA):
        logger.debug("!!! Tag 'media' already selected. Skipped... value = %s", str(result.tags[CONSTS.TAG_MEDIA]))
      else:
        self.addTag(result=result, tag_name=CONSTS.TAG_MEDIA, tag_value=getattr(article, "top_image.src", None))

    except IOError, err:
      ExceptionLog.handler(logger, err, "Goose open file error. It may be unsupported encoding like jp", (), \
                           {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})
    except Exception, err:
      ExceptionLog.handler(logger, err, 'Extraction by goose has error:', (), {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return result
