"""@package docstring
 @file newspaper_extractor.py
 @author Alexey <developers.hce@gmail.com>, alexv
 @link http://hierarchical-cluster-engine.com/
 @copyright Copyright &copy; 2013 IOIX Ukraine
 @license http://hierarchical-cluster-engine.com/license/
 @package HCE project node API
 @since 0.1
"""

import signal
import dc_processor.Constants as CONSTS
from dc_processor.base_extractor import BaseExtractor
# from dc_processor.scraper_result import Result
from dc_processor.NewspaperWrapper import NewspaperWrapper
from app.Utils import varDump
from app.Utils import ExceptionLog
import app.Utils as Utils  # pylint: disable=F0401
import app.Consts as APP_CONSTS

# Logger initialization
logger = Utils.MPLogger().getLogger()


class NewspaperExtractor(BaseExtractor):


  # # Initialization
  def __init__(self, config, templ=None, domain=None, processorProperties=None):
    BaseExtractor.__init__(self, config, templ, domain, processorProperties)

    # self.processorProperties = processorProperties
    logger.debug("Properties: %s", varDump(self.properties))

    # set module rank from module's properties
    self.rankReading(self.__class__.__name__)
    self.setName(CONSTS.EXTRACTOR_NEWSPAPER_NAME)


  def imagesProcessing(self, article):
    ret = None
    if article.top_img is not None:
      ret = []
      ret.append(article.top_img)
      ret.extend([x for x in article.imgs if x != article.top_img])
    else:
      ret = article.imgs

    if ret is not None:
      ret = self.tagValueValidate(CONSTS.TAG_MEDIA, ret)
      if ret is not None:
        localValue = self.imgDelimiter.join(ret)
        ret = []
        ret.append(localValue)

    return ret


  def extractTags(self, resource, result):
    try:
      if resource.raw_html is None:
        raise Exception("resource.raw_html is None")

      if isinstance(resource.raw_html, basestring) and resource.raw_html == "":
        raise Exception("resource.raw_html is empty string")

      # set time execution limit
      if isinstance(self.processorProperties, dict) and CONSTS.EXTRACTOR_NEWSPAPER_MAX_EXECUTION_NAME in self.processorProperties:
        self.maxExecutionTimeLimit = int(self.processorProperties[CONSTS.EXTRACTOR_NEWSPAPER_MAX_EXECUTION_NAME])

      logger.debug("Max execution time signal handler set timeout as: %s", str(self.maxExecutionTimeLimit))
      signal.signal(signal.SIGALRM, self.signal_handler)
      signal.alarm(self.maxExecutionTimeLimit)

      isLoadUrls = False
      if isinstance(self.processorProperties, dict) and CONSTS.SCRAPER_DOWNLOAD_IMAGES_NAME in self.processorProperties:
        isLoadUrls = bool(int(self.processorProperties[CONSTS.SCRAPER_DOWNLOAD_IMAGES_NAME]))

      imageRation = None
      if isinstance(self.processorProperties, dict) and CONSTS.IMAGE_RATION_NAME in self.processorProperties:
        imageRation = float(self.processorProperties[CONSTS.IMAGE_RATION_NAME])

      userAgent = None
      if isinstance(self.processorProperties, dict) and APP_CONSTS.EXTRACTOR_USER_AGENT in self.processorProperties:
        userAgent = self.processorProperties[APP_CONSTS.EXTRACTOR_USER_AGENT]

      # filling initialization parameters dictionary
      kArgs = {"request_timeout":self.maxExecutionTimeLimit}
      if imageRation is not None:
        kArgs["image_dimension_ration"] = imageRation
        logger.debug("Set image_dimension_ration: %s", str(imageRation))

      if userAgent is not None:
        kArgs["browser_user_agent"] = userAgent
        logger.debug("Set user-agent: %s", str(userAgent))

      if CONSTS.TAG_MEDIA in result.tags.keys() and not self.isTagNotFilled(result, CONSTS.TAG_MEDIA):
        logger.debug("Tag '%s' already selected. Skipped...", str(CONSTS.TAG_MEDIA))
        isLoadUrls = False

      article = NewspaperWrapper(" ", config=None, isLoadUrls=isLoadUrls, kwargs=kArgs)

      article.html = resource.raw_html
      article.is_downloaded = True

      # parse execution
      article.parse()
      self.addTag(result=result, tag_name=CONSTS.TAG_TITLE, tag_value=article.title)
      # self.addTag(result=result, tag_name=CONSTS.TAG_LINK, tag_value=article.canonical_link)
      self.addTag(result=result, tag_name=CONSTS.TAG_LINK, tag_value=resource.url)
      self.addTag(result=result, tag_name=CONSTS.TAG_DESCRIPTION, tag_value=article.summary)
      if hasattr(article, "published_date"):
        self.addTag(result=result, tag_name=CONSTS.TAG_PUB_DATE, tag_value=str(article.published_date))  # pylint: disable=E1101
      elif hasattr(article, "publish_date"):
        self.addTag(result=result, tag_name=CONSTS.TAG_PUB_DATE, tag_value=str(article.publish_date))
      self.addTag(result=result, tag_name=CONSTS.TAG_AUTHOR, tag_value=article.authors)
      self.addTag(result=result, tag_name=CONSTS.TAG_DC_DATE, tag_value=article.additional_data)
      self.addTag(result=result, tag_name=CONSTS.TAG_CONTENT_UTF8_ENCODED, tag_value=article.text)
      self.addTag(result=result, tag_name=CONSTS.TAG_KEYWORDS, tag_value=article.meta_keywords)
      self.addTag(result=result, tag_name=CONSTS.HTML_LANG, tag_value=article.meta_lang)

      imgList = self.imagesProcessing(article)
      logger.debug("!!! Tag 'media' imgList: %s", str(imgList))
      if imgList is not None:
        self.addTag(result=result, tag_name=CONSTS.TAG_MEDIA, tag_value=imgList)

    except Exception as err:
      ExceptionLog.handler(logger, err, 'Extraction by newspaper has error:', (), {ExceptionLog.LEVEL_NAME_ERROR:ExceptionLog.LEVEL_VALUE_DEBUG})

    return result
