#!/usr/bin/python
# coding: utf-8

"""
Created on Aug 30, 2019

@author Evgeniy Arefa <developers.hce@gmail.com>
"""

import unittest
import logging
import json
import time
import os
import re
import shutil
import copy
import sys
sys.path.append('../')

from collections import deque

import app.Consts as APP_CONSTS
import app.Utils as Utils


def getLogger(file, encoding='utf8'):
  # create logger
  logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
  logger.setLevel(logging.DEBUG)

  # create file handler and set level to debug
  fh = logging.FileHandler(file, encoding=encoding)
  fh.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

  # add formatter to fh and ch
  fh.setFormatter(formatter)
  ch.setFormatter(formatter)

  # add fh and ch to logger
  logger.addHandler(fh)
  logger.addHandler(ch)

  return logger


class BaseTest(unittest.TestCase):

  FILE_LOG = 'log.txt'
  PATH_FILE_LOG = './' + FILE_LOG
  ERROR = "- ERROR - "
  INFO = "- INFO - "

  def setUp(self):
    self.logger = getLogger(self.FILE_LOG)

  def tearDown(self):
    if os.path.exists(self.PATH_FILE_LOG):
      os.remove(self.PATH_FILE_LOG)

  def _check_log(self, message=None, prefix="- DEBUG - "):
    message = prefix + message if message else None

    if message and os.path.isfile(self.PATH_FILE_LOG):
      with open(self.PATH_FILE_LOG, 'r') as fd:
        lastLine = deque(fd, maxlen=1) or ['']
        self.assertIn(message, lastLine[0])
