"""
Created on Jul 13, 2016

@author: alexander
"""

import unittest
import logging.config
import ConfigParser
from datetime import datetime, timedelta
import sys
sys.path.append('../')

import dc_crawler.DBTasksWrapper as DBTasksWrapper
import dc.EventObjects
from app.FieldsSQLExpressionEvaluator import FieldsSQLExpressionEvaluator
import app.Consts as APP_CONSTS
from dc_crawler.CrawledResource import CrawledResource
from app.Utils import SQLExpression


def getLogger(name='test'):
  # create logger
  logger = logging.getLogger(name)
  logger.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add ch to logger
  logger.addHandler(ch)

  return logger


class Test(unittest.TestCase):
  CONFIG_NAME = "../../ini/db-task.ini"

  SITE_ID = '1234567890'
  URL = 'http://127.0.0.1'

  TEST_URL_PROPERTY_CAST_TO_INTEGER = "[{\"URL\":{\"Status\":{\"IF(%ERRORMASK% & 128 > 0, 1, %STATUS%)\":0}}}]"
  TEST_URL_PROPERTY_CAST_TO_STRING = \
  "[{\"URL\":{\"HTTPMethod\":{\"IF(%ERRORMASK% & 128 > 0, 'post', %HTTPMETHOD%)\":1}}}]"
  TEST_URL_PROPERTY_CAST_TO_DATETIME = \
  "[{\"URL\":{\"PDate\":{\"IF(%ERRORMASK% & 128 > 0, '2016-07-01 00:00:00', %PDATE%)\":2}}}]"
  TEST_URL_PROPERTY_CAST_TO_UNSUPPORT = "[{\"URL\":{\"Status\":{\"IF(%ERRORMASK% & 128 > 0, 1, %STATUS%)\":3}}}]"
  TEST_URL_PROPERTY_WITH_HEADERS = "[{\"URL\":{" \
                                   "\"Status\":{\"IF(%HEADERS% REGEXP 'socket hang up' = 1, 1, %STATUS%)\": 0}, " \
                                   "\"UDate\":{\"IF(%HEADERS% REGEXP 'socket hang up' = 1, DATE_ADD(%UDATE%, INTERVAL 2 HOUR), %UDATE%)\": 2}}}]"

  TEST_SITE_PROPERTY_CAST_TO_INTEGER = "[{\"Site\":{\"Errors\":{\"IF(%ERRORMASK% & 128 > 0, 1, %ERRORS%)\":0}}}]"

  TEST_PDATE_TIME = "[{\"pattern\": \".*bbc.com\/dir1\/.*\", \"value\":\"IF(TIME(%PDATE%)='00:00:01', TIME(NOW()), TIME(%PDATE%))\"}]"

  ERROR_MESSAGE = "Return wrong '{}' value"

  def setUp(self):
    # cfgParser = ConfigParser.ConfigParser()
    # cfgParser.read(self.CONFIG_NAME)

    CONFIG_SECTION = "TasksManager"
    CONFIG_SECTION_2 = "Application"
    CONFIG_SECTION_3 = "Profiler"
    config = ConfigParser.RawConfigParser()
    config.add_section(CONFIG_SECTION)
    config.set(CONFIG_SECTION, "db_host", "127.0.0.1")
    config.set(CONFIG_SECTION, "db_port", "3306")
    config.set(CONFIG_SECTION, "db_user", "hce")
    config.set(CONFIG_SECTION, "db_pwd", "hce12345")
    config.set(CONFIG_SECTION, "primary_db", "dc_sites")
    config.set(CONFIG_SECTION, "secondary_db", "dc_urls")
    config.set(CONFIG_SECTION, "third_db", "")
    config.set(CONFIG_SECTION, "fourth_db", "dc_urls_deleted")
    config.set(CONFIG_SECTION, "fifth_db", "dc_contents")
    config.set(CONFIG_SECTION, "stat_db", "dc_stat_freqs")
    config.set(CONFIG_SECTION, "log_db", "dc_stat_logs")
    config.set(CONFIG_SECTION, "att_db", "dc_attributes")
    config.set(CONFIG_SECTION, "stat_domains_db", "dc_stat_domains")
    config.set(CONFIG_SECTION, "content_storage_type", "1")
    config.set(CONFIG_SECTION, "dc_site_template", "../ini_default/dc_sites_template.sql")
    config.set(CONFIG_SECTION, "dc_content_template", "../ini_default/dc_contents_template.sql")
    config.set(CONFIG_SECTION, "dc_statistics_freq_template", "../ini_default/dc_statistics_freq_template.sql")
    config.set(CONFIG_SECTION, "dc_statistics_log_template", "../ini_default/dc_statistics_log_template.sql")
    config.set(CONFIG_SECTION, "dc_attribute_template", "../ini_default/dc_attributes_template.sql")
    config.set(CONFIG_SECTION, "key_value_storage_dir", "../data_default/dc_dbdata")
    config.set(CONFIG_SECTION, "raw_data_dir", "../data_default/dc_rdata")
    config.set(CONFIG_SECTION, "log_cfg", "../ini_default/db-task_log.ini")
    config.set(CONFIG_SECTION, "key_value_default_file", "../ini_default/db-task_keyvalue_template.db")
    config.add_section(CONFIG_SECTION_2)
    config.set(CONFIG_SECTION_2, "log", "../ini_default/db-task_log.ini")
    config.add_section(CONFIG_SECTION_3)
    config.set(CONFIG_SECTION_3, "profile", "0")
    config.set(CONFIG_SECTION_3, "sortby", "cumulative")
    config.set(CONFIG_SECTION_3, "limit", "0.1")
    config.set(CONFIG_SECTION_3, "traceback", "0")

    self.wrapper = DBTasksWrapper.DBTasksWrapper(config)
    self.logger = getLogger('hce')


  def tearDown(self):
    pass


  # # Get object ('Site' or 'URL')
  #
  # @param objectName - name of object
  # @param fields - dict of names and values for filling of object
  # @return object instance
  def getObject(self, objectName, fields):
    # variable for result
    obj = None

    if objectName == FieldsSQLExpressionEvaluator.OBJECT_NAME_SITE:
      # Create instance of 'Site'
      obj = dc.EventObjects.Site(self.URL)
    elif objectName == FieldsSQLExpressionEvaluator.OBJECT_NAME_URL:
      obj = dc.EventObjects.URL(self.SITE_ID, self.URL)

    if obj is not None:
      for fieldName, fieldValue in fields.items():
        if hasattr(obj, fieldName):
          setattr(obj, fieldName, fieldValue)

    return obj


  # # Get site properties
  #
  # @param propertyName
  # @param propertyValue
  # @return site properties dict
  def getSiteProperties(self, propertyName, propertyValue):
    # variable for result
    ret = {}
    if isinstance(propertyName, basestring) and propertyValue is not None:
      ret[propertyName] = propertyValue

    return ret


  def testURLCastToInteger(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, self.TEST_URL_PROPERTY_CAST_TO_INTEGER
    )

    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {'status': 7, 'errorMask': 128})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict["status"] == 1, self.ERROR_MESSAGE.format('status'))


    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {'status': 7, 'errorMask': 127})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict["status"] == 7, self.ERROR_MESSAGE.format('status'))


  def testURLCastToString(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, self.TEST_URL_PROPERTY_CAST_TO_STRING
    )

    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {'httpMethod': 'post', 'errorMask': 128})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict["httpMethod"] == 'post', self.ERROR_MESSAGE.format('httpMethod'))


    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {'httpMethod': 'get', 'errorMask': 127})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict["httpMethod"] == 'get', self.ERROR_MESSAGE.format('httpMethod'))


  def testURLCastToDatetime(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, self.TEST_URL_PROPERTY_CAST_TO_DATETIME
    )

    urlObj = self.getObject(
      FieldsSQLExpressionEvaluator.OBJECT_NAME_URL,
      {'pDate': '2016-01-01 00:00:00', 'errorMask': 128}
    )
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(
      changedFieldsDict["pDate"] == '2016-07-01 00:00:00',
      self.ERROR_MESSAGE.format('pDate') + ' pDate: ' + str(urlObj.pDate)
    )

    urlObj = self.getObject(
      FieldsSQLExpressionEvaluator.OBJECT_NAME_URL,
      {'pDate': '2016-01-01 00:00:00', 'errorMask': 127}
    )
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(
      changedFieldsDict["pDate"] == '2016-01-01 00:00:00',
      self.ERROR_MESSAGE.format('pDate') + ' pDate: ' + str(urlObj.pDate)
    )


  def testURLCastToUnsupportType(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, self.TEST_URL_PROPERTY_CAST_TO_UNSUPPORT
    )

    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {'status': 4, 'errorMask': 128})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict == {}, self.ERROR_MESSAGE.format('status'))


  def testURLWrongStageForUpdate(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_PROCESSOR, self.TEST_URL_PROPERTY_CAST_TO_UNSUPPORT
    )

    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {'status': 4, 'errorMask': 128})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict == {}, self.ERROR_MESSAGE.format('status'))


  def testSiteCastToInteger(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, self.TEST_SITE_PROPERTY_CAST_TO_INTEGER
    )

    siteObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_SITE, {'errors': 2, 'errorMask': 128})

    self.logger.debug('siteObj.errors = ' + str(siteObj.errors))
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, siteObj, None, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict["errors"] == 1, self.ERROR_MESSAGE.format('errors'))

    siteObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_SITE, {'errors': 2, 'errorMask': 127})
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, siteObj, None, self.logger, APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER
    )
    self.assertTrue(changedFieldsDict["errors"] == 2, self.ERROR_MESSAGE.format('errors'))


  def testPDateTime(self):
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_PDATE_TIME, self.TEST_PDATE_TIME
    )
    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL,
                            {'status': 4, 'errorMask': 0,
                             'pDate': datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                             'url': 'http://bbc.com/dir1/index.html'})
    pubdate = FieldsSQLExpressionEvaluator.evaluatePDateTime(siteProperties, self.wrapper, urlObj, self.logger, None)

    self.logger.debug("pubdate = " + str(pubdate))
    self.assertTrue(pubdate == str(datetime.now().strftime("%Y-%m-%d %H:%M:%S")), self.ERROR_MESSAGE.format('pDate'))


  def testHeaders(self):
    checking_data = datetime.now() + timedelta(hours=1)
    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, self.TEST_URL_PROPERTY_WITH_HEADERS
    )

    urlObj = self.getObject(FieldsSQLExpressionEvaluator.OBJECT_NAME_URL, {
      'status': 7,
      'UDate': datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
      'errorMask': 128
    })
    responseHeader = 'crawledResource.response_header\n' \
                     'Date: Mon, 19 Aug 2019 13:04:50 GMT\n' \
                     'Server: Apache/1.3.42 (Unix) mod_deflate/1.0.21\n' \
                     'Location: https://www.i.ua\n' \
                     'Connection: close\n' \
                     'Transfer-Encoding: chunked\n' \
                     'Content-Type: text/html; charset=windows-1251\n' \
                     'X-lmp-error: socket hang up'
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger,
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, responseHeader
    )
    self.assertTrue(changedFieldsDict["status"] == 1, self.ERROR_MESSAGE.format('status'))
    self.assertGreater(datetime.strptime(changedFieldsDict["UDate"], '%Y-%m-%d %H:%M:%S'),
                       checking_data, self.ERROR_MESSAGE.format('UDate'))

    siteProperties = self.getSiteProperties(
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER,
      self.TEST_URL_PROPERTY_WITH_HEADERS.replace('socket hang up', 'wrong value')
    )
    changedFieldsDict = FieldsSQLExpressionEvaluator.execute(
      siteProperties, self.wrapper, None, urlObj, self.logger,
      APP_CONSTS.SQL_EXPRESSION_FIELDS_UPDATE_CRAWLER, responseHeader
    )
    self.assertTrue(changedFieldsDict["status"] == 7, self.ERROR_MESSAGE.format('status'))
    self.assertLess(datetime.strptime(changedFieldsDict["UDate"], '%Y-%m-%d %H:%M:%S'),
                    checking_data, self.ERROR_MESSAGE.format('UDate'))


if __name__ == "__main__":
  # import sys;sys.argv = ['', 'Test.testName']
  unittest.main()
