#!/usr/bin/python
# coding: utf-8

import logging
import datetime

from dc_crawler.HTTPHeaders import HTTPHeaders
from dc_crawler.CrawledResource import CrawledResource
import app.Consts as APP_CONSTS


def getLogger():
  # create logger
  log = logging.getLogger()
  log.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add ch to logger
  log.addHandler(ch)

  return log


if __name__ == '__main__':

  logger = getLogger()

  responseHeader = ''

  baseUrl = 'http://test.html'
  pubdate = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

  HTTPHeaders.addBaseUrlToHeader(responseHeader, baseUrl)
  responseHeader = HTTPHeaders.addPubdateRssFeedToHeader(responseHeader, pubdate)
  responseHeader = HTTPHeaders.addPubdateRssRawToHeader(responseHeader, pubdate)
  logger.debug("headers:\n%s", str(responseHeader))

  crawledResource = CrawledResource()
  HTTPHeaders.addBaseUrlToHeader(crawledResource, baseUrl + '_1')
  logger.debug("crawledResource.response_header:\n%s", str(crawledResource.response_header))

  HTTPHeaders.addRequestDateToHeader(crawledResource, requestDate=datetime.datetime.now(), log=logger)
  logger.debug("crawledResource.response_header:\n%s", str(crawledResource.response_header))

  HTTPHeaders.addHeaderToHeader(crawledResource, responseHeader)
  logger.debug("crawledResource.response_header:\n%s", str(crawledResource.response_header))

  headersDict = HTTPHeaders.headerToDict(crawledResource.response_header)
  logger.debug("headersDict: %s", str(headersDict))

  restoredHeader = HTTPHeaders.dictToHeader(headersDict)
  logger.debug("restoredHeader:\n%s", str(restoredHeader))

  restoredHeader = HTTPHeaders.setValue(restoredHeader, HTTPHeaders.BASE_URL_NAME, baseUrl + '_2')
  logger.debug("after setValue restoredHeader:\n%s", str(restoredHeader))
  updatedBaseUrl = HTTPHeaders.getValue(restoredHeader, HTTPHeaders.BASE_URL_NAME)
  logger.debug("after getValue updatedBaseUrl: %s", str(updatedBaseUrl))

