#!/usr/bin/python
# coding: utf-8

"""
Created on Sep 04, 2019

@author Evgeniy Arefa <developers.hce@gmail.com>
"""

from base.BaseTest import *

from dc_crawler.HTTPHeadersExtractor import HTTPHeadersExtractor


class Test(BaseTest):
  MY_IP = Utils.getIP()
  TESTCOUNT = 0
  PROPERTIES = []
  HEADERS = {"User-Agent:My bot"}
  URL = 'http://test.com/test.html'
  LOCAL_PATH = '/tmp/dc_crawler'
  LOCAL_FILE_PATH = 'file://' + LOCAL_PATH + '/test_headers.txt'
  STORAGE_DIR = LOCAL_PATH + '/headers'
  SITE_ID = '0000111'
  DOMAIN = 'test'
  TEMPORARY_FILE = os.path.join(STORAGE_DIR, DOMAIN, str(SITE_ID) + '.json')

  EXPECTED_RESULT_0 = {"User-Agent": "My bot"}
  EXPECTED_RESULT_1 = [{'User-Agent': 'my-bot'}, {'User-Agent': 'Geko-bot'}, {'User-Agent': 'some-bot'}]
  EXPECTED_RESULT_2 = {"User-Agent": 'my-bot', 'Pragma': 'cache'}
  EXPECTED_RESULT_3 = {
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "en-US,en;q=0.8,en;q=0.6,us;q=0.4,us;q=0.2,ja;q=0.2",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome 51.0.2704.84 Safari/537.36",
    "--disable-web-security": ""
  }
  EXPECTED_RESULT_4 = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.2704.84 Safari/537.36"
  }
  EXPECTED_RESULT_5 = [
    {'User-Agent': 'my-bot', 'Pragma': 'cache'}, {'User-Agent': 'Geko-bot', 'Pragma': 'cache'}, {'User-Agent': 'some-bot', 'Pragma': 'cache'},
    {'User-Agent': 'my-bot', 'Pragma': 'no cache'}, {'User-Agent': 'Geko-bot', 'Pragma': 'no cache'}, {'User-Agent': 'some-bot', 'Pragma': 'no cache'}
  ]

  FILE_PROPERTIES_0 = {"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}
  FILE_PROPERTIES_1 = "my-bot\r\n" \
                      "Geko-bot\r\n" \
                      "some-bot"
  FILE_PROPERTIES_2 = [{"User-Agent": "my-bot"}, {"Pragma": "cache"}]
  FILE_PROPERTIES_3 = "Accept-Encoding:gzip, deflate\r\n" \
                      "Accept-Language:en-US,en;q=0.8,en;q=0.6,us;q=0.4,us;q=0.2,ja;q=0.2\r\n" \
                      "User-Agent:Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome 51.0.2704.84 Safari/537.36\r\n" \
                      "--disable-web-security:\r\n" \
                      "#--log-chrome-debug-log:\r\n"

  MSG_RAISE_ISNT_LIST = HTTPHeadersExtractor.MSG_RAISE_ISNT_LIST

  MSG_ERROR_ISNT_STRING = HTTPHeadersExtractor.MSG_ERROR_ISNT_STRING.strip("'%s'")
  MSG_ERROR_ISNT_SYMBOL = HTTPHeadersExtractor.MSG_ERROR_ISNT_SYMBOL.split("'%s'")[0]
  MSG_RAISE_ISNT_NAME_VALUE = HTTPHeadersExtractor.MSG_RAISE_ISNT_NAME_VALUE
  MSG_RAISE_VALUE_MUST_BE_LIST = HTTPHeadersExtractor.MSG_RAISE_VALUE_MUST_BE_LIST
  MSG_ERROR_EXPECTED = 'expected string or buffer'
  MSG_ERROR_FIRST_ARGUMENT = 'first argument must be string or compiled pattern'
  MSG_ERROR_NO_UPDATE = "object has no attribute 'update'"
  MSG_ERROR_NOT_SUPPORT = 'object does not support item assignment'
  MSG_ERROR_ARGUMENT = 'argument of type '
  MSG_ERROR_NOT_ITERABLE = 'is not iterable'


  MSG_DEBUG_DOMAIN_PASSED = HTTPHeadersExtractor.MSG_DEBUG_DOMAIN_PASSED.strip('%s') + URL
  MSG_DEBUG_DOMAIN_NOT_PASSED = HTTPHeadersExtractor.MSG_DEBUG_DOMAIN_NOT_PASSED.strip('%s') + URL

  def setUp(self):
    super(Test, self).setUp()
    if os.path.exists(self.LOCAL_PATH):
      shutil.rmtree(self.LOCAL_PATH, ignore_errors=True)

  def tearDown(self):
    super(Test, self).tearDown()
    if os.path.exists(self.LOCAL_PATH):
      shutil.rmtree(self.LOCAL_PATH, ignore_errors=True)

  @staticmethod
  def init():
    pass

  @staticmethod
  def getSiteProperties(inputData):
    return {APP_CONSTS.HTTP_HEADERS: inputData}

  @staticmethod
  def fillFileHeaders(data, log=None):
    fileName = Test.LOCAL_FILE_PATH
    if fileName.startswith('file://'):
      parts = fileName.split('file://')
      if len(parts) > 1:
        fileName = parts[1]

    if log is not None:
      log.debug("Header file name: %s", str(fileName))

    dirName = os.path.dirname(fileName)
    if not os.path.isdir(dirName):
      os.makedirs(dirName)

    fileData = data if isinstance(data, basestring) else json.dumps(data)
    if log is not None:
      log.debug("Header file data: %s", fileData)

    f = open(fileName, 'w')
    f.write(fileData)
    f.close()

  def checkHeaders(self, inputData, expectedResult):
    siteHeaders = HTTPHeadersExtractor.extract(
      siteProperties=inputData, url=self.URL, headers=self.HEADERS, log=self.logger, siteId=self.SITE_ID, domain=self.DOMAIN
    )
    if isinstance(expectedResult, list):
      self.assertIn(siteHeaders, expectedResult)
    elif isinstance(expectedResult, dict):
      self.assertEqual(siteHeaders, expectedResult)
    else:
      self.assertTrue(False, "Unknown type of 'expectedResult'!!!")

  def testExtract(self):
    # "HTTP_HEADERS": "<local_file_path>" - [{"User-Agent": "my-bot"}, {"Pragma": "cache"}]
    Test.fillFileHeaders(self.FILE_PROPERTIES_2, self.logger)
    self.checkHeaders(self.getSiteProperties(Test.LOCAL_FILE_PATH), self.EXPECTED_RESULT_2)

    # "HTTP_HEADERS": "<local_file_path>" - "Accept-Encoding": "...", "Accept-Language": "...", "User-Agent": "...", "--disable-web-security": ""
    Test.fillFileHeaders(self.FILE_PROPERTIES_3, self.logger)
    self.checkHeaders(self.getSiteProperties(Test.LOCAL_FILE_PATH), self.EXPECTED_RESULT_3)

    # "HTTP_HEADERS": "<local_file_path>" - {"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}
    Test.fillFileHeaders(self.FILE_PROPERTIES_0, self.logger)
    self.checkHeaders(self.getSiteProperties(Test.LOCAL_FILE_PATH), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {"User-Agent":["my-bot","Geko-bot",…]}
    inputData = {'User-Agent': ['my-bot', 'Geko-bot', 'some-bot']}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {"User-Agent":"<local_file_path>"} - {"Pragma": "cache", "User-Agent":["my-bot","Geko-bot","some-bot"]}
    newProperies = {"Pragma": "cache", "User-Agent": ["my-bot", "Geko-bot", "some-bot"]}
    Test.fillFileHeaders(newProperies, self.logger)
    inputData = {"User-Agent": Test.LOCAL_FILE_PATH}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {"User-Agent":"<local_file_path>"} - "my-bot \r\n Geko-bot \r\n some-bot"
    Test.fillFileHeaders(self.FILE_PROPERTIES_1, self.logger)
    inputData = {"User-Agent": Test.LOCAL_FILE_PATH}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {"User-Agent":"My bot"}
    inputData = {"User-Agent": "My bot"}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_0)

    # "HTTP_HEADERS": {"User-Agent":{"127.0.0.1":"My bot"}}
    inputData = {"User-Agent": {self.MY_IP: "My bot"}}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_0)

    # "HTTP_HEADERS": {"User-Agent":{"127.0.0.1":["my-bot","Geko-bot",…]}}
    inputData = {"User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {"User-Agent":{"127.0.0.1":"<local_file_path>"}} - {"User-Agent":{"127.0.0.1":["my-bot","Geko-bot","some-bot"]}}
    Test.fillFileHeaders(self.FILE_PROPERTIES_0, self.logger)
    inputData = {"User-Agent": {self.MY_IP: Test.LOCAL_FILE_PATH}}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": [{"User-Agent":{"127.0.0.1":"My bot"}},{"User-Agent":{"111.1.1.1":["my-bot","Geko-bot","some-bot"]}}]
    inputData = "[{\"User-Agent\": {\"%s\": \"My bot\"}}, {\"User-Agent\": {\"111.1.1.1\": [\"my-bot\", \"Geko-bot\", \"some-bot\"]}}]" % self.MY_IP
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_0)

    # "HTTP_HEADERS": [{"User-Agent":{"127.0.0.1":"My bot"}},{"User-Agent":{"111.1.1.1":["my-bot","Geko-bot","some-bot"]}}]
    # inputData = "[{\"User-Agent\": {\"%s\": \"My bot\"}}, {\"User-Agent\": {\"111.1.1.1\": [\"my-bot\", \"Geko-bot\", \"some-bot\"]}}]" % self.MY_IP
    if os.path.exists(self.LOCAL_PATH):
      shutil.rmtree(self.LOCAL_PATH, ignore_errors=True)
    inputData = ("[{\"storage\":\"%s\", \"domain\":\"https.*\", \"headers\": {\"User-Agent\":\"%s\"}}, {\"domain\":\"http.*\", \"headers\": "
                 "{\"User-Agent\":{\"%s\":[\"my-bot\",\"Geko-bot\",\"some-bot\"]}}}]" % (self.STORAGE_DIR, Test.LOCAL_FILE_PATH, self.MY_IP))
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": [{"User-Agent":{"127.0.0.1":"My bot"}},{"Pragma":{"111.1.1.1":["cache","no cache"]}}]
    inputData = [{"User-Agent": {self.MY_IP: "My bot"}}, {"Pragma": {"111.1.1.1": ["cache", "no cache"]}}]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_0)

    # "HTTP_HEADERS": [{"User-Agent":["my-bot","Geko-bot","some-bot"]},{"Pragma":["cache", "no cache"]}]
    inputData = [{"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}, {"Pragma": ["cache", "no cache"]}]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_5)

    # "HTTP_HEADERS": [{"User-Agent":{"127.0.0.1":["my-bot","Geko-bot","some-bot"]}},{"Pragma":["cache","no cache"]}]
    inputData = [{"User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}, {"Pragma": ["cache", "no cache"]}]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_5)

    # "HTTP_HEADERS": [{"User-Agent":{"127.0.0.1":["my-bot","Geko-bot","some-bot"]}},{"Pragma":{"127.0.0.1":["cache","no cache"]}}]
    inputData = [{"User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}, {"Pragma": {self.MY_IP: ["cache", "no cache"]}}]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_5)

    # "HTTP_HEADERS": [{"User-Agent":{"127.0.0.1":["my-bot","Geko-bot","some-bot"]}},{"Pragma":{"111.1.1.1":["cache","no cache"]}}]
    inputData = [{"User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}, {"Pragma": {"111.1.1.1": ["cache", "no cache"]}}]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {“domain”:”http(.*)”, “headers”: {"User-Agent":"<local_file_path>"}} - "my-bot \r\n Geko-bot \r\n some-bot"
    Test.fillFileHeaders(self.FILE_PROPERTIES_1, self.logger)
    inputData = {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
                 HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": Test.LOCAL_FILE_PATH}}
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {“domain”:”http(.*)”, “headers”: {"User-Agent":{"127.0.0.1":["my-bot","Geko-bot",…]}}}
    inputData = {
      HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
      HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}
    }
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS":[{“domain”:”https.*”,“headers”:{"User-Agent": "My bot"}},{“domain”:”http.*”,“headers”:{"User-Agent":"<local_file_path>"}}]
    Test.fillFileHeaders(self.FILE_PROPERTIES_0, self.logger)
    inputData = [
      {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "https.*", HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": "My bot"}},
      {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*", HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": Test.LOCAL_FILE_PATH}}
    ]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": [{“domain”:”http.*”, “headers”: {"User-Agent":["my-bot", "Geko-bot", "some-bot"]}}]
    inputData = [
      {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*", HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}}
    ]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": [
    # {“domain”:”http.*”, “headers”:[{"User-Agent":{"111.1.1.1":"My bot"}},{"User-Agent":{"127.0.0.1":["my-bot","Geko-bot","some-bot"]}}]}
    # {“domain”:”https.*”,“headers”:{"User-Agent":"My BOT"}}
    # ]
    inputData = [
      {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
       HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: [{"User-Agent": {self.MY_IP: "My bot"}}, {"User-Agent": {"111.1.1.1":["my-bot", "Geko-bot", "some-bot"]}}]},
      {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "https.*",
       HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": "My BOT"}}
    ]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_0)

    # "HTTP_HEADERS": [{“domain”:”http.*”, “headers”: {"Accept":"...", "User-Agent":"..."}}]
    inputData = [
      {
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {
          "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
          "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.2704.84 Safari/537.36"
        }
      }
    ]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_4)

    # "HTTP_HEADERS": [{“domain”:”http.*”, “headers”: {"Accept":"...", "User-Agent":{"127.0.0.1":"..."}}}]
    inputData = [
      {
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {
          "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
          "User-Agent": {self.MY_IP:"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.2704.84 Safari/537.36"}
        }
      }
    ]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_4)

    # "HTTP_HEADERS": {“domain”:”.*test.com.*”, “headers”: {"User-Agent":"<local_file_path>"}} - "my-bot \r\n Geko-bot \r\n some-bot"
    Test.fillFileHeaders(self.FILE_PROPERTIES_1, self.logger)
    inputData = {
      HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: ".*test.com.*",
      HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": Test.LOCAL_FILE_PATH}
    }
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)

    # "HTTP_HEADERS": {"storage":”<storage_value>”, “domain”:”http.*”, “headers”: {"User-Agent":["my-bot", "Geko-bot", "some-bot"]}}
    inputData = {
      HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR,
      HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
      HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}
    }
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)
    self.assertTrue(os.path.exists(self.TEMPORARY_FILE))
    with open(self.TEMPORARY_FILE, 'r') as fd:
      nodeElem = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.STORAGE_DIR: 0}, log=self.logger)
    self.assertEqual(nodeElem, {'User-Agent': {'1': 0, '0': 1, '2': 0}})

    # "HTTP_HEADERS": [{"storage":”<storage_value>”, “domain”:”http.*”, “headers”: {"User-Agent":["my-bot", "Geko-bot", "some-bot"]}}]
    inputData = [
      {
        HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR,
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}
      }
    ]
    self.checkHeaders(self.getSiteProperties(inputData), self.EXPECTED_RESULT_1)
    self.assertTrue(os.path.exists(self.TEMPORARY_FILE))
    with open(self.TEMPORARY_FILE, 'r') as fd:
      nodeElem = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.STORAGE_DIR: 0}, log=self.logger)
    self.assertEqual(nodeElem, {'User-Agent': {'1': 1, '0': 1, '2': 0}})

    # "HTTP_HEADERS": [{"storage":”<storage_value>”, “domain”:”http.*”, “headers”: {"User-Agent":"<local_file_path>"}}]
    Test.fillFileHeaders(self.FILE_PROPERTIES_0, self.logger)
    inputData = [
      {
        HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR,
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": Test.LOCAL_FILE_PATH}
      }
    ]
    self.checkHeaders(Test.getSiteProperties(inputData), self.EXPECTED_RESULT_1)
    self.assertTrue(os.path.exists(self.TEMPORARY_FILE))
    with open(self.TEMPORARY_FILE, 'r') as fd:
      nodeElem = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.STORAGE_DIR: 0}, log=self.logger)
    self.assertEqual(nodeElem, {'User-Agent': {'1': 1, '0': 1, '2': 1}})

    # "HTTP_HEADERS": [
    # {"storage":”<storage_value>”, “domain”:”http.*”, “headers”: {"User-Agent": "<local_file_path>", "Pragma": ["cache", "no cache"]}},
    # {"storage":”<storage_value>”, “domain”:”https.*”, “headers”: {"Accept-Encoding": ["gzip, deflate", "default"]}}
    # ]
    Test.fillFileHeaders(self.FILE_PROPERTIES_0, self.logger)
    inputData = [
      {
        HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR,
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": Test.LOCAL_FILE_PATH, "Pragma": ["cache", "no cache"]}
      },
      {
        HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR,
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "https.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"Accept-Encoding": ["gzip, deflate", "default"]}
      }
    ]
    self.checkHeaders(Test.getSiteProperties(inputData), self.EXPECTED_RESULT_5)
    self.assertTrue(os.path.exists(self.TEMPORARY_FILE))
    with open(self.TEMPORARY_FILE, 'r') as fd:
      nodeElem = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.STORAGE_DIR: 0}, log=self.logger)
    self.assertEqual(nodeElem, {'User-Agent': {'0': 2, '1': 1, '2': 1}, 'Pragma': {'0': 1, '1': 0}})

  def testExtractWrongData(self):
    inputData = 'file:///tmp/test_headers_9.txt'
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = ''
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = None
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {'User-Agent': ['my-bot', 'Geko-bot', 'some-bot']}
    self.checkHeaders(inputData, {})

    inputData = {'User-Agent': {'User-Agent': ['my-bot', 'Geko-bot', 'some-bot']}}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {'User-Agent': None}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {'User-Agent': {'111.111.9.11': 'My bot'}}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*", "User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": ["my-bot", "Geko-bot", "some-bot"]}}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*", "User-Agent": {self.MY_IP: ["my-bot", "Geko-bot", "some-bot"]}}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = [
      {HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "http.*"},
      {HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8"}}
    ]
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = {HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR}
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = [{HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR}, {'User-Agent': 'My bot'}]
    self.checkHeaders(self.getSiteProperties(inputData), {})

    inputData = [
      {
        HTTPHeadersExtractor.PROPERTY_STORAGE_NAME: self.STORAGE_DIR,
        HTTPHeadersExtractor.PROPERTY_DOMAIN_NAME: "https.*",
        HTTPHeadersExtractor.PROPERTY_HEADERS_NAME: {"User-Agent": Test.LOCAL_FILE_PATH}
      }
    ]
    self.checkHeaders(self.getSiteProperties(inputData), {})

  def testPlainTextToDict(self):
    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value'], log=self.logger), {'Test': 'value'})

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', '123: 567'], log=self.logger), {'Test': 'value', '123': ' 567'})

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', ':123'], log=self.logger), {'Test': 'value'})

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', '123:'], log=self.logger), {'Test': 'value', '123': ''})

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', '#--123:567', '--cat:dog'], log=self.logger), {'Test': 'value', '--cat': 'dog'})

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', '--123:567'], log=self.logger), {'Test': 'value', '--123': '567'})

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', 34, '123:'], log=self.logger), {'Test': 'value', '123': ''})
    self._check_log("'34'" + self.MSG_ERROR_ISNT_STRING, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value', '123:456:789'], log=self.logger), {'Test': 'value'})
    self._check_log(self.MSG_ERROR_ISNT_SYMBOL, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict('Test:value', log=self.logger), {})
    self._check_log("'Test:value'" + self.MSG_RAISE_ISNT_LIST, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict({'Test': 'value'}, log=self.logger), {})
    self._check_log("'{'Test': 'value'}'" + self.MSG_RAISE_ISNT_LIST, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(12345, log=self.logger), {})
    self._check_log("'12345'" + self.MSG_RAISE_ISNT_LIST, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict([''], log=self.logger), {})
    self._check_log(self.MSG_ERROR_ISNT_SYMBOL, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(None, log=self.logger), {})
    self._check_log("'None'" + self.MSG_RAISE_ISNT_LIST, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict([['Test:value'], ['--123:567']], log=self.logger), {})
    self._check_log("'['--123:567']'" + self.MSG_ERROR_ISNT_STRING, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.plainTextToDict(['Test:value'], ['--123:567']), {'Test': 'value'})

  def testcheckDomain(self):
    self.assertEqual(HTTPHeadersExtractor.checkDomain('http.*', True, self.URL, log=self.logger), True)
    self._check_log(self.MSG_DEBUG_DOMAIN_PASSED.replace('%s', 'http.*'))

    self.assertEqual(HTTPHeadersExtractor.checkDomain('.*test.html', True, self.URL, log=self.logger), True)
    self._check_log(self.MSG_DEBUG_DOMAIN_PASSED.replace('%s', '.*test.html'))

    self.assertEqual(HTTPHeadersExtractor.checkDomain('test.com', True, self.URL, log=self.logger), True)
    self._check_log(self.MSG_DEBUG_DOMAIN_PASSED.replace('%s', 'test.com'))

    self.assertEqual(HTTPHeadersExtractor.checkDomain('https.*', True, self.URL, log=self.logger), {})
    self._check_log(self.MSG_DEBUG_DOMAIN_NOT_PASSED.replace('%s', 'https.*'))

    self.assertEqual(HTTPHeadersExtractor.checkDomain('http.*', True, '', log=self.logger), {})
    self._check_log(self.MSG_DEBUG_DOMAIN_NOT_PASSED.replace('%s', 'http.*').split(self.URL)[0])

    self.assertEqual(HTTPHeadersExtractor.checkDomain('http.*', True, None, log=self.logger), {})
    self._check_log(self.MSG_ERROR_EXPECTED, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.checkDomain('http.*', True, 1234, log=self.logger), {})
    self._check_log(self.MSG_ERROR_EXPECTED, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.checkDomain('http.*', True, [self.URL], log=self.logger), {})
    self._check_log(self.MSG_ERROR_EXPECTED, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.checkDomain('http.*', True, {self.URL: self.URL}, log=self.logger), {})
    self._check_log(self.MSG_ERROR_EXPECTED, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.checkDomain('', True, self.URL, log=self.logger), True)
    self._check_log(self.MSG_DEBUG_DOMAIN_PASSED.replace('%s', ''))

    self.assertEqual(HTTPHeadersExtractor.checkDomain(1234, True, self.URL, log=self.logger), {})
    self._check_log(self.MSG_ERROR_FIRST_ARGUMENT, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.checkDomain(None, True, self.URL, log=self.logger), {})
    self._check_log(self.MSG_ERROR_FIRST_ARGUMENT, self.ERROR)

    self.assertEqual(HTTPHeadersExtractor.checkDomain(None, True, None, log=self.logger), {})
    self._check_log(self.MSG_ERROR_FIRST_ARGUMENT, self.ERROR)

  def testUseFrequencyAlgorithm(self):
    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {'User-Agent': '1-bot'})
    self.assertEqual(nodeElem, {'User-Agent': {'0': 1, '1': 0, '2': 0}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 3, '1': 3, '2': 2}}, 'User', ['1-bot', '2-bot', '3-bot'])
    self.assertEqual(siteHeaders, {'User': '3-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 3, '1': 3, '2': 3}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 3, '1': 3, '2': 3}}, 'User', ['1-bot', '2-bot', '3-bot'])
    self.assertEqual(siteHeaders, {'User': '1-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 4, '1': 3, '2': 3}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 4, '1': 3, '2': 3}}, 'User', ['1-bot', '2-bot', '3-bot'])
    self.assertEqual(siteHeaders, {'User': '2-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 4, '1': 4, '2': 3}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 4, '1': 4, '2': 3}}, 'User', ['1-bot', '2-bot', '3-bot'])
    self.assertEqual(siteHeaders, {'User': '3-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 4, '1': 4, '2': 4}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm(
      {'Pragma': 'cache'}, {'User': {'0': 2, '1': 1, '2': 1}}, 'User', ['1-bot', '2-bot', '3-bot'], log=self.logger
    )
    self.assertEqual(siteHeaders, {'User': '2-bot', 'Pragma': 'cache'})
    self.assertEqual(nodeElem, {'User': {'0': 2, '1': 2, '2': 1}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 3, '1': 1, '2': 4}}, 'User', ['1-bot', '2-bot', '3-bot'])
    self.assertEqual(siteHeaders, {'User': '2-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 3, '1': 2, '2': 4}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': -1, '1': 1, '2': 4}}, 'User', ['1-bot', '2-bot', '3-bot'])
    self.assertEqual(siteHeaders, {'User': '1-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 0, '1': 1, '2': 4}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 3, '1': 5, '2': 1}}, 'User', ['1-bot', '2-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {'User': '1-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 1, '1': 0}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': { '0': 2, '1': 3,'2': 1}}, 'User', ['1-bot', '2-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {'User': '1-bot'})
    self.assertEqual(nodeElem, {'User': {'0': 1, '1': 0}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {'User': {'0': 3, '1': 3}}, 'User-Agent', ['1-b', '2-b', '3-b'], log=self.logger)
    self.assertEqual(siteHeaders, {'User-Agent': '1-b'})
    self.assertEqual(nodeElem, {'User': {'0': 3, '1': 3}, 'User-Agent': {'0': 1, '1': 0, '2': 0}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, None, 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {'User-Agent': '1-bot'})
    self.assertEqual(nodeElem, {'User-Agent': {'1': 0, '0': 1, '2': 0}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm(None, {}, 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {'User-Agent': '1-bot'})
    self.assertEqual(nodeElem, {'User-Agent': {'1': 0, '0': 1, '2': 0}})

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm('Nothing', {}, 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, 'Nothing')
    self.assertEqual(nodeElem, {'User-Agent': {'1': 0, '0': 0, '2': 0}})
    self._check_log("'str' " + self.MSG_ERROR_NO_UPDATE, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm(1234, {}, 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, 1234)
    self.assertEqual(nodeElem, {'User-Agent': {'1': 0, '0': 0, '2': 0}})
    self._check_log("'int' " + self.MSG_ERROR_NO_UPDATE, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, 'Nothing', 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, 'Nothing')
    self._check_log("'str' " + self.MSG_ERROR_NOT_SUPPORT, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, 1234, 'User-Agent', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, 1234)
    self._check_log(self.MSG_ERROR_ARGUMENT + "'int' " + self.MSG_ERROR_NOT_ITERABLE, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, None, ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_ISNT_NAME_VALUE, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, '', ['1-bot', '2-bot', '3-bot'], log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_ISNT_NAME_VALUE, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, 'User-Agent', None, log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_VALUE_MUST_BE_LIST, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, 'User-Agent', '', log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_VALUE_MUST_BE_LIST, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, 'User-Agent', '1-bot', log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_VALUE_MUST_BE_LIST, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, 'User-Agent', {'value': '1-bot'}, log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_VALUE_MUST_BE_LIST, self.ERROR)

    siteHeaders, nodeElem = HTTPHeadersExtractor.useFrequencyAlgorithm({}, {}, 'User-Agent', 1234, log=self.logger)
    self.assertEqual(siteHeaders, {})
    self.assertEqual(nodeElem, {})
    self._check_log(self.MSG_RAISE_VALUE_MUST_BE_LIST, self.ERROR)


if __name__ == '__main__':
  unittest.main()
