#!/usr/bin/python

import sys
import json
import ConfigParser
import logging
from dc_crawler.DBTasksWrapper import DBTasksWrapper
from dc_crawler.DBProxyWrapper import DBProxyWrapper
from dc_crawler.HTTPProxyResolver import HTTPProxyResolver
from app.Utils import varDump


def getLogger():
  # create logger
  log = logging.getLogger('hce')
  log.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(funcName)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add ch to logger
  log.addHandler(ch)

  return log


def getDBWrapper(configFileName):
  # variable for result
  ret = None
  try:
    configParser = ConfigParser.ConfigParser()
    configParser.read(configFileName)
    ret = DBTasksWrapper(configParser)
  except Exception, err:
    sys.stderr.write(str(err) + '\n')

  return ret


logger = getLogger()

# # initialization DBProxyWrapper
configFile = '../../ini_default/db-task.ini'
dbWrapper = getDBWrapper(configFile)
dbProxyWrapper = DBProxyWrapper(dbWrapper)

logger.info("dbWrapper: %s", varDump(dbWrapper))
logger.info("dbProxyWrapper: %s", varDump(dbProxyWrapper))

# userProxyDict = {"source": 0, "proxies":{"proxy1.hce-project.com:3129":{"host":"proxy1.hce-project.com:3129", "domains": ["*"], "priority":10, "limits":None}}}
userProxyDict = {"tries_count":2, "source":0, "proxies":{"proxy1.hce-project.com:3129":{"host":"proxy1.hce-project.com:3129", "domains":["economistsview.typepad.com", "delong.typepad.com", "www.bluestemprairie.com", "socialistunity.com", "www.ridenbaugh.com", "ridenbaugh.com", "www.aclu.org", "www.economist.com", "economist.com", "www.wcvb.com", "wcvb.com", "www.wmur.com", "wmur.com", "feeds.washingtonpost.com", "www.washingtonpost.com", "washingtonpost.com", "www.latimes.com", "latimes.com", "www.moneycontrol.com", "moneycontrol.com", "www.theamericanconservative.com", "theamericanconservative.com", "time.com", "feeds.slate.com", "slate.com", "www.chicagotribune.com", "chicagotribune.com", "www.usnews.com", "usnews.com", "conservativehome.blogs.com", "www.sun-sentinel.com", "sun-sentinel.com", "www.redstate.com", "redstate.com", "www.powerlineblog.com", "powerlineblog.com", "www.npr.org", "npr.org", "www.channelnewsasia.com", "channelnewsasia.com", "www.buzzfeed.com", "buzzfeed.com", "www.cbs46.com", "cbs46.com", "www.wnem.com", "wnem.com", "www.kptv.com", "kptv.com", "www.kctv5.com", "kctv5.com"], "priority":44, "limits":None}}}
siteProperties = {"USER_PROXY": json.dumps(userProxyDict)}

# siteId = '0'
siteId = None
# url = 'localhost'
# url = 'http://www.economist.com/sections/united-states/rss.xml'
url = 'https://www.channelnewsasia.com/news/asia/thailand-must-postpone-election-again-until-march-officials-11125800'

# proxyName = HTTPProxyResolver.getProxy(siteProperties=siteProperties, siteId=siteId, url=url, dbProxyWrapper=dbProxyWrapper)
# HTTPProxyResolver.addFaults(siteProperties=siteProperties, siteId=siteId, proxyName=proxyName, dbProxyWrapper=dbProxyWrapper)
# logger.info("proxyResolver.getProxy() = %s", str(proxyName))

retriesCount = HTTPProxyResolver.getTriesCount(siteProperties)
logger.debug("retriesCount = %s", str(retriesCount))

proxyName, isValid = HTTPProxyResolver.getProxyName(siteProperties=siteProperties, siteId=siteId, url=url, dbWrapper=dbWrapper, log=logger)
logger.info("Result proxyName: %s, isValid = %s", str(proxyName), str(isValid))
