"""
Created on Aug 27, 2019

@author Evgeniy Arefa <developers.hce@gmail.com>
"""


from base.BaseTest import *

from app.HostRequestStorage import HostRequestStorage


class Test(BaseTest):
  MY_IP = Utils.getIP()
  URL = 'https://www.mypage/'
  SITE_ID = '7f130da2d40f3722ab12458f18ed8a83'
  FILE_SUFF = ".json"
  PATH_STORAGE_KEY = "storage"
  HTTP_FILE_PREFIX = "http_limits_"
  LAST_START = "last_start"

  HTTP_FREQ_LIMITS_1 = {
    PATH_STORAGE_KEY: "/tmp/",
    ".*www.another_page.*": {
      MY_IP: {
        "randomized": 1,
        "min_delay": 30,
        "max_delay": 50
      }
    }
  }
  HTTP_FREQ_LIMITS_2 = {
    PATH_STORAGE_KEY: "/tmp/",
    MY_IP: {
      "randomized": 1,
      "min_delay": 3,
      "max_delay": 5
    },
    "9.9.9.9": {
      "randomized": 0,
      "min_delay": 30
    }
  }
  HTTP_FREQ_LIMITS_3 = {
    PATH_STORAGE_KEY: "",
    ".*www.mypage.*": {
      MY_IP: {
        "randomized": 0,
        "min_delay": 50
      },
      "3.3.3.3": {
        "randomized": 0,
        "min_delay": 100,
        "max_delay": 200
      }
    },
    "9.9.9.9": {
      "randomized": 0,
      "min_delay": 100,
      "max_delay": 200
    }
  }

  MSG_DEBUG_PATH_TEMP = 'Path of temp: '
  MSG_DEBUG_USE_RANDOMIZED = HostRequestStorage.MSG_DEBUG_USE_RANDOMIZED.split("%s")[0]

  MSG_ERROR_DEFAULT_DELAY = HostRequestStorage.MSG_ERROR_DEFAULT_DELAY.split("%s")[0]


  def setUp(self):
    super(Test, self).setUp()

    self.hostStorage = HostRequestStorage()
    self.path = self.HTTP_FREQ_LIMITS_1[self.PATH_STORAGE_KEY] + Utils.UrlParser.getDomain(self.URL)
    self.fullPath = os.path.join(self.path, self.MY_IP)
    self.temporaryFile = os.path.join(self.fullPath, self.HTTP_FILE_PREFIX + str(self.SITE_ID) + self.FILE_SUFF)
    if os.path.exists(self.path):
      shutil.rmtree(self.path, ignore_errors=True)

  def tearDown(self):
    super(Test, self).tearDown()
    if os.path.exists(self.path):
      shutil.rmtree(self.path, ignore_errors=True)
    if os.path.exists("/tmp/00"):
      shutil.rmtree("/tmp/00", ignore_errors=True)

  def testPositive(self):
    if os.path.exists(self.fullPath):
      shutil.rmtree(self.fullPath, ignore_errors=True)
    self.assertFalse(os.path.exists(self.fullPath))

    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_1), log=self.logger)
    self.assertTrue(res)
    self.assertFalse(os.path.exists(self.fullPath))
    self.assertFalse(os.path.exists(self.temporaryFile))
    self._check_log(self.MSG_DEBUG_PATH_TEMP)

    timeNow = int(time.time())
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)
    self.assertTrue(os.path.exists(self.fullPath))
    self.assertTrue(os.path.exists(self.temporaryFile))
    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertFalse(res)
    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    time.sleep(6)
    timeNow = int(time.time())
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)
    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_3), log=self.logger)
    self.assertFalse(res)
    self._check_log(self.MSG_DEBUG_PATH_TEMP)

    time.sleep(6)
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_3), log=self.logger)
    self.assertFalse(res)
    self._check_log(self.MSG_DEBUG_PATH_TEMP)

    path = "/tmp/00/long_path"
    fullPath = os.path.join(path, Utils.UrlParser.getDomain(self.URL), self.MY_IP)
    filePath = os.path.join(fullPath, self.HTTP_FILE_PREFIX + str(self.SITE_ID) + self.FILE_SUFF)
    HTTP_FREQ_LIMITS = copy.deepcopy(self.HTTP_FREQ_LIMITS_2)
    HTTP_FREQ_LIMITS.update({self.PATH_STORAGE_KEY: path})
    self.assertFalse(os.path.exists(fullPath))
    self._check_log(self.MSG_DEBUG_PATH_TEMP)

    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(HTTP_FREQ_LIMITS), log=self.logger)
    self.assertTrue(res)
    self.assertTrue(os.path.exists(fullPath))
    self.assertTrue(os.path.exists(filePath))
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

  def testWrongTemporaryFile(self):
    os.makedirs(self.fullPath)
    with open(self.temporaryFile, 'w') as fd:
      fd.write('Any text')
    self.assertTrue(os.path.exists(self.temporaryFile))

    timeNow = int(time.time())
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)
    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    with open(self.temporaryFile, 'w') as fd:
      fd.write(json.dumps({"start": 155560}, ensure_ascii=False))
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    with open(self.temporaryFile, 'w') as fd:
      fd.write(json.dumps({self.LAST_START: int(time.time()) + 10000}, ensure_ascii=False))
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)

    with open(self.temporaryFile, 'w') as fd:
      fd.write(json.dumps({self.LAST_START: "Very much time"}, ensure_ascii=False))
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)
    self._check_log(self.MSG_DEBUG_USE_RANDOMIZED)

  def testWrongInputData(self):
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, "Some text", log=self.logger)
    self.assertFalse(res)

    HTTP_FREQ_LIMITS = copy.deepcopy(self.HTTP_FREQ_LIMITS_2)
    HTTP_FREQ_LIMITS[self.MY_IP].pop("min_delay")
    HTTP_FREQ_LIMITS[self.MY_IP].update({"Min_Delaei": 15})
    timeNow = int(time.time())
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(HTTP_FREQ_LIMITS), log=self.logger)
    self.assertTrue(res)
    self._check_log(self.MSG_ERROR_DEFAULT_DELAY, self.ERROR)

    with open(self.temporaryFile, 'r') as fd:
      limitJson = Utils.jsonLoadsSafe(jsonString=fd.read(), default={self.LAST_START: 0}, log=self.logger)
    self.assertGreaterEqual(limitJson[self.LAST_START], timeNow)
    self.assertLessEqual(limitJson[self.LAST_START], timeNow + 5)

    time.sleep(6)
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(HTTP_FREQ_LIMITS), log=self.logger)
    self.assertFalse(res)
    self._check_log(self.MSG_ERROR_DEFAULT_DELAY, self.ERROR)

    # import pdb
    # pdb.set_trace()

    if os.path.exists(self.path):
      shutil.rmtree(self.path, ignore_errors=True)
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertTrue(res)

    HTTP_FREQ_LIMITS = copy.deepcopy(self.HTTP_FREQ_LIMITS_2)
    HTTP_FREQ_LIMITS[self.MY_IP].pop("min_delay")
    HTTP_FREQ_LIMITS[self.MY_IP].update({"Min_DELAY": "5 sec"})
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(HTTP_FREQ_LIMITS), log=self.logger)
    self.assertFalse(res)
    self._check_log(self.MSG_DEBUG_PATH_TEMP)

    time.sleep(5)
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, json.dumps(HTTP_FREQ_LIMITS), log=self.logger)
    self.assertTrue(res)

    res = self.hostStorage.isAvailableHost(None, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertFalse(res)
    res = self.hostStorage.isAvailableHost(1234, self.SITE_ID, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertFalse(res)

    res = self.hostStorage.isAvailableHost(self.URL, None, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertFalse(res)
    res = self.hostStorage.isAvailableHost(self.URL, {"Test": None}, json.dumps(self.HTTP_FREQ_LIMITS_2), log=self.logger)
    self.assertFalse(res)

    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, None, log=self.logger)
    self.assertFalse(res)
    res = self.hostStorage.isAvailableHost(self.URL, self.SITE_ID, "Test", log=self.logger)
    self.assertFalse(res)


if __name__ == "__main__":
  unittest.main()
