#!/usr/bin/python
# coding: utf-8

import os
import sys
import json
import logging
from app.PDateFormat import PDateFormat
from app.DateTimeType import DateTimeType
from app.PDateExtractor import PDateExtractor
import app.Consts as APP_CONSTS


def getLogger():
  # create logger
  logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)
  logger.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add ch to logger
  logger.addHandler(ch)

  return logger


def testExecution(property, url, log, dataString, stage=17):
  log.debug("Input dataString: %s", str(dataString))
  log.debug("Input property: %s", str(property))

  formatList, ifNotDetected, stage = PDateFormat.loadProperty(property, url, log)
  log.debug("formatList: %s, ifNotDetected = %s, stage = %s", str(formatList), str(ifNotDetected), str(stage))

  dateTimeType = DateTimeType(dataString=dataString, formatObj=formatList, continueIfNotDetected=bool(ifNotDetected), log=log)
  log.debug("Extracted: %s", str(dateTimeType.getString()))
  
  dt = PDateExtractor.extract(dataString=dataString, siteProperties={'PDATE_FORMAT':property}, url=url, stage=stage, log=log)
  if dt is not None:
    log.debug("Extracted: %s", str(dt.isoformat(' ')))
  
  log.debug("="*50)


if __name__ == '__main__':

  logger = getLogger()

  # url = 'http://bbc.com/dir1/index.html'
  # url = None
  url = 'http://fortest.local/'
  # dataString = '二月, 2017-18-05 12:05'
  # dataString = 'Posted : 2018-07-18&nbsp;17:20'
  dataString = 'Posted : 2018-07-18 17:20'

  properties = []

#   properties.append("二月, %Y-%d-%m %H:%M")
#   properties.append({"pattern": ".*bbc.com/dir1/.*", "format": "%macro%, %Y-%d-%m %H:%M", "macro": ["一月", "二月"], "if_not_detected":0, "stage":1})
  # properties.append([{"pattern": ".*bbc.com/dir1/.*", "format": "%macro%, %Y-%d-%m %H:%M", "macro": ["一月", "二月"], "if_not_detected":0}])
  # properties.append([{"pattern": ".*fortest.local.*", "format": "%macro%%Y-%m-%d&nbsp;%H:%M", "macro": ["Posted : "], "if_not_detected":0}])
  properties.append([{"pattern": ".*fortest.local.*", "format": "%macro%%Y-%m-%d %H:%M", "macro": ["Posted : "], "if_not_detected":0}])

#   for property in properties:
#     testExecution(property, url, logger, dataString)


# print datetime.datetime.strptime("二月, 2017-18-05 12:05", "二月, %Y-%d-%m %H:%M")
  # dataString = '2018-07-18 16:17:00+09:00'

  dataString = 'Posted : 2018-07-18 17:20'
  format = u'Posted : %Y-%m-%d %H:%M'
  res = DateTimeType.extractByFormatSafe(dataString, format, logger)

  print res
