#!/usr/bin/python
# coding: utf-8

import logging
from app.PDateSourceMask import PDateSourceMask
from app.PDateExtractor import PDateExtractor
import app.Consts as APP_CONSTS


def getLogger():
  # create logger
  log = logging.getLogger(APP_CONSTS.LOGGER_NAME)
  log.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add handler to logger
  log.addHandler(ch)

  return log


def testExecution(properties, url, log, dataString, stage=63):
  log.debug("Input properties: %s", str(properties))

  pDateSourceMask = PDateSourceMask(properties, url, log)
  log.debug("pdateSourceMask = %s, pdateSourceMaskOverwrite = %s, formats = %s",
            str(pDateSourceMask.pdateSourceMask), str(pDateSourceMask.pdateSourceMaskOverwrite), str(pDateSourceMask.formats))
  log.debug("="*50)

  dt = PDateExtractor.extract(dataString=dataString, siteProperties={'PDATE_SOURCES_MASK':properties}, url=url, stage=stage, log=log,
                              stagesFormatList=pDateSourceMask.formats)
  if dt is not None:
    log.debug("Extracted: %s", str(dt.isoformat(' ')))


if __name__ == '__main__':

  logger = getLogger()

  # url = 'http://bbc.com/dir1/index.html'
  url = 'https://www.firstpost.com/politics/lok-sabha-election-2019-live-updates-amarinder-singh-says-punjab-ministers-who-dont-ensure-congress-win-in-their-seats-will-be-dropped-6505791.html'
  dataString = '二月, 2017-18-05 12:05'

  properties = []

#   properties.append(34)
#   properties.append("34")
#   properties.append({"pattern": ".*bbc.com/dir1/.*", "mask": 224})
#   properties.append("{\"pattern\": \".*bbc.com\/dir1\/.*\", \"mask\": 224}")
#   properties.append([{"pattern": ".*bbc.com/dir1/.*", "mask": 224}])
#   properties.append([{"default_mask": 34}])
#   properties.append([{"overwrite_pattern": ".*bbc.com/dir1/.*", "overwrite_mask": 23}])
#   properties.append([{"overwrite_default_mask": 34}])
#   properties.append([{"pattern": ".*bbc.com/dir1/.*", "mask": 224, "overwrite_pattern": ".*bbc.com/dir1/.*", "overwrite_mask": 23, "default_mask": 34, "overwrite_default_mask": 34}])
#   properties.append([{"pattern": ".*bbc.com/dir1/.*", "mask": 224, "formats":[None, None, {"pattern": ".*bbc.com/dir1/.*", "format": "%macro%, %Y-%d-%m %H:%M", "macro": ["一月", "二月"], "if_not_detected":0}]}])
  properties.append([{"pattern":".*koreatimes.co.kr/.*", "mask":32, "default_mask":34}, {"pattern":".*firstpost.com/.*", "mask":32, "default_mask":34}])

  for property in properties:
    testExecution(property, url, logger, dataString)
