#!/usr/bin/python
# coding: utf-8

import os
import json
import base64
import logging
import ConfigParser
from dc.EventObjects import URL
from dc.EventObjects import Batch
from dc.EventObjects import BatchItem
from dc.EventObjects import URLContentResponse
from dc_postprocessor.SocialModule import SocialModule
from dc_postprocessor.SocialModuleCache import SocialModuleCache
from dc_postprocessor.PostProcessingApplicationClass import PostProcessingApplicationClass
from app.CacheDataStorage import CacheDataStorage
import app.Consts as APP_CONSTS
from app.Utils import varDump
import app.Utils as Utils

def getLogger():
  # create logger
  log = logging.getLogger(APP_CONSTS.LOGGER_NAME)
  log.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add ch to logger
  log.addHandler(ch)

  return log


def getFileLogger(configFileName='../ini/postprocessor_task_log-rt.ini'):
  # read config
  logging.config.fileConfig(configFileName)

  # create logger
  log = Utils.MPLogger().getLogger()
  # log = logging.getLogger(APP_CONSTS.LOGGER_NAME)

  return log


if __name__ == '__main__':
  retval = os.getcwd()
  os.chdir('..')

  logger = getLogger()

  # iniDirName = 'ini'
  iniDirName = 'ini_default'

  logConfigFileName = '../' + iniDirName + '/postprocessor_task_log-rt.ini'
  configName = '../' + iniDirName + '/postprocessor_task-rt.ini'
  headerFileName = '../' + iniDirName + '/crawler-task_headers.txt'

  postProcessingApplicationClass = PostProcessingApplicationClass()
  postProcessingApplicationClass.configParser = ConfigParser.RawConfigParser()
  postProcessingApplicationClass.configParser.optionxform = str
  readOk = postProcessingApplicationClass.configParser.read(configName)
  logger.debug("Read config: %s", str(readOk))

  postProcessingApplicationClass.configParser.set('SocialModule', SocialModule.OPTION_EXECUTION_LOCAL, 'cat %INPUT_FILE% > %OUTPUT_FILE%')
  postProcessingApplicationClass.configParser.set('SocialModule', SocialModule.OPTION_EXECUTION_TYPE, SocialModule.EXECUTION_TYPE_LOCAL)
  postProcessingApplicationClass.configParser.set('SocialModule', SocialModule.OPTION_DB_TASK_INI, '../' + iniDirName + '/db-task.ini')
  postProcessingApplicationClass.configParser.set('SocialModule', SocialModule.OPTION_BATCH_ITEMS_COUNT, 3)
  postProcessingApplicationClass.configParser.set('SocialModule', SocialModule.OPTION_USAGE_CACHE, 1)

  postProcessingApplicationClass.configParser.set('SocialModuleCache', SocialModuleCache.CONFIG_OPTION_VALIDATION_MAP, {"tw":["social_tw_posts"]})

  postProcessingApplicationClass.configParser.set('SocialDataCacheDB', CacheDataStorage.DBConnectionOptions.CONFIG_OPTION_DB_HOST, "localhost")

  postProcessingApplicationClass.logger = logger

  siteId = 12345
  url = 'https://www.theguardian.com/us-news/2016/jan/05/obama-gun-control-executive-action-background-checks-licenses-gun-shows-mental-health-funding'
  urlObj = URL(siteId, url)

  processedContent = [{"title":"Tearful Obama tightens gun control and tells inactive Congress: 'We can't wait'",
                      "source_url": url,
                      "social_tw_posts":0,
                      "social_fb_posts":0,
                      "social_tw_likes":55,
                      "social_fb_pw":[["PITTSBURGH", "Pittsburgh", {"Pittsburgh": [["Le'Veon Bell's extended absence affects the <font color=\"red\">Pittsburgh</font> Steelers\u2018 locker room", "http://www.espn.com/blog/pittsburgh-steelers/post/_/id/28569/how-steelers-get-leveon-bell-game-ready-in-five-days#0", {"SENTIMENT_RATE":-7.142857142857142, "pubdate": "2018-09-05 14:53", "title": "PITTSBURGH - Le'Veon Bell's extended absence affects the Pittsburgh Steelers\u2018 locker room, a general manager... \u2026"}], ["Steelers get Le\u2019Veon Bell game-ready in five days \u2013 <font color=\"red\">Pittsburgh</font> Steelers Blog", "http://www.espn.com/blog/pittsburgh-steelers/post/_/id/28569/how-steelers-get-leveon-bell-game-ready-in-five-days#4"]]}, 1, {"SENTIMENT_RATE":-10.0}], ["STEELER", "Steelers", {"Steelers": [["Le'Veon Bell's extended absence affects the Pittsburgh <font color=\"red\">Steelers</font>\u2018 locker room", "http://www.espn.com/blog/pittsburgh-steelers/post/_/id/28569/how-steelers-get-leveon-bell-game-ready-in-five-days#0", {"SENTIMENT_RATE":-7.142857142857142, "pubdate": "2018-09-05 14:53", "title": "PITTSBURGH - Le'Veon Bell's extended absence affects the Pittsburgh Steelers\u2018 locker room, a general manager... \u2026"}], ["How the Pittsurgh <font color=\"red\">Steelers</font> get Le\u2019Veon Bell game-ready", "http://www.espn.com/blog/pittsburgh-steelers/post/_/id/28569/how-steelers-get-leveon-bell-game-ready-in-five-days#4"]]}, 1, {"SENTIMENT_RATE":-10.0}], ["BELL", "Bell", {"Bell": [["PITTSBURGH - Le'Veon <font color=\"red\">Bell</font>'s extended absence affects the Pittsburgh Steelers", "http://www.espn.com/blog/pittsburgh-steelers/post/_/id/28569/how-steelers-get-leveon-bell-game-ready-in-five-days#0", {"SENTIMENT_RATE":-7.142857142857142, "pubdate": "2018-09-05 14:53", "title": "PITTSBURGH - Le'Veon Bell's extended absence affects the Pittsburgh Steelers\u2018 locker room, a general manager... \u2026"}], ["How the Pittsurgh Steelers get Le\u2019Veon <font color=\"red\">Bell</font> game-ready in five days \u2013 Pittsburgh", "http://www.espn.com/blog/pittsburgh-steelers/post/_/id/28569/how-steelers-get-leveon-bell-game-ready-in-five-days#4"]]}, 1, {"SENTIMENT_RATE":-10.0}]]
                      }]

  processedContents = [base64.b64encode(json.dumps(processedContent))]
  urlContentResponse = URLContentResponse(url=url, processedContents=processedContents)
  urlObj.tagsCount = len(processedContent)

  batchItem = BatchItem(siteId=siteId, urlId=urlObj.urlMd5, urlObj=urlObj, urlContentResponse=urlContentResponse)
#   batchItem.properties = {"SOCIAL_RATE":json.dumps({"retries":2, "retries_delay":5, "retries_type":1, "interval":10, "lang":"en", "sentiment":1, "debug":1, "timeout":400, "social_list":{"tw":["https://www.twitter.com", "window.IFRAME_QUERY_URL=\"https://twitter.com/search?f=tweets&vertical=default&q=%25QUERY_STRING%25&src=typd\",window.IFRAME_CSCROLL_COUNT=10;window.IFRAME_MAX_TIME=350;window.IFRAME_SFIELD='source_url';", {"name":"tests", "sets":[{"name":"set1", "items":["1", "%MACRO_DATA%", "http://127.0.0.1/social.js", "!5:76:return window.IFRAME_DATA_READY;", "return window.MACRO_COLLECT;"], "repeat":1, "delay":0}], "result_type":0, "result_content_type":"text/json"}]} })}
#   batchItem.properties = {"SOCIAL_RATE": "{\"retries\":2, \"retries_delay\":10, \"retries_type\":1, \"interval\":10,\"lang\":\"en\", \"sentiment\":1, \"debug\":1, \"timeout\":400, \"social_list\":{\"tw\":[\"https:\/\/www.twitter.com\/\",\"window.IFRAME_QUERY_URL=\\\"https:\/\/twitter.com\/search?f=tweets&vertical=default&q=%25QUERY_STRING%25&src=typd\\\",window.IFRAME_CSCROLL_COUNT=10;window.IFRAME_MAX_TIME=360;window.IFRAME_SFIELD='source_url';\",{\"name\":\"tests\", \"sets\":[{\"name\":\"set1\", \"items\":[\"3\", \"%MACRO_DATA%\", \"http:\/\/127.0.0.1\/social.js\", \"!5:76:return window.IFRAME_DATA_READY;\", \"return window.MACRO_COLLECT;\"], \"repeat\":1, \"delay\":0}], \"result_type\":0, \"result_content_type\":\"text\/json\"}]} }"}
  batchItem.properties = {"SOCIAL_RATE": "{\"retries\":2, \"retries_delay\":5, \"retries_type\":1, \"interval\":10,\"lang\":\"en\", \"sentiment\":1, \"debug\":1, \"timeout\":400, \"social_list\":{\"tw\":[\"https:\/\/www.twitter.com\",\"window.IFRAME_QUERY_URL=\\\"https:\/\/twitter.com\/search?f=tweets&vertical=default&q=%25QUERY_STRING%25&src=typd\\\",window.IFRAME_CSCROLL_COUNT=100;window.IFRAME_MAX_TIME=350;window.IFRAME_SFIELD='source_url';\",{\"name\":\"tests\", \"sets\":[{\"name\":\"set1\", \"items\":[\"1\", \"%MACRO_DATA%\", \"http:\/\/127.0.0.1\/social.js\", \"!5:20:return window.IFRAME_DATA_READY;\", \"return window.MACRO_COLLECT;\"], \"repeat\":1, \"delay\":0}], \"result_type\":0, \"result_content_type\":\"text\/json\"}]} }"}
  batch = Batch(77777, [batchItem])

  logger.debug("Input batch: %s", varDump(batch))
  socialModule = SocialModule(postProcessingApplicationClass.getConfigOption, postProcessingApplicationClass.logger)
  socialModule.init()
  batch = socialModule.processBatch(batch)
  logger.debug("Output batch: %s", varDump(batch))

  # logger.debug("Resolved url: %s", str(json.loads(base64.b64decode(batch.items[0].urlContentResponse.processedContents[0]))['link']))

  os.chdir(retval)


