#!/usr/bin/python

import unittest
import logging
import ConfigParser

from app.Utils import varDump
import app.Consts as APP_CONSTS
from dc_processor.TagLimitsHandler import TagLimitsHandler
from dc_processor.scraper_result import Result
from dc_processor.base_extractor import BaseExtractor
import dc_processor.Constants as PCONSTS


def getLogger():
  # create logger
  log = logging.getLogger(APP_CONSTS.LOGGER_NAME)
  log.setLevel(logging.DEBUG)

  # create console handler and set level to debug
  ch = logging.StreamHandler()
  ch.setLevel(logging.DEBUG)

  # create formatter
  formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

  # add formatter to ch
  ch.setFormatter(formatter)

  # add ch to logger
  log.addHandler(ch)

  return log


def getConfig():
  APP_SECTION_NAME = "Application"
  config = ConfigParser.RawConfigParser()
  config.add_section(APP_SECTION_NAME)
  config.set(APP_SECTION_NAME, "property_file_name", None)
  return config


def getResponseData(response, tagName):
  # variable for result
  ret = None
  if isinstance(response, Result) and isinstance(response.tags, dict):
    for elemTagName, elemTagValue in response.tags.items():
      if elemTagName.lower() == tagName.lower() and isinstance(elemTagValue, dict) and Result.TAGS_DATA_NAME in elemTagValue:
        ret = str(elemTagValue[Result.TAGS_DATA_NAME])

  return ret


def isExistsData(response, tagName):
  return getResponseData(response, tagName) is not None


def initData():
  logger = getLogger()
  tagName = 'description'
  url = 'http://test1.html'
  response = Result(None, 0)
  baseExtractor = BaseExtractor(getConfig())
  baseExtractor.addTag(response, tagName, 'A' * 50, 'xpath_value', callAdjustment=False)
  baseExtractor.addTag(response, tagName + '_', 'B' * 10, 'xpath_value', callAdjustment=False)

  return tagName, url, response, logger


class Test(unittest.TestCase):


  def setUp(self):
    pass


  def tearDown(self):
    pass


  def test1(self):
    tagName, url, response, logger = initData()
    siteProperties = {PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME:{tagName:{".*":{"min_bytes":32, "max_bytes":1000, "default":"qwerty", "truncate":1}}}}
    response = TagLimitsHandler.execute(siteProperties[PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME], url, response, logger)

    self.assertTrue(isExistsData(response, tagName), "isExistsData()")
    self.assertTrue(len(getResponseData(response, tagName)) == 50, "getResponseData()")


  def test2(self):
    tagName, url, response, logger = initData()
    siteProperties = {PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME:{tagName:{".*":{"min_bytes":52, "max_bytes":1000, "default":"", "truncate":1}}}}
    response = TagLimitsHandler.execute(siteProperties[PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME], url, response, logger)

    self.assertTrue(isExistsData(response, tagName), "isExistsData()")
    self.assertTrue(len(getResponseData(response, tagName)) == 0, "getResponseData()")


  def test3(self):
    tagName, url, response, logger = initData()
    siteProperties = {PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME:{tagName:{".*":{"min_bytes":5, "max_bytes":10, "truncate":1}}}}
    response = TagLimitsHandler.execute(siteProperties[PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME], url, response, logger)

    self.assertTrue(isExistsData(response, tagName), "isExistsData()")
    self.assertTrue(len(getResponseData(response, tagName)) == 10, "getResponseData()")


  def test4(self):
    tagName, url, response, logger = initData()
    siteProperties = {PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME:{tagName:{".*":{"min_bytes":5, "max_bytes":10, "truncate":0}}}}
    response = TagLimitsHandler.execute(siteProperties[PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME], url, response, logger)

    self.assertTrue(not isExistsData(response, tagName), "not isExistsData()")
    self.assertTrue(getResponseData(response, tagName) is None, "getResponseData()")


  def test5(self):
    tagName, url, response, logger = initData()
    siteProperties = {PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME:{tagName:{".*":{"min_bytes":5, "max_bytes":10}}}}
    response = TagLimitsHandler.execute(siteProperties[PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME], url, response, logger)

    self.assertTrue(not isExistsData(response, tagName), "not isExistsData()")
    self.assertTrue(getResponseData(response, tagName) is None, "getResponseData()")


  def test6(self):
    tagName, url, response, logger = initData()
    siteProperties = {PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME:{tagName:{"https://.*":{"min_bytes":5, "max_bytes":10}}}}
    response = TagLimitsHandler.execute(siteProperties[PCONSTS.PROCESSED_CONTENTS_TAGS_LIMITS_NAME], url, response, logger)

    self.assertTrue(isExistsData(response, tagName), "isExistsData()")
    self.assertTrue(len(getResponseData(response, tagName)) == 50, "getResponseData()")


if __name__ == '__main__':
  unittest.main()

#   tagName, url, response, logger = initData()
#   siteProperties = {PCONSTS.TAGS_LIMITS_NAME:{tagName:{".*":{"min_bytes":32, "max_bytes":1000, "default":"qwerty", "truncate":1}}}}
#   response = TagLimitsHandler.execute(siteProperties[PCONSTS.TAGS_LIMITS_NAME], url, response, logger)
#
#   logger.debug("response: %s", varDump(response, stringifyType=0))
