#!/usr/bin/python
# coding: utf-8

"""
Created on Aug 30, 2019

@author Evgeniy Arefa <developers.hce@gmail.com>
"""

from base.BaseTest import *

from app.UrlsLogging import UrlsLogging
from app.Filters import Filters

from dc_crawler.CollectURLs import CollectURLs
import dc.EventObjects as dc_event


class Test(BaseTest):

  PROPERTY = {x: {"fields": [], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","} for x in
              ("feed", "to_collect", "to_insert", "skipped", "aged", "freshen", "deleted", "filtered")}

  PROPERTY["feed"].update({"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate", "httpmethod"]})
  PROPERTY["to_collect"].update({"file": "/tmp/urls_%PROJECT_ID%_to_collect.txt", "fields": ["urlput"]})
  PROPERTY["to_insert"].update({"file": "/tmp/urls_%PROJECT_ID%_to_insert.txt", "fields": ["pdate", "httptimeout"]})
  PROPERTY["skipped"].update({"file": "/tmp/urls_%PROJECT_ID%_skipped.txt"})
  PROPERTY["aged"].update({"file": "/tmp/urls_%PROJECT_ID%_aged.txt", "fields": ["requestdelay", "siteid", "siteselect"]})
  PROPERTY["freshen"].update({"file": "/tmp/urls_%PROJECT_ID%_freshen.txt"})
  PROPERTY["deleted"].update({"file": "/tmp/urls_%PROJECT_ID%_deleted.txt"})
  PROPERTY["filtered"].update({
    "file": "/tmp/urls_%PROJECT_ID%_filtered.txt",
    "fields": [CollectURLs.FIELD_IS_PASSED_NAME, CollectURLs.FIELD_ACTION_NAME, CollectURLs.FIELD_STAGE_NAME]
  })

  FIELDS_FILTERED = {
    CollectURLs.FIELD_IS_PASSED_NAME: CollectURLs.FIELD_PASSED,
    CollectURLs.FIELD_ACTION_NAME: CollectURLs.FIELD_ACTION_INCLUDE,
    CollectURLs.FIELD_STAGE_NAME: Filters.STAGES_NAMES[0]
  }

  URL = 'http://test/index.html'
  SITE_ID = '00000TestSiteID00000'
  URL_OBJ = dc_event.URL(SITE_ID, URL)

  MSG_INFO_FIELDS = "fields: {}"

  MSG_ERROR_WRONG_TYPE = "Wrong type (<type '{0}'>) of the"
  MSG_ERROR_FEED = "'feed'"
  MSG_ERROR_EXPECTED = 'expected string or buffer'
  ERROR_MSG_UNSUPPORTED_VALUE = UrlsLogging.ERROR_MSG_UNSUPPORTED_VALUE.split("'")[0]

  TYPE_STR = 'str'
  TYPE_INT = 'int'
  TYPE_DICT = 'dict'
  TYPE_TYPE = 'type'
  TYPE_NONE_TYPE = 'NoneType'


  def setUp(self):
    super(Test, self).setUp()

    for stage in UrlsLogging.STAGES_NAMES:
      self.removeFile(self.PROPERTY, self.SITE_ID, stage)
      self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, stage))

  def tearDown(self):
    super(Test, self).tearDown()

  @staticmethod
  def getFileName(property, siteId, stage):
    # variable for result
    ret = None
    if isinstance(property, dict) and isinstance(siteId, basestring) and isinstance(stage, int) and stage in UrlsLogging.STAGES_NAMES and \
            UrlsLogging.STAGES_NAMES[stage] in property and UrlsLogging.PROPERTY_FILE_NAME in property[UrlsLogging.STAGES_NAMES[stage]]:
      fileName = property[UrlsLogging.STAGES_NAMES[stage]][UrlsLogging.PROPERTY_FILE_NAME]
      ret = re.sub(pattern=UrlsLogging.MACRO_PROJECT_ID, repl=siteId, string=fileName, flags=re.U + re.I)

    return ret

  @staticmethod
  def isExistFile(property, siteId, stage, fields=None, log=None):
    # variable for result
    fields = fields if fields else []
    exist = False
    text = True

    fileName = Test.getFileName(property, siteId, stage)
    if fileName is not None:
      exist = os.path.isfile(fileName)
    if exist:
      with open(fileName, 'r') as fd:
        textFromFile = fd.read()
        for field in fields:
          if str(field) not in textFromFile:
            text = False
            break

    if log is not None:
      log.info("%s is exists %s", str(fileName), str(exist))
      log.info("Fields %s are exist in %s - %s", str(fields), str(fileName), str(text))

    return exist and text

  @staticmethod
  def removeFile(property, siteId, stage, log=None):
    fileName = Test.getFileName(property, siteId, stage)
    if fileName is not None and os.path.isfile(fileName):
      os.remove(fileName)
      if log is not None:
        log.info("Removed %s", str(fileName))

  def testAddUrl(self):
    self.URL_OBJ.pDate = '2011-01-01 12:12:12'
    self.URL_OBJ.httpMethod = 'get'
    self.URL_OBJ.urlPut = 'http://test2/index2.html'
    self.URL_OBJ.httpTimeout = 10000
    self.URL_OBJ.requestDelay = 400
    self.URL_OBJ.siteSelect = 3

    for stage in UrlsLogging.STAGES_NAMES:
      fields = self.FIELDS_FILTERED if UrlsLogging.STAGES_NAMES[stage] == UrlsLogging.PROPERTY_STAGE_FILTERED_NAME else None
      UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=self.SITE_ID, stage=stage, urlObj=self.URL_OBJ, fields=fields, log=self.logger)

      if UrlsLogging.STAGES_NAMES[stage] == UrlsLogging.PROPERTY_STAGE_FEED_NAME:
        fieldsCheck = [self.URL_OBJ.pDate, self.URL_OBJ.httpMethod]
      elif UrlsLogging.STAGES_NAMES[stage] == UrlsLogging.PROPERTY_STAGE_COLLECT_NAME:
        fieldsCheck = [self.URL_OBJ.urlPut]
      elif UrlsLogging.STAGES_NAMES[stage] == UrlsLogging.PROPERTY_STAGE_INSERT_NAME:
        fieldsCheck = [self.URL_OBJ.pDate, self.URL_OBJ.httpTimeout]
      elif UrlsLogging.STAGES_NAMES[stage] == UrlsLogging.PROPERTY_STAGE_AGED_NAME:
        fieldsCheck = [self.URL_OBJ.requestDelay, self.SITE_ID, self.URL_OBJ.siteSelect]
      elif UrlsLogging.STAGES_NAMES[stage] == UrlsLogging.PROPERTY_STAGE_FILTERED_NAME:
        fieldsCheck = [CollectURLs.FIELD_PASSED, CollectURLs.FIELD_ACTION_INCLUDE, Filters.STAGES_NAMES[0]]
      else:
        fieldsCheck = []

      self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, stage, fieldsCheck))

  def testWrongInputData(self):
    # invalid properties
    UrlsLogging.addUrl(properties=None, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_NONE_TYPE), self.ERROR)
    UrlsLogging.addUrl(properties='start', url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_TYPE), self.ERROR)
    UrlsLogging.addUrl(properties={"properties": self.PROPERTY}, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))

    # invalid url
    UrlsLogging.addUrl(properties=self.PROPERTY, url=None, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_NONE_TYPE), self.ERROR)
    UrlsLogging.addUrl(properties=self.PROPERTY, url=1234, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_INT), self.ERROR)
    UrlsLogging.addUrl(properties=self.PROPERTY, url={"url": self.URL}, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_DICT), self.ERROR)

    # invalid siteId
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=None, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_NONE_TYPE), self.ERROR)
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=56789, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_INT), self.ERROR)
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId={"id": self.SITE_ID}, stage=UrlsLogging.STAGE_FEED, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_DICT), self.ERROR)

    # invalid stage
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=self.SITE_ID, stage=None, urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_NONE_TYPE), self.ERROR)
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=self.SITE_ID, stage='feed', urlObj=self.URL_OBJ, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_STR), self.ERROR)
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=self.SITE_ID, stage={"stage": UrlsLogging.STAGE_FEED}, urlObj=self.URL_OBJ)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))

    # invalid urlObj
    UrlsLogging.addUrl(properties=self.PROPERTY, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED, [self.URL]))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

  def testWrongDataInProperty(self):
    self.URL_OBJ.pDate = '2011-01-01 12:12:12'

    property = {"feed": '"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","'}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_WRONG_TYPE.format(self.TYPE_STR), self.ERROR)

    property = {"feeds": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_FEED, self.ERROR)

    property = {"feed": {"files": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl( properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": None, "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_EXPECTED, self.ERROR)

    property = {"feed": {"file": "", "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.ERROR_MSG_UNSUPPORTED_VALUE, self.ERROR)

    property = {"feed": {"file": 123, "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_ERROR_EXPECTED, self.ERROR)

    property = {"feed": {"files": ["/tmp/urls_%PROJECT_ID%_feed.txt"], "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": None, "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": 12345, "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": "pdate", "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED, [self.URL_OBJ.pDate]))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["siteid"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self.assertFalse(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED, [self.URL_OBJ.pDate]))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate"], "timestamp": None, "delimiter": ","}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": None}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED, ','))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiters": ','}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED, ','))
    self.removeFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED)
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)

    property = {"feed": {"file": "/tmp/urls_%PROJECT_ID%_feed.txt", "fields": ["pdate", "httpmethod"], "timestamp": "%Y-%m-%d %H:%M:%S", "delimiter": "____"}}
    UrlsLogging.addUrl(properties=property, url=self.URL, siteId=self.SITE_ID, stage=UrlsLogging.STAGE_FEED, urlObj=None, log=self.logger)
    self.assertTrue(self.isExistFile(self.PROPERTY, self.SITE_ID, UrlsLogging.STAGE_FEED, '____'))
    self._check_log(self.MSG_INFO_FIELDS, self.INFO)


if __name__ == '__main__':
  unittest.main()
