"""
HCE project, Python bindings, Distributed Tasks Manager application.
Json checker main functional.

@package: tools
@file JsonChecker.py
@author scorp, Alexander Vybornyh <alexander.hce.cluster@gmail.com>
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2018 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
"""

import sys
import logging
from cement.core import foundation

import app.Utils as Utils
import app.Consts as APP_CONSTS


# # Class contents main functional of JsonChecker application
class JsonChecker(object):
  ##Constants used in class
  VALUE_SEPARATOR = ','


  # # Internal class for application parameters
  class AppParams(object):
    # default values
    DEFAULT_VALUE_CHECK = False

    # #Initialization
    def __init__(self):
      self.path = None
      self.value = None
      self.file = None
      self.check = self.DEFAULT_VALUE_CHECK
      self.logfile = None


  # # Initialization of class
  def __init__(self):
    self.appParams = self.getAppParams()
    self.logger = self.getLogger(self.appParams.logfile)
    self.exitCode = APP_CONSTS.EXIT_SUCCESS


  # # Get exit code
  #
  # @param - None
  # @return exit code
  def getExitCode(self):
    return self.exitCode
    

  # # get logger instance
  #
  # @param logfile - log file name
  # @return logger instance
  @staticmethod
  def getLogger(logfile):
    # create logger
    logger = logging.getLogger('T')
    logger.setLevel(logging.DEBUG)

    # create console handler and set level to debug
    ch = logging.StreamHandler() if logfile is None else logging.FileHandler(logfile)
    ch.setLevel(logging.DEBUG)

    # create formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # add formatter to ch
    ch.setFormatter(formatter)

    # add ch to logger
    logger.addHandler(ch)

    return logger


# # Get application arguments
  #
  # @param - None
  # @return AppParams instance
  def getAppParams(self):
    app = foundation.CementApp(self.__class__.__name__)
    app.setup()
    app.args.add_argument("-p", "--path", action="store", metavar='path', help='path', required=False)
    app.args.add_argument("-v", "--value", action="store", metavar='value', help='value', required=False)
    app.args.add_argument("-f", "--file", action="store", metavar='file_name', help='file name', required=False)
    app.args.add_argument("-c", "--check", action="store", metavar='check_duplicate', help='check duplicate key', required=False)
    app.args.add_argument("-l", "--logfile", action="store", metavar='log_file', help='log file', required=False)

    app.run()

    if not app.pargs.file:
      raise Exception("Initialization failed. Missing input file name")

    # variable for result
    appParams = self.AppParams()
    appParams.path = app.pargs.path if app.pargs.path else None
    appParams.value = app.pargs.value if app.pargs.value else None
    appParams.file = app.pargs.file if app.pargs.file else None
    appParams.check = bool(int(app.pargs.check)) if app.pargs.check else self.AppParams.DEFAULT_VALUE_CHECK
    appParams.logfile = app.pargs.logfile if app.pargs.logfile else None

    app.close()

    return appParams


  # Method reads json string from file and search value by path
  #
  # @param fileName - file name
  # @param path - path in json
  def readJSONValue(self, fileName, path):
    fd = None
    value = None
    
    with open(fileName, 'r') as fd:
      jsonString = fd.read()
      fd.close()
      try:
        value = Utils.getPath(None, jsonString, path)
        value = str(value)
      except (TypeError, KeyError, IndexError):
        self.logger.error("Value not found. Wrong input path.")
        self.exitCode = APP_CONSTS.EXIT_FAILURE
      except Exception, err:
        self.logger.error(str(err))
        self.exitCode = APP_CONSTS.EXIT_FAILURE

    return value


  #Method compares incoming jsonValue with value, or find jsonValue in value
  #if value is value's list
  #
  # @param jsonValue - json string
  # @param value - value for check
  def valueChecker(self, jsonValue, value):
    found = False
    valueList = value.split(self.VALUE_SEPARATOR)
    for item in valueList:
      if item == jsonValue:
        found = True
        break
    if found == False:
      self.logger.error("Value '%s' not found.", str(value))
      self.exitCode = APP_CONSTS.EXIT_FAILURE


  # Main processing execution
  #
  # @param - None
  # @return - None
  def run(self):

    if self.appParams.path is not None:
      jsonValue = self.readJSONValue(self.appParams.file, self.appParams.path)
      if jsonValue != None and self.appParams.value != None:
        self.valueChecker(jsonValue, self.appParams.value)

    if self.appParams.check:
      with open(self.appParams.file, 'r') as fd:
        if Utils.hasDuplicateKey(fd.read(), self.logger):
          self.exitCode = APP_CONSTS.EXIT_FAILURE
          
