'''
Created on Feb 28, 2014

@author: igor, bgv
'''

# import pickle
try:
  import cPickle as pickle
except ImportError:
  import pickle

import logging
import zmq
from transport.Connection import TransportInternalErr
import transport.Consts as consts
import app.Consts as APP_CONSTS
from app.Utils import varDump


# Logger initialization
logger = logging.getLogger(APP_CONSTS.LOGGER_NAME)

# #Wrapper class on zmq.Socket that operate python objects
#
class ConnectionLight(object):

  POLL_DEFAULT_TIMEOUT = 5000
  MSG_UNHANDLED_UNKNOUN_EXCEPTION = "Unhandled unknown exception"

  def __init__(self, zmq_socket, socket_type, addr="", connected=True):
    '''
    Constructor
    '''
    self.zmq_socket = zmq_socket
    self.socket_type = socket_type
    self.connected = connected
    self.addr = addr

  # #send python object
  #
  # @param obj python object
  # @return None or throw TransportInternalErr
  def send(self, event_obj):
    try:
      logger.debug("event_obj: %s", varDump(event_obj))
      self.connect()
      if self.socket_type == consts.CLIENT_CONNECT:
        self.zmq_socket.send_pyobj(event_obj)
      else:
        pickle_event = pickle.dumps(event_obj)
        self.zmq_socket.send_multipart([event_obj.connect_identity, pickle_event])
    except zmq.ZMQError as err:  # pylint: disable-msg=E1101
      logger.error("zmq.ZMQError: %s", str(err))
      raise TransportInternalErr(str(err))
    except Exception, err:
      logger.error("Error: %s", str(err))
    except:  # pylint: disable=W0702
      try:
        logger.error(self.MSG_UNHANDLED_UNKNOUN_EXCEPTION)
      except Exception as err:
        del err



  # #recieve python object from connection
  #
  # @return python object(event) or throw TransportInternalErr
  def recv(self):
    ret = None

    try:
      self.connect()
      if self.socket_type == consts.CLIENT_CONNECT:
        pyObj = None
        try:
          pyObj = self.zmq_socket.recv_pyobj()
        except zmq.ZMQError, err:  # pylint: disable-msg=E1101
          logger.error("Error on transport: %s", str(err))
          raise TransportInternalErr('ZMQError recv_pyobj()' + str(err))
        except Exception, err:
          logger.error("Error recv_pyobj(): %s", str(err))
        if pyObj is not None:
          logger.debug("Received pyObj: %s", varDump(pyObj))
          ret = pyObj
      else:
        try:
          identity = None
          pickle_event = None
          identity, pickle_event = self.zmq_socket.recv_multipart()
        except zmq.ZMQError, err:  # pylint: disable-msg=E1101
          logger.error("Error on transport: %s", str(err))
          raise TransportInternalErr('ZMQError recv_multipart(): ' + str(err))
        except Exception, err:
          logger.error("Error recv_multipart(): %s", str(err))
        if pickle_event is not None:
          event = None
          try:
            event = pickle.loads(pickle_event)
          except Exception, err:
            logger.error("Error event pickle loads(): %s", str(err))
          if event is not None:
            event.connect_identity = identity
            logger.debug("Received event: %s", varDump(event))
            ret = event
    except zmq.ZMQError, err:  # pylint: disable-msg=E1101
      logger.error("Error on transport: %s", str(err))
      raise TransportInternalErr('ZMQError: ' + str(err))
    except Exception, err:
      logger.error("Error: %s", str(err))
    except:  # pylint: disable=W0702
      try:
        logger.error(self.MSG_UNHANDLED_UNKNOUN_EXCEPTION)
      except Exception as err:
        del err

    return ret



  # #poll the socket for events
  #
  # @param timeout the timeout (in milliseconds) to wait for an event
  # @param flags the event flags to poll for
  # @return The events that are ready and waiting.
  # Will be 0 if no events were ready by the time timeout was reached
  def poll(self, timeout=POLL_DEFAULT_TIMEOUT, flags=zmq.POLLIN):  # pylint: disable-msg=E1101
    ret = None

    try:
      self.connect()
      ret = self.zmq_socket.poll(int(timeout), flags)
    except zmq.ZMQError, err:  # pylint: disable-msg=E1101
      raise TransportInternalErr('ZMQError poll(): ' + str(err))
    except Exception, err:
      logger.error("Error: %s", str(err))
    except:  # pylint: disable=W0702
      try:
        logger.error(self.MSG_UNHANDLED_UNKNOUN_EXCEPTION)
      except Exception as err:
        del err

    return ret



  # close zqm.socket
  #
  # @return None
  def close(self):
    #self.connect()
    if self.connected:
      try:
        self.zmq_socket.close()
      except zmq.ZMQError, err:  # pylint: disable-msg=E1101
        raise TransportInternalErr('ZMQError close(): ' + str(err))
      except Exception, err:
        logger.error("Error: %s", str(err))
      except:  # pylint: disable=W0702
        try:
          logger.error(self.MSG_UNHANDLED_UNKNOUN_EXCEPTION)
        except Exception as err:
          del err
      finally:
        self.connected = False



  # #Really connect to listen socket with addr
  #
  #
  def connect(self):
    if not self.connected:
      try:
        self.zmq_socket.connect(self.addr)
        self.connected = True
      except zmq.ZMQError, err:  # pylint: disable-msg=E1101
        raise TransportInternalErr('ZMQError connect(): ' + str(err))
      except Exception, err:
        logger.error("Error: %s", str(err))
      except:  # pylint: disable=W0702
        try:
          logger.error(self.MSG_UNHANDLED_UNKNOUN_EXCEPTION)
        except Exception as err:
          del err
