#!/bin/bash
#ghk

DIALOG=${DIALOG=dialog}
BACKTITLE='Install Dependencies for HCE project components: hce-node, DTM and DC'
HCE_DEV_FILE='/etc/hce-dev'

RED='\e[0;31m'
BLUE='\e[1;34m'
GREEN='\e[0;32m'
DEF='\e[0m'

HCE_USER=hce
MYSQL_USER=hce
MYSQL_PWD=hce12345
WWW_USER='www-data'

SCRIPT_NAME=$(basename $0)

TEMPFILE=`mktemp /tmp/$SCRIPT_NAME.XXXXX`
TMPDIR=`mktemp -d /tmp/$SCRIPT_NAME.XXXXXX`
trap "rm -f $TEMPFILE; rm -rf $TMPDIR" 0 1 2 5 15

# FUNCTIONS #

function infoDialog() {
    # $1 - title
    # $2 - info messages
    # $3 - height
    if [[ -z $3 ]]
    then
        HEIGHT=5
    else
        HEIGHT=$3
    fi
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --colors --msgbox "$2" $HEIGHT 80
}

function infoBox() {
    # $1 - title
    # $2 - info messages
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
    --infobox "$2" 20 80
}

function yesNo() {
    # $1 - title
    # $2 - info messages
    $DIALOG --backtitle "$BACKTITLE" --title "$1" --defaultno \
            --yesno "$2" 20 80
    case $? in
        0)
            CHECK="YES";;
        1)
            CHECK="NO";;
        255)
            infoDialog "$1" "Cancel. Exit" && clear && exit 1;;
    esac
}

function textDialog() {
    # $1 - title
    # $2 - info messages
    $DIALOG --backtitle "$BACKTITLE" --title "$1" --exit-label "NEXT"\
            --textbox "$2" 20 80
}



function indicatorDialog() {
    # $1 - info messages
    # $2 - info %%
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --gauge "$1" 20 80 "$2"

}

function checkSudo() {
    if [[ $(whoami) != root ]]
    then
        infoDialog "Error" "Run script with root permissions or sudo: sudo $0 \"USERNAME\""
        clear
        exit 1
    fi
}

function checkOs() {
    if [[ ! -f '/etc/debian_version' ]]
    then
        infoDialog "Error" "This script is only for the Debian system! Cancelled."
        clear
        exit 1
    fi
}
function checkDebianVersion() {
    DEBIAN_VERSION_NUM=$(cat /etc/debian_version | awk -F\. '{print $1}')
}

function checkHomeDir() {
    # $1 = hce user
    if [[ ! -d /home/$1 ]]
    then
        infoDialog "Error" "Home dir (/home/${HCE_USER}) for user ${HCE_USER} not exists! Cancelled."
        clear
        exit 1
    fi
}

function checkInstalledPackage() {
    # $1 - package name
    if [[ $(dpkg -s "$1" 2>/dev/null) ]]
    then
        if [[ ! $(dpkg -s "$1" 2>/dev/null | grep deinstall) =~ "deinstall" ]]
        then
            echo 'INSTALLED'
        fi
    else
        exit 1
    fi
}

function start7() {
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --checklist "Check HCE-project dependencies to install" 20 80 9 \
                                                0 "The en_US.UTF-8 locale (if not present)" on \
                                                1 "Install mysql-server" $MYSQL_CHECKBOX \
                                                2 "PHP language API and management tools" on \
                                                3 "DC service" on \
                                                4 "Python language API and management tools" on \
                                                5 "The pyzmq library with zmq version 4.0.3 from source" on \
                                                6 "Copy test site files to the apache virtual host DocumentRoot" on \
                                                7 "The mysql DB support for default site" on \
                                                8 "Dynamic pages fetcher support (only from local repository) " on 2> $TEMPFILE
    case $? in
        0)
            CHECK=$(cat $TEMPFILE);;
        1)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
    esac
}

function start() {
    $DIALOG --backtitle "$BACKTITLE" --title "$1" \
            --checklist "Check HCE-project dependencies to install" 20 80 8 \
                                                0 "The en_US.UTF-8 locale (if not present)" on \
                                                1 "Install mysql-server" $MYSQL_CHECKBOX \
                                                2 "PHP language API and management tools" on \
                                                3 "DC service" on \
                                                4 "Python language API and management tools" on \
                                                5 "Copy test site files to the apache virtual host DocumentRoot" on \
                                                6 "The mysql DB support for default site" on \
                                                7 "Dynamic pages fetcher support " on 2> $TEMPFILE
    case $? in
        0)
            CHECK=$(cat $TEMPFILE);;
        1)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Select components" "Cancel. Exit" && clear && exit 1;;
    esac
}

function mysqlPwd() {
    $DIALOG --backtitle "$BACKTITLE" --title "$TITLE" \
            --insecure \
            --passwordbox "New password for the MySQL \"root\" user:" 20 80 2> $TEMPFILE
    case $? in
        0)
            MYSQLPWD=$(cat $TEMPFILE);;
        1)
            infoDialog "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Cancel. Exit" && clear && exit 1;;
    esac
    $DIALOG --backtitle "$BACKTITLE" --title "$TITLE" \
            --insecure \
            --passwordbox "Repeat password for the MySQL \"root\" user:" 20 80 2> $TEMPFILE
    case $? in
        0)
            MYSQLPWDCONFIRM=$(cat $TEMPFILE);;
        1)
            infoDialog "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Cancel. Exit" && clear && exit 1;;
    esac
    if [[ $MYSQLPWD != $MYSQLPWDCONFIRM ]]
    then
        infoDialog "The two passwords you entered were not the same. Please try again." && clear
        mysqlPwd
    elif [[ ! $MYSQLPWD ]]
    then
        infoDialog "Passwork not be null. Please try again." && clear
        mysqlPwd
    else
        return
    fi
}

function svnPwd(){
    $DIALOG --backtitle "$BACKTITLE" --title "Svn username" \
            --inputbox "Enter SVN username" 20 80 2> $TEMPFILE
    case $? in
        0)
            SVN_USER=$(cat $TEMPFILE);;
        1)
            infoDialog "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Cancel. Exit" && clear && exit 1;;
    esac
    $DIALOG --backtitle "$BACKTITLE" --title "Svn password" \
            --passwordbox "Enter SVN password" 20 80 2> $TEMPFILE
    case $? in
        0)
            SVN_PWD=$(cat $TEMPFILE);;
        1)
            infoDialog "Cancel. Exit" && clear && exit 1;;
        255)
            infoDialog "Cancel. Exit" && clear && exit 1;;
    esac
}

function locales() {
    if [[ -z $(checkLocale) ]]
    then
        echo -e '#\nLANG=en_US.UTF-8\n' >> /etc/default/locale
        sed -i 's/# en_US.UTF-8 UTF-8/en_US.UTF-8 UTF-8/' /etc/locale.gen
        (
            COUNT=99
            echo "XXX"
            echo $COUNT
            echo "generate locales..."
            echo "XXX"
            locale-gen
        ) | indicatorDialog "Added locales" "Generate locales..."
        infoDialog "Added locales" "Locale en_US.UTF-8 added" && clear
    else
        infoDialog "Added locales" "Locale en_US.UTF-8 present" && clear
    fi
}

function mysql_server() {
    if [[ $MYSQL_SERVER ]]
    then
        yesNo "Mysql server installation" "Mysql server installed on you system. Reinstall (not recommended)?" && clear
        if [[ $CHECK = "YES" ]]
        then
            mysqlPwd
            infoBox "Mysql server installation" "Mysql server is installed..."
            apt-get update > $TEMPFILE 2>&1
            debconf-set-selections <<< "mysql-server mysql-server/root_password password $MYSQLPWD"
            debconf-set-selections <<< "mysql-server mysql-server/root_password_again password $MYSQLPWD"
            apt-get -y install mysql-server >> $TEMPFILE 2>&1
            textDialog "Mysql server installation..." "$TEMPFILE" && clear
        fi
    else
        mysqlPwd
        infoBox "Mysql server installation" "Mysql server is installed..."
        apt-get update > $TEMPFILE 2>&1
        debconf-set-selections <<< "mysql-server mysql-server/root_password password $MYSQLPWD"
        debconf-set-selections <<< "mysql-server mysql-server/root_password_again password $MYSQLPWD"
        apt-get -y install mysql-server >> $TEMPFILE 2>&1
        textDialog "Mysql server installation..." "$TEMPFILE" && clear
    fi
}

function dtse_php() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"
        case $DEBIAN_VERSION_NUM in
            10)
            apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 php7.3 php7.3-dev php-pear php-curl php-gd php5-mcrypt pkg-config libpgm-dev sphinxsearch > $TEMPFILE 2>&1;
            phpenmod mcrypt;;
            *)
            apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 php5 php5-dev php-pear php5-curl php5-gd php5-mcrypt pkg-config libpgm-dev sphinxsearch > $TEMPFILE 2>&1;
            php5enmod mcrypt;;
        esac
        COUNT=80
        echo "XXX"
        echo $COUNT
        echo "run install zmq-beta..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pecl list | grep zmq
        if [ "$?" = "1" ]; then
            printf "\n" | sudo pecl install --ignore-errors zmq-beta >> $TEMPFILE 2>&1
        fi
        if [[ -z $(checkZmqSo) ]]
        then
            COUNT=90
            echo "XXX"
            echo $COUNT
            echo "add zmq module to php ini and reload apache"
            echo "XXX"
            case $DEBIAN_VERSION_NUM in
                10)
                sudo echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php/7.3/cli/conf.d/20-zmq.ini;;
                *)
                sudo echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php5/cli/conf.d/20-zmq.ini;;
            esac
        fi
        service apache2 reload
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE 2>&1
    ) | indicatorDialog "Install PHP language API and management tools..." ""
    textDialog "Install PHP language API and management tools..." "$TEMPFILE" && clear
}

function dtse_dc() {
    (
        COUNT=50
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"
        case $DEBIAN_VERSION_NUM in
            10)
            apt-get install -y bc openjdk-8-jdk ruby g++ > $TEMPFILE 2>&1;;
            *)
            apt-get install -y bc openjdk-7-jdk ruby g++ > $TEMPFILE 2>&1;;
        esac
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Install DC service dependencies..." ""
    textDialog "Install DC service dependencies..." "$TEMPFILE" && clear
}

function dtse_python() {
    (
        COUNT=30
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"

        #sudo apt-get install curl
	#sudo apt install python-pip
	sudo curl "https://bootstrap.pypa.io/get-pip.py" -o get-pip.py
	sudo python get-pip.py pip==19.2.1 wheel==0.30.0 setuptools==40.0.0
        #sudo curl "https://raw.githubusercontent.com/codelucas/newspaper/master/download_corpora.py | python2.7"
	sudo pip install --upgrade setuptools
    sudo pip install --upgrade wheel
    sudo pip install --user --upgrade nltk==3.0.1
    #sudo python -m nltk.downloader all
	#pip install pystan
        #python setup.py install
	sudo pip install distribute
    sudo apt-get install -y libpgm-dev python-dev python-flask python-flaskext.wtf libffi-dev libxml2-dev libxslt1-dev mysql-client libmysqlclient-dev python-mysqldb libicu-dev libgmp3-dev libtidy-dev libjpeg-dev > $TEMPFILE 2>&1

        case $DEBIAN_VERSION_NUM in
            *)
                COUNT=80
                echo "XXX"
                echo $COUNT
                echo "run pip install..."
                echo "XXX"
                echo '' >> $TEMPFILE
                apt-get install -y python-cement >> $TEMPFILE 2>&1
                pip install Flask-SQLAlchemy scrapy==0.24.4  gmpy mysql-python pyicu goose-extractor pytidylib uritools python-magic feedparser pillow==2.5.1 beautifulsoup4 w3lib lepl iso639 snowballstemmer soundex langdetect pycountry validators dateutils jsonpath_ng >> $TEMPFILE 2>&1
                pip install --no-cache-dir -I pillow==2.5.1
                COUNT=90
                echo "XXX"
                echo $COUNT
                echo "run pip install pyzmq"
                echo "XXX"
                echo '' >> $TEMPFILE
                pip install pyzmq >> $TEMPFILE 2>&1
                ;;
            8)
                COUNT=80
                echo "XXX"
                echo $COUNT
                echo "run pip install..."
                echo "XXX"
                echo '' >> $TEMPFILE
                easy_install --upgrade pip >> $TEMPFILE 2>&1
                pip install cement Flask-SQLAlchemy scrapyscrapy==0.24.4 gmpy mysql-python pyicu goose-extractor pytidylib uritools python-magic feedparser pillow==2.5.1 beautifulsoup4 w3lib lepl iso639 snowballstemmer soundex langdetect pycountry validators dateutils jsonpath_ng >> $TEMPFILE 2>&1
                pip install --no-cache-dir -I pillow==2.5.1
                COUNT=90
                echo "XXX"
                echo $COUNT
                echo "run pip install pyzmq"
                echo "XXX"
                echo '' >> $TEMPFILE
                pip install pyzmq >> $TEMPFILE 2>&1
                ;;
            7)
                COUNT=80
                echo "XXX"
                echo $COUNT
                echo "run pip install..."
                echo "XXX"
                echo '' >> $TEMPFILE
                pip install cement Flask-SQLAlchemy scrapy==0.24.4 gmpy mysql-python pyicu goose-extractor pytidylib uritools python-magic feedparser pillow==2.5.1 beautifulsoup4 w3lib lepl iso639 snowballstemmer soundex langdetect pycountry validators dateutils jsonpath_ng >> $TEMPFILE 2>&1
                pip install --no-cache-dir -I pillow==2.5.1
                COUNT=90
                echo "XXX"
                echo $COUNT
                echo "run pip install pyzmq (with option zmq=bundle)..."
                echo "XXX"
                echo '' >> $TEMPFILE
                pip install pyzmq --install-option="--zmq=bundled" >> $TEMPFILE 2>&1
                ;;
        esac

        newspaperSSL >> $TEMPFILE 2>&1
        #pip install -U newspaper==0.0.9.8
        pip install -U email dateutils
        pip install -U scrapy==0.24.4
        pip install -U psutil==4.1.0
        pip install -U goose-extractor==1.0.22
	pip uninstall pillow
        pip install -u pillow==2.5.1
	#pip install --user -U nltk==2.0.5 
        #pip install -U requests[security]==2.7
        # comment 28.10.2018 skuridin
        #pip install -U newspaper
        #pip install -U scrapy
        #pip install -U goose-extractor==1.0.22
        pip install -U requests[security]==2.6.0
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "run pip install urlnorm..."
        echo "XXX"
        echo '' >> $TEMPFILE
        pip install -U urlnorm >> $TEMPFILE 2>&1
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE 2>&1
    ) | indicatorDialog "Install Python language API and management tools..." ""
    textDialog "Install Python language API and management tools..." "$TEMPFILE" && clear
}

function pyzmq() {
    (
        COUNT=20
        echo "XXX"
        echo $COUNT
        echo "download zeromq v 4.0.4..."
        echo "XXX"
        cd $TMPDIR
        wget -q https://archive.org/download/zeromq_4.0.4/zeromq-4.0.4.tar.gz
        COUNT=40
        echo "XXX"
        echo $COUNT
        echo "untar zeromq v 4.0.4..."
        echo "XXX"
        tar -xf zeromq-4.0.4.tar.gz > $TEMPFILE 2>&1
        cd zeromq-4.0.4 2>>$TEMPFILE
        COUNT=60
        echo "XXX"
        echo $COUNT
        echo "configure, make and make install zeromq..."
        echo "XXX"
        echo '' >> $TEMPFILE
        ./configure -q 2>>$TEMPFILE >> $TEMPFILE 2>&1 && make -s >> $TEMPFILE 2>&1 && make -s install >> $TEMPFILE 2>&1
        COUNT=90
        echo "XXX"
        echo $COUNT
        echo "install pyzmq with zmq version 4.0.3..."
        echo "XXX"
        echo '' >> $TEMPFILE
        ldconfig >> $TEMPFILE 2>&1
        echo '' >> $TEMPFILE
        pip install -q pyzmq --install-option="--zmq=4.0.3" >> $TEMPFILE 2>&1
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "remove tmp files..."
        echo "XXX"
        cd $TMPDIR
        rm -rf $TMPDIR/*
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
     ) | indicatorDialog "Install pyzmq with zmq v4.0.3 from source..." ""
    textDialog "Install pyzmq with zmq v4.0.3 from source..." "$TEMPFILE" && clear
}

function dpfs(){
    # GET SVN USER #
    svnPwd
    (
        COUNT=20
        echo "XXX"
        echo $COUNT
        echo "run apt-get install..."
        echo "XXX"
        apt-get install -y libexif-dev xvfb libxss1 > $TEMPFILE 2>&1
        COUNT=40
        echo "XXX"
        echo $COUNT
        echo "install google-chrome..."
        echo "XXX"
        checkoutFromSvn "/hce/download/trunk/chrome"
        if [ -f /home/$HCE_USER/chrome/google-chrome-stable_51.0.2704.84_amd64.deb ]; then
            dpkg -i /home/$HCE_USER/chrome/google-chrome-stable_51.0.2704.84_amd64.deb
        else
            echo -e "${RED}google-chrome-stable_51.0.2704.84_amd64.deb do not exists$DEF"
        fi
        apt-get install -y -f
        COUNT=70
        echo "XXX"
        echo $COUNT
        echo "run apt-get update..."
        echo "XXX"
        apt-get update -qq
        COUNT=99
        echo "XXX"
        echo $COUNT
        echo "install the python Selenium package and create link to chromedriver64"
        echo "XXX"
        pip install -q -U selenium
        checkoutFromSvn "/hce/bundle_tools/trunk/chrome_drivers"
        if [ -f /home/$HCE_USER/chrome_drivers/chromedriver64_chrome51 ]; then
            ln -s /home/$HCE_USER/chrome_drivers/chromedriver64_chrome51 /home/$HCE_USER/hce-node-bundle/api/python/bin/chromedriver64
        else
            echo -e "${RED}chromedriver64_chrome51 not exists$DEF"
        fi
        COUNT=100
        echo "XXX"
        echo $COUNT
        echo "Add to ~/.bashrc file"
        echo "XXX"
        echo "export LC_ALL=en_US.UTF-8
export LANG=en_US.UTF-8
export LANGUAGE=en_US.UTF-8
DISPLAY=:1.0
export DISPLAY" >> /home/${HCE_USER}/.bashrc
        echo '' >> $TEMPFILE
        echo 'Done' >> $TEMPFILE
    ) | indicatorDialog "Dynamic pages fetcher support..." ""
    textDialog "Dynamic pages fetcher support..." "$TEMPFILE" && clear
    echo "ATTENTION!!! After end of work script
Run the Xvfb for the resolution 1024x768x16:
Xvfb :1 -screen 0 1024x768x16 &> /tmp/xvfb.log &
or this way for OpenVZ container:
Xvfb +extension RANDR :1 -screen 0 1024x768x16 &> /tmp/xvfb.log &

And apply the configuration for the current user (run from target user session):
source ~/.bashrc" > $TEMPFILE
    textDialog "Dynamic pages fetcher support..." "$TEMPFILE" && clear
}

function mvTestSite() {
    cp -r /home/${HCE_USER}/hce-node-bundle/api/python/data/ftests/test_site/* $WWW_DIR 2>$TEMPFILE
    chown -R $WWW_USER:$WWW_USER $WWW_DIR
    chmod oug+w $WWW_DIR
    usermod -a -G $WWW_USER $HCE_USER
    infoDialog "Set up test web site pages" "Test web site copied and file permissions set\n$(<$TEMPFILE)" 8 && clear
}

function mysqlSetup() {
    if [[ $(pgrep mysql) ]]
    then
        cd /home/${HCE_USER}/hce-node-bundle/api/python/manage
        ./mysql_create_user.sh 2> $TEMPFILE
        sed -i '47d' /etc/mysql/my.cnf
        ./mysql_create_db.sh 2>> $TEMPFILE
        textDialog "Install mysql-server..." "$TEMPFILE"
        infoDialog "Setup mysql (user, permissions, databases)" "Databases successfully created" && clear
    else
        textDialog "Install mysql-server..." "$TEMPFILE"
        infoDialog "Setup mysql (user, permissions, databases)" "Mysql server is not worked. User and databases not created" && clear
    fi
}


function checkZmqSo() {
    if [[ $(php -m | grep zmq) ]]
    then
        echo 'Y'
    else
        exit 1
    fi
}

function checkLocale() {
    if [[ $(locale -a 2>/dev/null | grep 'en_US.utf8') ]]
    then
        echo 'OK'
    fi
    exit 1
}

function newspaperSSL() {
   # wget http://pypi.python.org/packages/source/d/distribute/distribute-0.6.21.tar.gz -O ../../distribute-0.6.21.tar.gz
   # curl -O http://pypi.python.org/packages/source/d/distribute/distribute-0.6.49.tar.gz
    #curl -O http://python-distribute.org/distribute_setup.py
    #python distribute_setup.py
    #tar xzf ../../distribute-0.6.49.tar.gz -C ../../
    #change
    #DEFAULT_URL = "http://pypi.python.org/packages/source/d/distribute/"
    #to
    #DEFAULT_URL = "https://pypi.python.org/packages/source/d/distribute/"
    #sed -i 's|DEFAULT_URL = "http://pypi.python.org/packages/source/d/distribute/"|DEFAULT_URL = "https://pypi.python.org/packages/source/d/distribute/"|' ../../distribute-0.6.21/distribute_setup.py
   #sed -i 's|DEFAULT_URL = "http://pypi.python.org/packages/source/d/distribute/"|DEFAULT_URL = "https://pypi.python.org/packages/source/d/distribute/"|' ../../distribute-0.6.21/distribute_setup.py
    #cat  ../../distribute-0.6.49/requirements.txt
    #sudo pip install distribute
    sudo pip install PyYAML==3.12 beautifulsoup4==4.3.2 cssselect==0.9.1 feedparser==5.1.3 jieba==0.35 lxml==3.3.5 python-dateutil==2.4.0 
six==1.7.3
    #python ../../distribute-0.6.49/distribute_setup.py
    wget -O newspaper-0.1.0.7.tar.gz https://files.pythonhosted.org/packages/d8/07/5765cc9c36e2be1a0f0e615b7a092129e1ba30a25182506dea437290c193/newspaper-0.1.0.7.tar.gz
    #curl -O "https://pypi.python.org/packages/d8/07/5765cc9c36e2be1a0f0e615b7a092129e1ba30a25182506dea437290c193/newspaper-0.1.0.7.tar.gz"
    tar -xf newspaper-0.1.0.7.tar.gz 
    cd newspaper-0.1.0.7
    sudo echo "beautifulsoup4==4.3.2
Pillow==2.5.1
PyYAML==3.12
cssselect==0.9.1
lxml==3.3.5
nltk==3.0.1
requests==2.6.0
six==1.7.3
jieba==0.35
feedparser==5.1.3
tldextract==1.5.1
feedfinder2==0.0.1
python-dateutil==2.4.0
" > requirements.txt
    sudo python setup.py install
    cd ..

    #sudo pip install -U nltk==3.0.1
    #pip install newspaper
}

function checkProcessRunning() {
    process=$1
    iteration_while=0
    iteration_while_max=20
    while : ; do
        echo -e 'Sleep for 10 seconds . . .'
        echo -e '. . .'
        sleep 10
        chcek=$(ps aux | grep $process | grep -v grep)
        iteration_while=$((iteration_while + 1))
        if [ $iteration_while -gt $iteration_while_max ]; then 
            echo "Max iteration $iteration_while_max gained"
            break
        fi
        if [ "$check" = "" ]; then
            echo "Continue installation"
            break
        else
            echo $check
            echo -e '. . .'"$iteration_while"
        fi
    done
}

function checkoutFromSvn() {
    WORKING_DIR_NAME=$(basename $1)
    date
    su - $HCE_USER -c "cd ~/;
    expect -c \"
    spawn svn co svn+ssh://$SVN_USER@192.168.253.1$1 --depth infinity;
    expect {
        \\\"password\\\" {
            send \\\"$SVN_PWD\n\\\"
            exp_continue
        }
        \\\"password\\\" {
            send \\\"$SVN_PWD\n\\\"
            exp_continue
        }
        \\\"svn: E155004: Working copy\\\" {
            send \\\"svn cleanup ~/${WORKING_DIR_NAME}\n\\\"
            expect \\\"$HCE_USER@\\\" {
                send \\\"cd ~/${WORKING_DIR_NAME};svn up\n\\\"
                exp_continue
            }
        }
        full_buffer {
            exp_continue
        }
    }
    \""
    date
}

function force() {
    if [[ $(whoami) != root ]]
    then
        echo "Error" "Run script with root permissions or sudo: sudo $0 \"USERNAME\""
        exit 1
    fi
    if [[ ! -f '/etc/debian_version' ]]
    then
        echo "Error" "This script is only for the Debian system! Cancelled."
        exit 1
    fi
    if [[ ! -d /home/$HCE_USER ]]
    then
        echo "Error" "Home dir (/home/${HCE_USER}) for user ${HCE_USER} not exists! Cancelled."
        exit 1
    fi
    # CHECK DEBIAN VERSION #
    checkDebianVersion
    case $DEBIAN_VERSION_NUM in
        *)
            WWW_DIR='/var/www/html'
            PMA_DIR='/var/www/html/pma';;
        7)
            WWW_DIR='/var/www'
            PMA_DIR='/var/www/pma';;
    esac
    # LOCALES #
    # skuridin
    echo -e '#\nLANG=en_US.UTF-8\n' > /etc/default/locale
    sed -i 's/# en_US.UTF-8 UTF-8/en_US.UTF-8 UTF-8/' /etc/locale.gen
    locale-gen en_US.UTF-8

    checkProcessRunning "apt"
    checkProcessRunning "dpkg"

    # DTSE_PHP #
    apt-get update -qq
    checkProcessRunning "apt"
    checkProcessRunning "dpkg"

    apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 pkg-config libpgm-dev sphinxsearch
    case $DEBIAN_VERSION_NUM in
        10)
        sudo apt-get install -y --force-yes php7.3 php7.3-curl php7.3-gd php7.3-mcrypt php7.3-dev
        phpenmod mcrypt
        sudo pecl install --ignore-errors zmq-beta
        sudo bash -c 'echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php/7.3/cli/conf.d/20-zmq.ini';;
        *)
        sudo apt-get install -y --force-yes bsdutils unzip libzmq3-dev apache2 php5 php5-curl php5-gd php5-mcrypt php5-dev php-pear pkg-config libpgm-dev sphinxsearch;
        php5enmod mcrypt
        apt-get install -y --force-yes php-pear
        pecl list | grep zmq
        if [ "$?" = "1" ]; then
            printf "\n" | pecl install zmq-beta
        fi
    esac


    if [[ -z $(checkZmqSo) ]]
    then
        case $DEBIAN_VERSION_NUM in
            10)
            echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php/7.3/cli/conf.d/20-zmq.ini;;
            *)
            echo -e "; configuration for php ZMQ module \n; priority=20 \nextension=zmq.so" > /etc/php5/cli/conf.d/20-zmq.ini;;
        esac    
    fi
    service apache2 reload

    # DTSE_DC #
    #apt-get update -qq
    case $DEBIAN_VERSION_NUM in
        10)
        apt-get install -y bc openjdk-8-jdk ruby g++ > $TEMPFILE 2>&1;;
        *)
        apt-get install -y bc openjdk-7-jdk ruby g++ > $TEMPFILE 2>&1;;
    esac
    # DTSE_PYTHON #
    #apt-get update -qq
    debconf-set-selections <<< "mysql-server mysql-server/root_password password ''"
    debconf-set-selections <<< "mysql-server mysql-server/root_password_again password ''"

    apt-get install -y mysql-server
    apt-get install -y libpgm-dev python-pip python-dev python-flask python-flaskext.wtf libffi-dev libxml2-dev libxslt1-dev mysql-client python-mysqldb libicu-dev libgmp3-dev libtidy-dev libjpeg-dev

    case $DEBIAN_VERSION_NUM in
        *)
        apt-get install -y default-libmysqlclient-dev python-cement
        ;;
        7|8)
        apt-get install -y libmysqlclient-dev;
        easy_install --upgrade pip
        pip install cement
        ;;
    esac
  
    pip install sqlalchemy Flask-SQLAlchemy scrapy gmpy mysql-python pyicu goose-extractor pytidylib uritools python-magic feedparser pillow==2.5.1 beautifulsoup4 w3lib lepl iso639 snowballstemmer soundex langdetect pycountry psutil==4.1.0 email validators dateutils jsonpath_ng
    newspaperSSL
    pip install --no-cache-dir -I pillow==2.5.1
    case $DEBIAN_VERSION_NUM in
        *)
            pip install pyzmq
            ;;
        7)
            #pip install pyzmq --install-option="--zmq=bundled"
            # PYZMQ #
            cd $TMPDIR
            wget -q https://archive.org/download/zeromq_4.0.4/zeromq-4.0.4.tar.gz
            tar -xf zeromq-4.0.4.tar.gz
            cd zeromq-4.0.4
            ./configure -q && make -s && make -s install
            ldconfig
            pip install -q pyzmq --install-option="--zmq=4.0.3"
            cd $TMPDIR
            rm -rf $TMPDIR/*
            ;;
    esac
    echo -e "${BLUE}install google chrome$DEF";
    checkoutFromSvn "/hce/download/trunk/chrome"
    if [ -f /home/$HCE_USER/chrome/google-chrome-stable_75.0.3770.80_amd64.deb ]; then
        dpkg -i /home/$HCE_USER/chrome/google-chrome-stable_75.0.3770.80_amd64.deb
    else
        echo -e "${RED}Google-chrome-stable_75.0.3770.80_amd64.deb do not exists$DEF"
    fi
    # DYNAMIC PAGE SUPPORT #
    pip install -U urlnorm
    echo -e "${BLUE}Dynamic page support${DEF}"
    apt-get install -y libexif-dev xvfb libxss1
    echo -e "${BLUE}install selenium$DEF";
    pip install -U selenium;
    echo -e "${BLUE}install scrapy$DEF";
    pip install -U scrapy==0.24.4;
    echo -e "${BLUE}install goose-extractor$DEF";
    pip install -U goose-extractor==1.0.22 ## 1.0.25
    echo -e "${BLUE}install requests$DEF";
    pip install -U requests[security]==2.6;
    # install chromedriver
    echo -e "${BLUE}Install chromedriver${DEF}"
    checkoutFromSvn "/hce/bundle_tools/trunk/chrome_drivers"
    if [ -f /home/$HCE_USER/chrome_drivers/chromedriver64_chrome75 ]; then
        if [ ! -L /home/$HCE_USER/hce-node-bundle/api/python/bin/chromedriver64 ]; then
            ln -s /home/$HCE_USER/chrome_drivers/chromedriver64_chrome75 /home/$HCE_USER/hce-node-bundle/api/python/bin/chromedriver64
        fi
    else
        echo -e "${RED}chromedriver64_chrome75 not exists$DEF"
    fi

    grep 'DISPLAY' /home/$HCE_USER/.bashrc;
    if [ "$?" = "1" ]; then
    echo -e "${GREEN}Export bash variables into .bashrc$DEF";
    echo 'export LC_ALL=en_US.UTF-8;
    export LANG=en_US.UTF-8;
    export LANGUAGE=en_US.UTF-8;
    DISPLAY=:1.0;
    export DISPLAY' >> /home/$HCE_USER/.bashrc;
    fi;
    cp -r /home/$HCE_USER/hce-node-bundle/api/python/data/ftests/test_site/* $WWW_DIR;
    chown -R $WWW_USER:$WWW_USER $WWW_DIR;
    ps axu| grep -i xvf[b];
    if [ "$?" = "1" ]; then
        echo -e "${GREEN}Xvfb start$DEF";
        Xvfb +extension RANDR :1 -screen 0 1024x768x16 &> /tmp/xvfb.log &
    else 
        echo -e "${BLUE}Xvfb already started$DEF";
    fi

    # MYSQL SETUP #
    echo -e "${BLUE}MySQL setup${DEF}"
    #set -f
    check_mysql=$(pgrep mysql)
    if [[ $check_mysql ]]; then
        cd /home/${HCE_USER}/hce-node-bundle/api/python/manage
        ./mysql_create_user.sh 2> $TEMPFILE
        echo -e "${BLUE}MySQL create user finish${DEF}"
        sed -i '47d' /etc/mysql/my.cnf
        ./mysql_create_db.sh 2>> $TEMPFILE
        echo -e "${BLUE}Databases successfully created${DEF}"
    else
        echo -e "${BLUE}Mysql server is not worked. User and databases not created{DEF}"
    fi
    exit 0
}

# END OF FUNCTIONS #

# START #

# CHECK HCE-DEVELOPMENTS CONTAINER #
if [[ -f $HCE_DEV_FILE ]]
then
    echo "It's a container for hce-developments, template created at  $(awk -F= '{print $2}' /etc/hce-dev), all dependencies installed"
fi

for ARG in $*
do
    case $ARG in
    'force'|'-force'|'--force')
        force
        exit 0;;
    'username='*|'-username='*|'--username='*)
        HCE_USER=$(echo $ARG | awk -F\= '{print $2}');;
    'help'|'-help'|'--help')
        echo "Usage $0 [username=user] [force] [help]"
        exit 0;;
    *)
        echo "Usage $0 [username=user] [force] [help]"
        exit 0;;
    esac
done

# CHECK DIALOG INSTALL #
if [[ -z $(which $DIALOG) ]]
then
    if [[ -f '/etc/debian_version' ]]
    then
        echo -e $RED"Dialog not installed, please install dialog using ("$BLUE"sudo apt-get install dialog"$RED") and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    elif [[ -f '/etc/redhat-release' || '/etc/centos-release' ]]
    then
        echo -e $RED"Dialog not installed, please install dialog ("$BLUE"sudo yum install dialog"$RED") and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    else
        echo -e $RED"Dialog not installed, please install dialog and run "$BLUE"$0"$RED" again"$DEF
        exit 1
    fi
fi

checkSudo
checkOs
checkHomeDir ${HCE_USER}
checkDebianVersion

MYSQL_SERVER=$(checkInstalledPackage "mysql-server")
if [[ $MYSQL_SERVER ]]
then
    MYSQL_CHECKBOX='off'
else
    MYSQL_CHECKBOX='on'
fi

case $DEBIAN_VERSION_NUM in
    *)
        WWW_DIR='/var/www/html'
        PMA_DIR='/var/www/html/pma'
        start "Select components";;
    7)
        WWW_DIR='/var/www'
        PMA_DIR='/var/www/pma'
        start7 "Select components";;
esac

case $DEBIAN_VERSION_NUM in
    8|10)
        for CHOISE in ${CHECK[@]}
        do
            case $CHOISE in
                0)
                    locales;;
                1)
                    mysql_server;;
                2)
                    dtse_php;;
                3)
                    dtse_dc;;
                4)
                    dtse_python;;
                5)
                    mvTestSite;;
                6)
                    mysqlSetup;;
                7)
                    dpfs;;
            esac
        done
        ;;
    *)
        for CHOISE in ${CHECK[@]}
        do
            case $CHOISE in
                '"0"')
                    locales;;
                '"1"')
                    mysql_server;;
                '"2"')
                    dtse_php;;
                '"3"')
                    dtse_dc;;
                '"4"')
                    dtse_python;;
                '"5"')
                    pyzmq;;
                '"6"')
                    mvTestSite;;
                '"7"')
                    mysqlSetup;;
                '"8"')
                    dpfs;;
            esac
        done
        ;;
esac

infoDialog "Finish" "All selected dependencies installed" && clear
