#!/bin/bash


parseOptions()
{
  args=("$@")
  for opt in "${args[@]}"; do
    if [[ ! "${opt}" =~ .*=.* ]]; then
      if [[ ("$1" = "-help") || ("$1" = "-h") || ("$1" = "--help") || ("$1" = "--h") ]]; then
        export ARG_HELP="1"
        return 0
      else
        echo "badly formatted option \"${opt}\" should be: option=value or [--help | -help | --h | -h] - for help"
        return 1
      fi
    fi
    local var="${opt%%=*}"
    local value="${opt#*=}"
    export ${var}="${value}"
  done
  return 0
}

printUsage()
{
	echo "Usage: $0 inputFile=<input_file_name> execute=<execute_value>"	
	echo ""  
	echo "<input_file_name>  - input file name. Mandatory parameter."
  echo "<execute_value>  - execute value."
	exit 1
}

#######################
## Main process block
#######################
if [ "$1" == "" ]
then
	printUsage
fi

parseOptions "$@"

if [[ "$inputFile" == "" ]]
then
  echo "Missing mandatory parameter."
  printUsage
fi

OUTFILE="/tmp/$(basename $inputFile).log"
#echo "OUTFILE: $OUTFILE"

if [[ "$execute" == "" ]]
then
  if [ -f $OUTFILE ]
  then
    cat $OUTFILE
  fi 

else
  PYTHON_DIR="/home/$USER/hce-node-bundle/api/python"

  TEMPFILE=$(mktemp /tmp/zabbix_contents_counter.XXXXXXXXX)


  cd $PYTHON_DIR/bin/ && python ./dc-client.py --config=../ini/dc-client_zabbix_contents_counter.ini --command=SITE_FIND --file=$inputFile > $TEMPFILE

  cd $PYTHON_DIR/bin/ && python ./zabbix_fields_sum.py --inputFile=$TEMPFILE --logfile=$PYTHON_DIR/log/$(basename $0).log > $OUTFILE

  rm -f $TEMPFILE  
fi

